package com.ejgv.gvi;

import android.content.Context;
import android.content.SharedPreferences;

import com.ejgv.gvi.utils.AES;

import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;

/**
 * Created by Batura Mobile Solutions (Koldo Calvo).
 */
public class AccessData
{
    public static String UUID_APP = null;

    private SharedPreferences mSharedPreferences;
    private static final String SHAREDPREF_ACCESS_DATA = "ACCESS_DATA";
    private static final String SHAREDPREF_KEY_SENT_TOKEN_TO_SERVER = "SENT_TOKEN_TO_SERVER";
    private static final String SHAREDPREF_KEY_GSM = "GSM";
    private static final String SHAREDPREF_KEY_UUID_APP = "UUID_APP";
    private static final String SHAREDPREF_KEY_UUID_DEVICE = "UUID_DEVICE";
    private static final String SHAREDPREF_KEY_LAST_TRANS_ID = "LAST_TRANS_ID";
    private static final String SHAREDPREF_KEY_LAST_TS = "LAST_TS";

    private AES mAES;
    private String AES_ENCRYPTION_KEY = "~Lc$PW_>FxVPRhsnp%bnk_wM/R-{\\{zFo\"-I}2;\\,m+j&Qn_Tk2'Ni*CO}v@2[c";

    /**
     * Constructor
     *
     * @param context
     */
    public AccessData(Context context)
    {
        mSharedPreferences = context.getSharedPreferences(SHAREDPREF_ACCESS_DATA, Context.MODE_PRIVATE);
        mAES = new AES();
        mAES.setKey(AES_ENCRYPTION_KEY);
        if (UUID_APP == null)
            UUID_APP = getUuidApp();
    }

    /**
     * Get app shared preferences
     *
     * @return SharedPreferences app shared preferences object
     */
    SharedPreferences getPreferences()
    {
        return mSharedPreferences;
    }

    /**
     *
     * @return boolean
     */
    boolean isTokenPnsRegistered()
    {
        return mSharedPreferences.getBoolean(SHAREDPREF_KEY_SENT_TOKEN_TO_SERVER, false);
    }

    /**
     *
     * @param bSentToServer
     */
    void setTokenPnsRegistered(boolean bSentToServer)
    {
        SharedPreferences.Editor editor = mSharedPreferences.edit();
        if (bSentToServer)
            editor.putBoolean(SHAREDPREF_KEY_SENT_TOKEN_TO_SERVER, true);
        else
            editor.remove(SHAREDPREF_KEY_LAST_TRANS_ID);
        editor.commit();
    }
    /**
     *
     * @return String
     */
    String getGsm()
    {
        String strGsmEncrypted = mSharedPreferences.getString(SHAREDPREF_KEY_GSM, null);
        if (strGsmEncrypted != null)
            return mAES.decrypt(strGsmEncrypted);
        else
            return null;
    }

    /**
     *
     *
     * @param strGsm registered gsm in the app
     */
    void setGsm(String strGsm)
    {
        SharedPreferences.Editor editor = mSharedPreferences.edit();
        if (strGsm != null && !strGsm.isEmpty())
            editor.putString(SHAREDPREF_KEY_GSM, mAES.encrypt(strGsm));
        else
            editor.remove(SHAREDPREF_KEY_GSM);
        editor.commit();
    }

    /**
     *
     * @return String
     */
    String getUuidApp()
    {
        String strUuidAppEncrypted = mSharedPreferences.getString(SHAREDPREF_KEY_UUID_APP, null);
        if (strUuidAppEncrypted != null)
            return mAES.decrypt(strUuidAppEncrypted);
        else
            return null;
    }

    /**
     *
     *
     * @param strUuidApp the app unic identifier
     */
    void setUuidApp(String strUuidApp)
    {
        SharedPreferences.Editor editor = mSharedPreferences.edit();
        if (strUuidApp != null && !strUuidApp.isEmpty())
            editor.putString(SHAREDPREF_KEY_UUID_APP, mAES.encrypt(strUuidApp));
        else
            editor.remove(SHAREDPREF_KEY_UUID_APP);
        editor.commit();

        UUID_APP = strUuidApp;
    }

    /**
    * @return String uuid device
    */
    String getUuidDevice()
    {
        String strUuidDeviceEncrypted = mSharedPreferences.getString(SHAREDPREF_KEY_UUID_DEVICE, null);
        if (strUuidDeviceEncrypted != null)
            return mAES.decrypt(strUuidDeviceEncrypted);
        else
            return null;
    }

    /**
     *
     * @param strUuidDevice
     */
    void setUuidDevice(String strUuidDevice)
    {
        SharedPreferences.Editor editor = mSharedPreferences.edit();
        if (strUuidDevice != null && !strUuidDevice.isEmpty())
            editor.putString(SHAREDPREF_KEY_UUID_DEVICE, mAES.encrypt(strUuidDevice));
        else
            editor.remove(SHAREDPREF_KEY_UUID_DEVICE);
        editor.commit();
    }

    /**
    *
    * @return String
    */
    String getLastTransId()
    {
        return mSharedPreferences.getString(SHAREDPREF_KEY_LAST_TRANS_ID, null);
    }

    /**
     *
     * @param strLastTransId
     */
    void setLastTransId(String strLastTransId)
    {
        SharedPreferences.Editor editor = mSharedPreferences.edit();
        if (strLastTransId != null && !strLastTransId.isEmpty())
            editor.putString(SHAREDPREF_KEY_LAST_TRANS_ID, strLastTransId);
        else
            editor.remove(SHAREDPREF_KEY_LAST_TRANS_ID);
        editor.commit();
    }

    /**
     *
     * @return long
     */
    long getLastTs()
    {
        return mSharedPreferences.getLong(SHAREDPREF_KEY_LAST_TS, 0);
    }

    /**
     *
     * @param lngLastTs
     */
    void setLastTs(long lngLastTs)
    {
        SharedPreferences.Editor editor = mSharedPreferences.edit();
        if (lngLastTs > 0)
            editor.putLong(SHAREDPREF_KEY_LAST_TS, lngLastTs);
        else
            editor.remove(SHAREDPREF_KEY_LAST_TS);
        editor.commit();
    }

    /**
     * Delete session preferences
     */
    void sessionExpired()
    {
        SharedPreferences.Editor editor = mSharedPreferences.edit();
        editor.remove(SHAREDPREF_KEY_UUID_APP);
        editor.remove(SHAREDPREF_KEY_UUID_DEVICE);
        editor.commit();
    }

    /**
     * Remove all preferences
     */
    void remove()
    {
        mSharedPreferences.edit().clear().commit();
    }

    /**
     *
     * @param cadena
     * @return String hash
     */
    private String sha1Cadena(String cadena)
    {
        String fileHash;
        try {
            MessageDigest sha1 = MessageDigest.getInstance("SHA1");
            sha1.update(cadena.getBytes());
            byte[] hashBytes ;
            hashBytes = sha1.digest();
            StringBuffer sb = new StringBuffer();
            for (int i = 0; i < hashBytes.length; i++) {
                sb.append(Integer.toString((hashBytes[i] & 0xff) + 0x100, 16).substring(1));
            }
            fileHash= sb.toString();
        } catch (NoSuchAlgorithmException e) {
            return cadena;
        }
        return fileHash;
    }

}