package com.ejgv.gvi;

/**
 * Copyright 2015 Google Inc. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import android.app.NotificationManager;
import android.app.PendingIntent;
import android.content.Context;
import android.content.Intent;
import android.media.RingtoneManager;
import android.net.Uri;
import android.os.AsyncTask;
import android.os.Bundle;
import android.support.v4.app.NotificationCompat;
import android.support.v4.content.LocalBroadcastManager;

import com.ejgv.gvi.dao.DAOService;
import com.ejgv.gvi.dao.dto.DTOResponseGetMessages;
import com.ejgv.gvi.restapipns.PNSHttpClient;
import com.ejgv.gvi.restapipns.dto.DTOResponseGetPendingPns;
import com.ejgv.gvi.restapipns.servicedataaccess.MezuPnsPCSService;
import com.ejgv.gvi.utils.UtilsLog;
import com.google.android.gms.gcm.GcmListenerService;

/**
 * Modified by Batura Mobile Solutions (Koldo Calvo).
 */
public class GCMListenerService extends GcmListenerService
                                implements /*DAOService.OnGetPrivateMessageListener,*/
                                            MezuPnsPCSService.OnGetPendingPnsListener,
                                            DAOService.OnAddMessagesListener

{
    private static final String TAG = "GCMListenerService";

    private static final String GCM_KEY_PAGE = "page";
    private static final String GCM_VALUE_PAGE = "W91DPNS";
    public static final String GCM_KEY_MESSAGE_ID = "message_id";
    public static final String GCM_KEY_PAYLOAD = "payload";
    private String mMessage;
    private AsyncTask mGetPendingPnsAsyncTask;
    private AsyncTask mAddMessagesAsyncTask;



    /**
     * Called when message is received.
     *
     * @param from SenderID of the sender.
     * @param data Data bundle containing message data as key/value pairs.
     *             For Set of keys use data.keySet().
     */
    // [START receive_message]
    @Override
    public void onMessageReceived(String from, Bundle data)
    {
        String page = data.getString(GCM_KEY_PAGE);
        mMessage = data.getString(GCM_KEY_PAYLOAD);
        UtilsLog.d(TAG, "From: " + from);
        UtilsLog.d(TAG, "Message: " + mMessage);
        // [START_EXCLUDE]
        /**
         * Production applications would usually process the message here.
         * Eg: - Syncing with server.
         *     - Store message in local database.
         *     - Update UI.
         */
        AccessData accessData = new AccessData(this);
        MezuPnsPCSService mezuPnsPCSService = new MezuPnsPCSService(this);
        UtilsLog.d(TAG, "last trans_id: " + accessData.getLastTransId());
        UtilsLog.d(TAG, "last ts_from: " + accessData.getLastTs());
        mGetPendingPnsAsyncTask = mezuPnsPCSService.createGetPendingPnsCall(this, accessData.getUuidApp(), accessData.getUuidDevice(), accessData.getLastTransId(), accessData.getLastTs());
        mGetPendingPnsAsyncTask.execute();
        // [END_EXCLUDE]
    }
    // [END receive_message]

    /*
        IMPLEMENTS
        MezuPnsPCSService.OnGetPendingPnsListener
     */
    
    @Override
    public void onPostGetPendingPnsAsyncTask(DTOResponseGetPendingPns response)
    {
        UtilsLog.d(TAG, "onPostGetPendingPnsAsyncTask");
        if (response.getResult().equals(PNSHttpClient.RESPONSE_RESULT_OK) && response.getPns() != null && response.getPns().length > 0)
        {
            UtilsLog.d(TAG, "onPostGetPendingPnsAsyncTask mAddMessagesAsyncTask");
            // Save new messages
            PreferencesApp preferencesApp = new PreferencesApp(getApplicationContext());
            DAOService daoService = new DAOService(getApplicationContext());
            mAddMessagesAsyncTask = daoService.createAddMessagesCall(GCMListenerService.this, response.getPns(), preferencesApp.getMessagesMaxLimit());
            mAddMessagesAsyncTask.execute();
        }
    }

    /*
        IMPLEMENTS
        DAOService.OnAddMessagesListener
     */

    @Override
    public void onPostAddMessagesAsyncTask(DTOResponseGetMessages response)
    {
        UtilsLog.d(TAG, "onPostAddMessagesAsyncTask");
        UtilsLog.d(TAG, "Message: " + mMessage);
        sendNotification(mMessage);
        // Notify UI that a notification has been processed
        Intent notificationReceived = new Intent(Constants.GCM_NOTIFICATION_PROCESSED);
        LocalBroadcastManager.getInstance(getApplicationContext()).sendBroadcast(notificationReceived);
    }

    /**
     * Create and show a simple notification containing the received GCM message.
     *
     * @param message GCM message received.
     */
    private void sendNotification(String message)
    {
        UtilsLog.d(TAG, "sendNotification start");
        Intent intent = new Intent(getApplicationContext(), MainActivity.class);
        intent.addFlags(Intent.FLAG_ACTIVITY_SINGLE_TOP);
        int intId = (int) System.currentTimeMillis();
        PendingIntent pendingIntent = PendingIntent.getActivity(getApplicationContext(), intId /* Request code */, intent,
                PendingIntent.FLAG_ONE_SHOT);

        Uri defaultSoundUri = RingtoneManager.getDefaultUri(RingtoneManager.TYPE_NOTIFICATION);
        NotificationCompat.Builder notificationBuilder = new NotificationCompat.Builder(getApplicationContext())
                .setSmallIcon(R.drawable.ic_stat_envelope)
                .setContentTitle(getString(R.string.app_name))
                .setContentText(message)
                .setAutoCancel(true)
                .setSound(defaultSoundUri)
                .setContentIntent(pendingIntent);

        NotificationManager notificationManager =
                (NotificationManager) getSystemService(Context.NOTIFICATION_SERVICE);

        notificationManager.notify(intId /* ID of notification */, notificationBuilder.build());
        UtilsLog.d(TAG, "sendNotification end");
    }
}