package com.ejgv.gvi;

/**
 * Copyright 2015 Google Inc. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import android.app.IntentService;
import android.content.Intent;
import android.os.Bundle;
import android.support.v4.content.LocalBroadcastManager;

import com.ejgv.gvi.restapipns.PNSHttpClient;
import com.ejgv.gvi.restapipns.dto.DTOResponseSimple;
import com.ejgv.gvi.restapipns.servicedataaccess.UpdateTokenPnsHttpClient;
import com.ejgv.gvi.utils.UtilsApp;
import com.ejgv.gvi.utils.UtilsLog;
import com.google.android.gms.gcm.GoogleCloudMessaging;
import com.google.android.gms.iid.InstanceID;
import com.google.gson.Gson;
import com.google.gson.GsonBuilder;

import org.json.JSONObject;

/**
 * Modified by Batura Mobile Solutions (Koldo Calvo).
 */
public class GCMRegistrationIntentService extends IntentService
{
    private static final String TAG = "RegIntentService";

    public GCMRegistrationIntentService()
    {
        super(TAG);
    }

    @Override
    protected void onHandleIntent(Intent intent)
    {
        AccessData accessData = new AccessData(getApplicationContext());
        String token = null;
        try
        {
            // [START register_for_gcm]
            // Initially this call goes out to the network to retrieve the token, subsequent calls
            // are local.
            // R.string.gcm_defaultSenderId (the Sender ID) is typically derived from google-services.json.
            // See https://developers.google.com/cloud-messaging/android/start for details on this file.
            // [START get_token]
            InstanceID instanceID = InstanceID.getInstance(this);
            token = instanceID.getToken(getString(R.string.gcm_defaultSenderId),
                    GoogleCloudMessaging.INSTANCE_ID_SCOPE, null);
            // [END get_token]
            UtilsLog.i(TAG, "GCM Registration Token: " + token);
            // Send pns registration to the app's server.

            if (intent.getBooleanExtra("GCM_TOKEN_REFRESHED", false))
                sendRegistrationUpdateToServer(token, accessData);
            // You should store a boolean that indicates whether the generated token has been
            // sent to your server. If the boolean is false, send the token to your server,
            // otherwise your server should have already received the token.
            accessData.setTokenPnsRegistered(true);
            // [END register_for_gcm]
        }
        catch (Exception e)
        {
            UtilsLog.d(TAG, "Failed to complete token refresh", e);
            UtilsApp.sendTraceToServer(UtilsApp.createStandardAddTraceParams(getClass().getName(), "onHandleIntent", e));
            // If an exception happens while fetching the new token or updating our registration data
            // on a third-party server, this ensures that we'll attempt the update at a later time.
            accessData.setTokenPnsRegistered(false);
        }
        // Notify UI that registration has completed, so the progress indicator can be hidden.
        Intent registrationComplete = new Intent(Constants.GCM_REGISTRATION_COMPLETE);
        Bundle args = new Bundle();
        args.putString(Constants.GCM_TOKEN_PNS, token);
        registrationComplete.putExtras(args);
        LocalBroadcastManager.getInstance(getApplicationContext()).sendBroadcast(registrationComplete);
    }

    /**
     * Persist registration to third-party servers.
     *
     * Modify this method to associate the user's GCM registration token with any server-side account
     * maintained by your application.
     *
     *
     * @param token The new token.
     * @param accessData
     */
    private void sendRegistrationUpdateToServer(String token, AccessData accessData) throws Exception
    {
        // Api call
        UpdateTokenPnsHttpClient updateTokenPnsHttpClient = new UpdateTokenPnsHttpClient(this);
        JSONObject jsonResponse = updateTokenPnsHttpClient.send(
                token,
                accessData.getUuidApp(),
                accessData.getUuidDevice(),
                UtilsApp.getChnProvider(),
                UtilsApp.getAppVersion(this),
                UtilsApp.getDevVendor(),
                UtilsApp.getDevModel(),
                UtilsApp.getOsVendor(),
                UtilsApp.getOsVersion()
        );
        // Parse JSON response to DTO response
        Gson gson = new Gson();
        DTOResponseSimple objResponse = gson.fromJson(jsonResponse.toString(), DTOResponseSimple.class);

        Gson gson_p = new GsonBuilder().setPrettyPrinting().create();
        UtilsLog.d(TAG, gson_p.toJson(objResponse));

        if (objResponse.getResult().equals(PNSHttpClient.RESPONSE_RESULT_ERROR))
            UtilsApp.sendTraceToServer(
                    UtilsApp.createHttpClientAddTraceParams(
                            getClass().getName(),
                            "sendRegistrationToServer",
                            UtilsApp.getTextLocale(getApplicationContext(), objResponse.getDescription())
                    )
            );
    }
}