package com.ejgv.gvi;

import android.animation.ValueAnimator;
import android.app.Activity;
import android.app.SearchManager;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.res.Configuration;
import android.os.Bundle;
import android.support.design.widget.NavigationView;
import android.support.v4.app.Fragment;
import android.support.v4.app.FragmentManager;
import android.support.v4.content.LocalBroadcastManager;
import android.support.v4.view.GravityCompat;
import android.support.v4.widget.DrawerLayout;
import android.support.v7.app.ActionBarDrawerToggle;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.Toolbar;
import android.util.Log;
import android.view.MenuItem;
import android.view.View;
import android.view.animation.DecelerateInterpolator;
import android.widget.TextView;

import com.ejgv.gvi.dao.DAOService;
import com.ejgv.gvi.dao.DDBBHelper;
import com.ejgv.gvi.dmo.DMOMessage;
import com.ejgv.gvi.interfaces.OnDeviceUnregisterListener;
import com.ejgv.gvi.rss.RSSFeedItem;
import com.ejgv.gvi.utils.Utils;
import com.ejgv.gvi.utils.UtilsLog;
import com.google.android.gms.common.ConnectionResult;
import com.google.android.gms.common.GoogleApiAvailability;

/**
 * Created by Batura Mobile Solutions (Koldo Calvo).
 */
public class MainActivity extends AppCompatActivity
                            implements NavigationView.OnNavigationItemSelectedListener,
                                        FragmentManager.OnBackStackChangedListener,
                                        MessageInboxFragment.OnFragmentInteractionListener,
                                        TagsFilterFragment.OnFragmentInteractionListener,
                                        MessagePrivateFragment.OnFragmentInteractionListener,
                                        SettingsFragment.OnFragmentInteractionListener,
                                        OnDeviceUnregisterListener,
                                        NewsFragment.OnRSSFeedListClickListener
{
    private static final String TAG = "MainActivity";

    // UI References
    private DrawerLayout mDrawer;
    private ActionBarDrawerToggle mDrawerToggle;
    private NavigationView mNavigationView;
    private BroadcastReceiver mRegistrationBroadcastReceiver;
    private String mTitle;

    // Variables

    // Interfaces

    /*
        EXTENDS
        AppCompatActivity
     */

    @Override
    protected void onCreate(Bundle savedInstanceState)
    {
        super.onCreate(savedInstanceState);

        setAppLanguage();

        setContentView(R.layout.activity_main);

        AccessData accessData = new AccessData(this);

        if (checkPlayServices())
        {
            mRegistrationBroadcastReceiver = new BroadcastReceiver()
            {
                @Override
                public void onReceive(Context context, Intent intent)
                {
                    AccessData accessData = new AccessData(MainActivity.this);
                    boolean sentToken = accessData.isTokenPnsRegistered();
                    if (sentToken)
                        UtilsLog.d(TAG, "token sent");
                    else
                        UtilsLog.d(TAG, "token error");
                }
            };
            // Start IntentService to register this application with GCM.
            Intent intent = new Intent(this, GCMRegistrationIntentService.class);
            startService(intent);
        }

        getSupportFragmentManager().addOnBackStackChangedListener(this);

        Toolbar toolbar = (Toolbar) findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);
        toolbar.setNavigationOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                if (!mDrawer.isDrawerOpen(GravityCompat.START) && getSupportFragmentManager().getBackStackEntryCount() == 0)
                    mDrawer.openDrawer(GravityCompat.START);
                else
                    onBackPressed();
            }
        });

        mDrawer = (DrawerLayout) findViewById(R.id.drawer_layout);
        mDrawerToggle = new ActionBarDrawerToggle(
                this, mDrawer, R.string.navigation_drawer_open, R.string.navigation_drawer_close);

        mDrawer.setDrawerListener(mDrawerToggle);

        mDrawerToggle.syncState();

        getSupportActionBar().setDisplayShowHomeEnabled(true);
        getSupportActionBar().setDisplayHomeAsUpEnabled(true);

        mNavigationView = (NavigationView) findViewById(R.id.nav_view);
        mNavigationView.setNavigationItemSelectedListener(this);

        // 23.1.0 has a bug related with header inflation. USE 23.1.1
        TextView txtRegisteredNumber = (TextView) mNavigationView.getHeaderView(0).findViewById(R.id.drawerNavHeader_registeredNumber_txt);
        txtRegisteredNumber.setText(accessData.getGsm());

        if (savedInstanceState == null)
        {
            mNavigationView.setCheckedItem(R.id.menuDrawer_messageInbox_item);
            replaceMessageInboxFragment();
        }
    }

    @Override
    protected void onPostCreate(Bundle savedInstanceState)
    {
        super.onPostCreate(savedInstanceState);
        mDrawerToggle.syncState();
    }

    @Override
    public void onConfigurationChanged(Configuration newConfig)
    {
        super.onConfigurationChanged(newConfig);
        mDrawerToggle.onConfigurationChanged(newConfig);
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data)
    {
        switch (requestCode)
        {
            case 0:
                if (resultCode == Activity.RESULT_OK)
                {
                    Fragment fragment = getSupportFragmentManager().findFragmentByTag("MessageInboxFragment");
                    if (fragment != null)
                        ((MessageInboxFragment) fragment).loadMessages();
                }
                break;
            default:
                super.onActivityResult(requestCode, resultCode, data);
        }
    }

    @Override
    public void onBackPressed()
    {
        if (mDrawer.isDrawerOpen(GravityCompat.START))
            mDrawer.closeDrawer(GravityCompat.START);
        else if (getSupportFragmentManager().getBackStackEntryCount() > 0)
            getSupportFragmentManager().popBackStackImmediate();
        else if ( !(getSupportFragmentManager().findFragmentById(R.id.content_lyt) instanceof MessageInboxFragment) )
           replaceMessageInboxFragment();
        else
            super.onBackPressed();
    }

    @Override
    public void onResume()
    {
        super.onResume();

        LocalBroadcastManager.getInstance(this).registerReceiver(mRegistrationBroadcastReceiver,
                new IntentFilter(Constants.GCM_REGISTRATION_COMPLETE));

        startCheckingMsgPendingOperations();

        Fragment fragment = getSupportFragmentManager().findFragmentByTag("MessageInboxFragment");
        if (fragment != null)
            ((MessageInboxFragment) fragment).activityOnResume();
    }

    @Override
    public void onPause()
    {
        LocalBroadcastManager.getInstance(this).unregisterReceiver(mRegistrationBroadcastReceiver);

        super.onPause();
    }

    /*
        PRIVATE METHODS
     */

    private void setAppLanguage()
    {
        PreferencesApp preferencesApp = new PreferencesApp(this);
        String strLanguage = preferencesApp.getUserLanguage();
        if ( strLanguage == null )
        {
            strLanguage = PreferencesApp.SHAREDPREF_VALUE_USER_LANGUAGE_DEFAULT;
            preferencesApp.setUserLanguage(strLanguage);
        }
        // Set app language
        String[] aStrLanguage = strLanguage.split("_");
        Utils.updateLanguage(this, aStrLanguage[0], aStrLanguage[1]);
    }

    private void replaceMessageInboxFragment()
    {
        MessageInboxFragment fragment = new MessageInboxFragment();
        getSupportFragmentManager().beginTransaction().replace(R.id.content_lyt, fragment, MessageInboxFragment.TAG).commit();
        mNavigationView.setCheckedItem(R.id.menuDrawer_messageInbox_item);
    }

    private void replaceSettingsFragment()
    {
        SettingsFragment fragment = new SettingsFragment();
        getSupportFragmentManager().beginTransaction().replace(R.id.content_lyt, fragment).commit();
    }

    private void replaceAboutFragment()
    {
        AboutFragment fragment = new AboutFragment();
        getSupportFragmentManager().beginTransaction().replace(R.id.content_lyt, fragment).commit();
    }

    private void replaceNewsFragment()
    {
        NewsPagerFragment fragment = new NewsPagerFragment();
        getSupportFragmentManager().beginTransaction().replace(R.id.content_lyt, fragment).commit();
    }

    private void replaceContactsFragment()
    {
        ContactsFragment fragment = new ContactsFragment();
        getSupportFragmentManager().beginTransaction().replace(R.id.content_lyt, fragment).commit();
    }

    private void restartApp()
    {
        Intent intent = new Intent(this, PresentationActivity.class);
        intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK|Intent.FLAG_ACTIVITY_CLEAR_TASK|Intent.FLAG_ACTIVITY_CLEAR_TOP);
        overridePendingTransition(0, 0);
        startActivity(intent);
        finish();
    }

    private void startCheckingMsgPendingOperations()
    {
        if (Utils.isConnected(this))
        {
            // Start IntentService to send pending operations to server
            Intent intent = new Intent(this, PendingOperationsIntentService.class);
            startService(intent);
        }
    }

    /**
     * Check the device to make sure it has the Google Play Services APK. If
     * it doesn't, display a dialog that allows users to download the APK from
     * the Google Play Store or enable it in the device's system settings.
     */
    private boolean checkPlayServices()
    {
        GoogleApiAvailability apiAvailability = GoogleApiAvailability.getInstance();
        int resultCode = apiAvailability.isGooglePlayServicesAvailable(this);
        if (resultCode != ConnectionResult.SUCCESS) {
            if (apiAvailability.isUserResolvableError(resultCode)) {
                apiAvailability.getErrorDialog(this, resultCode, Constants.PLAY_SERVICES_RESOLUTION_REQUEST)
                        .show();
            } else {
                Log.i(RegisterActivity.TAG, "This device is not supported.");
                finish();
            }
            return false;
        }
        return true;
    }

    /*
        IMPLEMENTS
        NavigationView.OnNavigationItemSelectedListener
     */

    @Override
    public boolean onNavigationItemSelected(MenuItem item)
    {
        // Handle navigation view item clicks here.
        int id = item.getItemId();

        if (id == R.id.menuDrawer_messageInbox_item)
            replaceMessageInboxFragment();
        else if (id == R.id.menuDrawer_settings_item)
            replaceSettingsFragment();
        else if (id == R.id.menuDrawer_about_item)
            replaceAboutFragment();
        else if (id == R.id.menuDrawer_news_item)
            replaceNewsFragment();
        else if (id == R.id.menuDrawer_contacts_item)
            replaceContactsFragment();

        item.setChecked(true);
        mDrawer.closeDrawer(GravityCompat.START);
        return true;
    }

    /*
        IMPLEMENTS
        FragmentManager.OnBackStackChangedListener
     */

    @Override
    public void onBackStackChanged()
    {
        ValueAnimator animation;

        if (getSupportFragmentManager().getBackStackEntryCount() == 0 )
        {
            animation = ValueAnimator.ofFloat(1, 0);
            mNavigationView.setCheckedItem(R.id.menuDrawer_messageInbox_item);
            mDrawer.setDrawerLockMode(DrawerLayout.LOCK_MODE_UNLOCKED);
            mDrawerToggle.setDrawerIndicatorEnabled(true);
        }
        else
        {
            animation = ValueAnimator.ofFloat(0, 1);
            mDrawer.setDrawerLockMode(DrawerLayout.LOCK_MODE_LOCKED_CLOSED);
        }

        animation.addUpdateListener(new ValueAnimator.AnimatorUpdateListener()
        {
            @Override
            public void onAnimationUpdate(ValueAnimator valueAnimator)
            {
                float slideOffset = (Float) valueAnimator.getAnimatedValue();
                mDrawerToggle.onDrawerSlide(mDrawer, slideOffset);
            }
        });
        animation.setInterpolator(new DecelerateInterpolator());
        animation.setDuration(400);
        animation.start();
    }

    /*
        IMPLEMENTS
        MessageInboxFragment.OnFragmentInteractionListener
     */

    @Override
    public void onMessageSelected(DMOMessage message)
    {
        MessagePrivateFragment messagePrivateFragment = MessagePrivateFragment.newInstance(message);
        getSupportFragmentManager().beginTransaction().replace(R.id.content_lyt, messagePrivateFragment).addToBackStack(null).commit();
    }

    @Override
    public void onActionMode(boolean isActive)
    {
        if (isActive)
            mDrawer.setDrawerLockMode(DrawerLayout.LOCK_MODE_LOCKED_CLOSED);
        else
            mDrawer.setDrawerLockMode(DrawerLayout.LOCK_MODE_UNLOCKED);
    }

    @Override
    public void onTagsActionClicked()
    {
        TagsFilterFragment tagsFilterFragment = new TagsFilterFragment();
        getSupportFragmentManager().beginTransaction().add(R.id.content_lyt, tagsFilterFragment).addToBackStack(null).commit();
        hideTitle();
    }

    private void hideTitle() {
        mTitle = getSupportActionBar().getTitle().toString();
        getSupportActionBar().setTitle("");
    }

    private void showTitle() {
        getSupportActionBar().setTitle(mTitle);
    }

    /*
        IMPLEMENTS
        TagsFilterFragment.OnFragmentInteractionListener,
     */

    @Override
    public void onQueryTextSubmit(String query)
    {
        onBackPressed();
        Intent intent = new Intent(this, SearchResultsActivity.class);
        intent.setAction(Intent.ACTION_SEARCH);
        intent.putExtra(SearchManager.QUERY, query);
        startActivityForResult(intent, 0);
    }

    @Override
    public void onTagFilterDetach() {
        showTitle();
    }

    /*
        IMPLEMENTS
        MessagePrivateFragment.OnFragmentInteractionListener
     */

    @Override
    public void onMarkAsRead(String trans_id, long tsRead)
    {
        Fragment fragment = getSupportFragmentManager().findFragmentByTag("MessageInboxFragment");
        if (fragment != null)
            ((MessageInboxFragment) fragment).messageRead(trans_id, tsRead);
    }

    @Override
    public void onPnsMessageDelete(String trans_id)
    {
        Fragment fragment = getSupportFragmentManager().findFragmentByTag("MessageInboxFragment");
        if (fragment != null)
        {
            ((MessageInboxFragment) fragment).messageDeleted(trans_id);
            onBackPressed();
        }
    }

    /*
        IMPLEMENTS
        SettingsFragment.OnFragmentInteractionListener
     */

    @Override
    public void onSettingLanguageChanged()
    {
        // Language has changed, restart app to update UI
        restartApp();
    }

    @Override
    public void onMessageMaxLimitChanged(int newLimit)
    {
        DAOService daoService = new DAOService(this);
        daoService.createDeleteMessagesOverLimitCall(newLimit).start();
    }

    /*
        IMPLEMENTS
        OnDeviceUnregisterListener
     */

    @Override
    public void onDeviceUnregistered()
    {
        DDBBHelper ddbbHelper = new DDBBHelper(this);
        deleteDatabase(ddbbHelper.getDatabaseName());

        AccessData accessData = new AccessData(this);
        accessData.remove();

        //Device has been unregistered, restart app to update UI
        restartApp();
    }

    /*
        IMPLEMENTS
        NewsFragment.OnRSSFeedListClickListener
     */

    @Override
    public void onRSSFeedSelected(int position, RSSFeedItem item) {

        RSSFeedDetailFragment rssFeedDetailFragment = new RSSFeedDetailFragment();
        Bundle args = new Bundle();
        args.putParcelable(Constants.PARAM_RSS_ITEM, item);
        rssFeedDetailFragment.setArguments(args);
        getSupportFragmentManager().beginTransaction().replace(R.id.content_lyt, rssFeedDetailFragment).addToBackStack(null).commit();
    }
}