package com.ejgv.gvi;

import android.app.NotificationManager;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.IntentFilter;
import android.media.Ringtone;
import android.media.RingtoneManager;
import android.net.Uri;
import android.os.AsyncTask;
import android.os.Bundle;
import android.os.SystemClock;
import android.os.Vibrator;
import android.support.design.widget.Snackbar;
import android.support.v4.app.Fragment;
import android.support.v4.content.LocalBroadcastManager;
import android.support.v4.widget.SwipeRefreshLayout;
import android.support.v7.app.ActionBar;
import android.support.v7.app.AlertDialog;
import android.support.v7.app.AppCompatActivity;
import android.view.ActionMode;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.MotionEvent;
import android.view.View;
import android.view.ViewGroup;

import com.ejgv.gvi.dao.DAOService;
import com.ejgv.gvi.dao.dto.DTOMessage;
import com.ejgv.gvi.dao.dto.DTOResponseGetMessages;
import com.ejgv.gvi.dmo.DMOMessage;
import com.ejgv.gvi.interfaces.OnDeviceUnregisterListener;
import com.ejgv.gvi.restapipns.PNSHttpClient;
import com.ejgv.gvi.restapipns.dto.DTOResponseGetPendingPns;
import com.ejgv.gvi.restapipns.servicedataaccess.MezuPnsPCSService;
import com.ejgv.gvi.ui.EmptyRecyclerView;
import com.ejgv.gvi.utils.Utils;
import com.ejgv.gvi.utils.UtilsApp;

/**
 * Created by Batura Mobile Solutions (Koldo Calvo).
 */

/**
 * A simple {@link Fragment} subclass.
 * Activities that contain this fragment must implement the
 * {@link MessageInboxFragment.OnFragmentInteractionListener} interface
 * to handle interaction events.
 */
public class MessageInboxFragment extends MessageListFragment
                                    implements
                                            SwipeRefreshLayout.OnRefreshListener,
                                            DAOService.OnGetMessagesListener,
                                            MezuPnsPCSService.OnGetPendingPnsListener,
                                            DAOService.OnAddMessagesListener,
                                            DAOService.OnGetMessagesFromListener
{
    public static final String TAG = "MessageInboxFragment";

    // Functionallity variables
    private SwipeRefreshLayout mSwipeRefreshLyt;
    private EmptyRecyclerView mRecyclerView;
    private AsyncTask mGetMessagesAsyncTask;
    private AsyncTask mGetPendingPnsAsyncTask;
    private AsyncTask mGetMessagesFromAsyncTask;
    private AsyncTask mAddMessagesAsyncTask;
    private BroadcastReceiver mNotificationBroadcastReceiver;
    private int mScrollPosition;

    // Interfaces
    private OnDeviceUnregisterListener mDeviceUnregisterListener;
    private OnFragmentInteractionListener mListener;
    /**
     * This interface must be implemented by activities that contain this
     * fragment to allow an interaction in this fragment to be communicated
     * to the activity and potentially other fragments contained in that
     * activity.
     * <p/>
     * See the Android Training lesson <a href=
     * "http://developer.android.com/training/basics/fragments/communicating.html"
     * >Communicating with Other Fragments</a> for more information.
     */
    public interface OnFragmentInteractionListener
    {
        void onMessageSelected(DMOMessage message);
        void onActionMode(boolean isActive);
        void onTagsActionClicked();
    }

    public MessageInboxFragment()
    {
        // Required empty public constructor
    }

    /*
        EXTENDS
        Fragment
     */

    @Override
    public void onAttach(Context context)
    {
        super.onAttach(context);
        try {
            mDeviceUnregisterListener = (OnDeviceUnregisterListener) context;
            mListener = (OnFragmentInteractionListener) context;
        } catch (ClassCastException e) {
            throw new ClassCastException(context.toString()
                    + " must implement MessageInboxFragment.OnFragmentInteractionListener");
        }
    }

    @Override
    public void onCreate(Bundle savedInstanceState)
    {
        super.onCreate(savedInstanceState);
        setHasOptionsMenu(true);

        mNotificationBroadcastReceiver = new BroadcastReceiver()
        {
            @Override
            public void onReceive(Context context, Intent intent)
            {
                NotificationManager notificationManager = (NotificationManager) getContext().getSystemService(Context.NOTIFICATION_SERVICE);
                notificationManager.cancelAll();
                loadNewMessages();
                soundAndVibrate();
            }
        };
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState)
    {
        ActionBar actionBar = ((AppCompatActivity) getActivity()).getSupportActionBar();
        actionBar.setDisplayShowTitleEnabled(true);
        actionBar.setTitle(R.string.title_fragment_message_inbox);

        // Inflate the layout for this fragment
        View v = inflater.inflate(R.layout.fragment_message_inbox, container, false);

        mSwipeRefreshLyt = (SwipeRefreshLayout) v.findViewById(R.id.messageInbox_swipeRefreshLayout);
        mSwipeRefreshLyt.setColorSchemeResources(R.color.gvi_blueLight);
        mSwipeRefreshLyt.setOnRefreshListener(this);

        mRecyclerView = (EmptyRecyclerView) v.findViewById(R.id.messageInbox_recyclerView);
        mRecyclerView.setEmptyView(v.findViewById(R.id.messageInbox_empty_lyt));
        setRecyclerView(mRecyclerView);

        return v;
    }

    @Override
    public void onActivityCreated(Bundle savedInstanceState)
    {
        super.onActivityCreated(savedInstanceState);
        if (mRecyclerView.getAdapter() == null)
            init();
        else
            loadNewMessages();
    }

    @Override
    public void onCreateOptionsMenu(final Menu menu, MenuInflater inflater)
    {
        super.onCreateOptionsMenu(menu, inflater);
        inflater.inflate(R.menu.menu_message_inbox, menu);
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item)
    {
        switch (item.getItemId())
        {
            case R.id.action_filter_messages_message_inbox:
                mListener.onTagsActionClicked();
                return true;
            default:
                return super.onOptionsItemSelected(item);
        }
    }

    @Override
    public void onResume()
    {
        super.onResume();
        LocalBroadcastManager.getInstance(getActivity()).registerReceiver(mNotificationBroadcastReceiver,
                new IntentFilter(Constants.GCM_NOTIFICATION_PROCESSED));
    }

    @Override
    public void onPause()
    {
        LocalBroadcastManager.getInstance(getActivity()).unregisterReceiver(mNotificationBroadcastReceiver);

        mScrollPosition = getLayoutManager().findFirstVisibleItemPosition();

        super.onPause();
    }

    @Override
    public void onDestroy()
    {
        if (mGetMessagesAsyncTask != null)
            mGetMessagesAsyncTask.cancel(true);

        if (mGetPendingPnsAsyncTask != null)
            mGetPendingPnsAsyncTask.cancel(true);

        if (mGetMessagesFromAsyncTask != null)
            mGetMessagesFromAsyncTask.cancel(true);

        super.onDestroy();
    }

    @Override
    public void onDetach()
    {
        super.onDetach();
        mDeviceUnregisterListener = null;
        mListener = null;
        mGetMessagesAsyncTask = null;
        mGetPendingPnsAsyncTask = null;
        mGetMessagesFromAsyncTask = null;
    }

    /*
        EXTENDS
        MessageListFragment
     */

    @Override
    public void onMessageSelected(DMOMessage message)
    {
        mListener.onMessageSelected(message);
    }

    @Override
    protected void onActionModeCreated(ActionMode mode, Menu menu)
    {
        super.onActionModeCreated(mode, menu);
        dispatchTouchEventForCancelPossibleSwipe();
        mSwipeRefreshLyt.setEnabled(false);
        mListener.onActionMode(true);
    }

    @Override
    protected void onActionModeDestroyed(ActionMode mode)
    {
        super.onActionModeDestroyed(mode);
        mSwipeRefreshLyt.setEnabled(true);
        mListener.onActionMode(false);
    }

    /*
        PRIVATE METHODS
     */

    private void init()
    {
        PreferencesApp preferencesApp = new PreferencesApp(getContext());
        DAOService daoService = new DAOService(getContext());
        mGetMessagesAsyncTask = daoService.createGetMessagesAfterDeleteOnesOverLimitCall(this, preferencesApp.getMessagesMaxLimit());
        mGetMessagesAsyncTask.execute();
    }

    private void loadNewMessages()
    {
        // Cancel previous running calls
        if (mGetMessagesFromAsyncTask != null && mGetMessagesFromAsyncTask.getStatus() == AsyncTask.Status.RUNNING)
            mGetMessagesFromAsyncTask.cancel(true);
        AccessData accessData = new AccessData(getContext());
        DAOService daoService = new DAOService(getContext());
        mGetMessagesFromAsyncTask = daoService.createGetMessagesFromCall(this, accessData.getLastTransId(), accessData.getLastTs());
        mGetMessagesFromAsyncTask.execute();
    }

    private void loadPendingPnsMessages()
    {
        // Cancel previous running calls
        if (mGetPendingPnsAsyncTask != null && mGetPendingPnsAsyncTask.getStatus() == AsyncTask.Status.RUNNING)
            mGetPendingPnsAsyncTask.cancel(true);
        AccessData accessData = new AccessData(getContext());
        MezuPnsPCSService mezuPnsPCSService = new MezuPnsPCSService(getContext());
        mGetPendingPnsAsyncTask = mezuPnsPCSService.createGetPendingPnsCall(this, accessData.getUuidApp(), accessData.getUuidDevice(), accessData.getLastTransId(), accessData.getLastTs());
        mGetPendingPnsAsyncTask.execute();
    }

    private void soundAndVibrate()
    {
        Vibrator v = (Vibrator) getContext().getSystemService(Context.VIBRATOR_SERVICE);
        v.vibrate(200);
        try {
            Uri notification = RingtoneManager.getDefaultUri(RingtoneManager.TYPE_NOTIFICATION);
            Ringtone r = RingtoneManager.getRingtone(getActivity(), notification);
            r.play();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void showGetPnsMessagesRetryAction(String message)
    {
        Snackbar mySnackbar = Snackbar.make(getView().findViewById(R.id.messageInbox_coordinatorLayout),
                message, Snackbar.LENGTH_LONG);
        mySnackbar.setAction(
                R.string.action_retry,
                new View.OnClickListener()
                {
                    @Override
                    public void onClick(View v)
                    {
                        loadMessages();
                    }
                }
        );
        mySnackbar.show();
    }

    private void dispatchTouchEventForCancelPossibleSwipe()
    {
        long downTime = SystemClock.uptimeMillis();
        long eventTime = SystemClock.uptimeMillis() + 100;
        float x = 0.0f;
        float y = 0.0f;
        // List of meta states found here:developer.android.com/reference/android/view/KeyEvent.html#getMetaState()
        int metaState = 0;
        // Obtain MotionEvent object
        MotionEvent motionEvent = MotionEvent.obtain(
                downTime,
                eventTime,
                MotionEvent.ACTION_UP,
                x,
                y,
                metaState
        );

        // Dispatch touch event to view
        getActivity().dispatchTouchEvent(motionEvent);
    }

    /*
        PUBLIC METHODS
     */

    public void loadMessages()
    {
        // Cancel previous running calls
        if (mGetMessagesAsyncTask != null && mGetMessagesAsyncTask.getStatus() == AsyncTask.Status.RUNNING)
            mGetMessagesAsyncTask.cancel(true);
        DAOService daoService = new DAOService(getContext());
        mGetMessagesAsyncTask = daoService.createGetMessagesCall(this);
        mGetMessagesAsyncTask.execute();
    }

    public void activityOnResume()
    {
        loadNewMessages();
    }

    /*
        IMPLEMENTS
        SwipeRefreshLayout.OnRefreshListener
     */

    @Override
    public void onRefresh()
    {
        if (Utils.isConnected(getContext()))
            loadPendingPnsMessages();
        else
        {
            showGetPnsMessagesRetryAction(getString(R.string.error_connection_no_connected));
            mSwipeRefreshLyt.setRefreshing(false);
        }
    }

    /*
        IMPLEMENTS
        DAOService.OnGetMessagesListener
     */

    @Override
    public void onPostGetMessagesAsyncTask(DTOResponseGetMessages response)
    {
        if (getContext() == null)
            return;

        if (response.getResult().equals(DAOService.RESPONSE_RESULT_OK) && response.getMessages() != null)
        {
            if (response.getMessages().length > 0)
            {
                // Set adapter's data
                setRecyclerViewAdapterData(response.getMessages());
                getLayoutManager().scrollToPosition(mScrollPosition);
            }
            else if (response.getMessages().length == 0 && Utils.isConnected(getContext()))
            {
                setRecyclerViewAdapterData(new DTOMessage[0]);
                loadPendingPnsMessages();
            }
            else
                showGetPnsMessagesRetryAction(getString(R.string.error_connection_no_connected));
        }
        else
        {
            onRefresh();
        }
    }

    /*
        IMPLEMENTS
        MezuPnsPCSService.OnGetPendingPnsListener
     */

    @Override
    public void onPostGetPendingPnsAsyncTask(DTOResponseGetPendingPns response)
    {
        if (getContext() == null)
            return;

        if (response.getResult().equals(PNSHttpClient.RESPONSE_RESULT_OK))
        {
            if(response.getPns() != null && response.getPns().length > 0)
            {
                PreferencesApp preferencesApp = new PreferencesApp(getContext());
                DAOService daoService = new DAOService(getContext());
                mAddMessagesAsyncTask = daoService.createAddMessagesCall(this, response.getPns(), preferencesApp.getMessagesMaxLimit());
                mAddMessagesAsyncTask.execute();
            }
        }
        else if (UtilsApp.getTextLocale(getContext(), response.getDescription()).equals(getString(R.string.error_httpclient_ioexception)))
        {
            showGetPnsMessagesRetryAction(UtilsApp.getTextLocale(getContext(), response.getDescription()));
        }
        else
        {
            final String errorCode = response.getCode();
            AlertDialog alertDialog = new AlertDialog.Builder(getActivity())
                    .setTitle(getActivity().getString(R.string.title_error_with_code, response.getCode()))
                    .setMessage(UtilsApp.getTextLocale(getActivity(), response.getDescription()))
                    .setCancelable(false)
                    .setPositiveButton(R.string.action_accept, new DialogInterface.OnClickListener()
                    {
                        @Override
                        public void onClick(DialogInterface dialog, int which)
                        {
                            if (errorCode.equals("1000"))
                                mDeviceUnregisterListener.onDeviceUnregistered();
                        }
                    })
                    .create();
            alertDialog.show();
        }
        mSwipeRefreshLyt.setRefreshing(false);
    }

    /*
        IMPLEMENTS
        DAOService.OnAddMessagesListener
     */

    @Override
    public void onPostAddMessagesAsyncTask(DTOResponseGetMessages response)
    {
        if (response.getResult().equals(PNSHttpClient.RESPONSE_RESULT_OK) && getContext() != null)
            addNewMessagesToRecyclerView(response.getMessages());
    }

    /*
        IMPLEMENTS
        DAOService.OnGetMessagesFromListener
     */

    @Override
    public void onPostGetMessagesFromAsyncTask(DTOResponseGetMessages response)
    {
        if (response.getResult().equals(PNSHttpClient.RESPONSE_RESULT_OK) && getContext() != null)
            addNewMessagesToRecyclerView(response.getMessages());
    }
}