package com.ejgv.gvi;

import android.content.Context;
import android.content.DialogInterface;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.os.AsyncTask;
import android.os.Bundle;
import android.os.Vibrator;
import android.support.v4.app.Fragment;
import android.support.v7.app.AlertDialog;
import android.support.v7.widget.DefaultItemAnimator;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.support.v7.widget.helper.ItemTouchHelper;
import android.view.ActionMode;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;

import com.ejgv.gvi.adapters.MessageListAdapter;
import com.ejgv.gvi.dao.DAOService;
import com.ejgv.gvi.dao.dto.DTOMessage;
import com.ejgv.gvi.dmo.DMOMessage;
import com.ejgv.gvi.listeners.MessageInboxItemTouchHelper;
import com.ejgv.gvi.restapipns.dto.DTOResponseSimple;
import com.ejgv.gvi.restapipns.servicedataaccess.MezuPnsPCSService;
import com.ejgv.gvi.ui.EmptyRecyclerView;
import com.ejgv.gvi.utils.UtilsLog;

import java.util.List;

/**
 * Created by Batura Mobile Solutions (Koldo Calvo).
 */
/**
 * A simple {@link Fragment} subclass.
 * Activities that contain this fragment must implement the
 */
public abstract class MessageListFragment extends Fragment
                                    implements
                                            MessageListAdapter.MessageListAdapterListener,
                                            MessageInboxItemTouchHelper.OnRecyclerItemTouchHelperListener,
                                            MezuPnsPCSService.OnDeleteListener
{
    private static final String TAG = "MessageListFragment";

    // Functionality Variables

    private MessageListAdapter mAdapter;
    private ActionMode mActionMode;
    private int mPositionToDelete;
    private AsyncTask mDeleteAsyncTask;
    private boolean mDeleting;

    // UI References
    private LinearLayoutManager mLayoutManager;
    private RecyclerView mRecyclerview;
    private AlertDialog mDeleteAlertDialog;

    public MessageListFragment()
    {
        // Required empty public constructor
    }

    public LinearLayoutManager getLayoutManager()
    {
        return mLayoutManager;
    }

    /*
        EXTENDS
        Fragment
     */

    @Override
    public void onCreate(Bundle savedInstanceState)
    {
        super.onCreate(savedInstanceState);
        setHasOptionsMenu(true);
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState)
    {
        // Inflate the layout for this fragment
        View v = inflater.inflate(R.layout.fragment_message_inbox, container, false);

        EmptyRecyclerView recyclerView = (EmptyRecyclerView) v.findViewById(R.id.messageInbox_recyclerView);
        setRecyclerView(recyclerView);

        return v;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item)
    {
        switch (item.getItemId())
        {
            case R.id.action_delete_messages:
                if (mActionMode != null)
                    return false;
                startContextActionBar();
                return true;
            default:
                return super.onOptionsItemSelected(item);
        }
    }


    @Override
    public void onDestroy()
    {
        if (mDeleteAsyncTask != null )
            mDeleteAsyncTask.cancel(false);
        super.onDestroy();
    }

    @Override
    public void onDetach()
    {
        super.onDetach();
        mActionMode = null;
    }

    /*
        PRIVATE METHODS
     */

    private void startContextActionBar()
    {
        ActionMode.Callback actionModeCallback = new ActionMode.Callback()
        {
            // Called when the action mode is created; startActionMode() was called
            @Override
            public boolean onCreateActionMode(ActionMode mode, Menu menu)
            {
                onActionModeCreated(mode, menu);
                // Inflate a menu resource providing context menu items
                MenuInflater inflater = mode.getMenuInflater();
                inflater.inflate(R.menu.menu_message_list_actionmode, menu);
                return true;
            }

            // Called each time the action mode is shown. Always called after onCreateActionMode, but
            // may be called multiple times if the mode is invalidated.
            @Override
            public boolean onPrepareActionMode(ActionMode mode, Menu menu)
            {
                String strTitle = getString(
                        R.string.title_selected_multiple_fragment_message_inbox,
                        0);
                mode.setTitle(strTitle);
                if (mAdapter != null)
                    mAdapter.startSelection();
                return false; // Return false if nothing is done
            }

            // Called when the user selects a contextual menu item
            @Override
            public boolean onActionItemClicked(ActionMode mode, MenuItem item)
            {
                switch (item.getItemId())
                {
                    case R.id.action_delete_messages:
                        if (mAdapter.getSelectedItemCount() > 0)
                            deleteSelectedItems();
                        else
                            mode.finish();
                        return true;
                    default:
                        return false;
                }
            }

            // Called when the user exits the action mode
            @Override
            public void onDestroyActionMode(ActionMode mode)
            {
                mActionMode = null;
                mAdapter.clearSelections();
                onActionModeDestroyed(mode);
            }
        };

        mActionMode = getActivity().startActionMode(actionModeCallback);
    }

    private void updateActionBarItemSelectionCounter(int selectedItemCount)
    {
        if (mActionMode != null)
        {
            String strTitle;
            if (selectedItemCount == 1)
                strTitle = getString(
                        R.string.title_selected_one_fragment_message_inbox,
                        selectedItemCount);
            else
                strTitle = getString(
                        R.string.title_selected_multiple_fragment_message_inbox,
                        selectedItemCount);
            mActionMode.setTitle(strTitle);

            if (mAdapter.getItemCount() == 0 || mAdapter.getSelectedItemCount() == 0)
                mActionMode.finish();
        }
    }

    private void deleteSelectedItems()
    {
        AlertDialog.Builder builder = new AlertDialog.Builder(getActivity());
        builder.setTitle(R.string.action_delete_messages)
                .setMessage(R.string.confirm_delete_messages)
                .setCancelable(false)
                .setPositiveButton(R.string.action_yes, new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(DialogInterface dialog, int which) {
                        mDeleting = true;
                        List<Integer> selectedItemPositions = mAdapter.getSelectedItems();
                        int intLength = selectedItemPositions.size();
                        String[] astrTransIdsToDelete = new String[intLength];
                        int cont = 0;
                        for (int i=intLength - 1; i >= 0; i--) {
                            final int position = selectedItemPositions.get(i);
                            astrTransIdsToDelete[cont] = mAdapter.getItem(position).getIdTrans();
                            cont++;
                            UtilsLog.d(TAG, "position: " + position);
                            mAdapter.removeAt(position);
                        }
                        deleteMessagesFromDatabase(astrTransIdsToDelete);
                    }
                })
                .setNegativeButton(R.string.action_no, null);

        builder.create().show();
    }

    /**
     * Delete mesasge from adapter and database
     * @param position
     */
    private void deleteMessage(int position)
    {
        deleteMessagesFromDatabase(new String[]{deleteMessageFromAdapter(position)});;
    }

    /**
     * Delete message from adapter and return deleted trans id
     * @param position
     * @return
     */
    private String deleteMessageFromAdapter(int position)
    {
        String trans_id = null;
        if (mAdapter.getItem(position) != null)
            trans_id = mAdapter.getItem(position).getIdTrans();
        mAdapter.removeAt(position);
        return trans_id;
    }

    private void deleteMessagesFromDatabase(String[] trans_ids)
    {
        updateActionBarItemSelectionCounter((mAdapter != null) ? mAdapter.getSelectedItemCount() : 0);

        DAOService daoService = new DAOService(getContext());
        daoService.createDeleteMessagesCall(trans_ids).start();
        AccessData accessData = new AccessData(getContext());
        MezuPnsPCSService mezuPnsPCSService = new MezuPnsPCSService(getContext());
        mDeleteAsyncTask = mezuPnsPCSService.createDeleteCall(this, accessData.getUuidApp(), accessData.getUuidDevice(), trans_ids);
        mDeleteAsyncTask.execute();
        onActionDeleted(mAdapter.getItemCount());
    }

    private void scrollToNewPosition()
    {
        int pos = mLayoutManager.findFirstVisibleItemPosition();
        if (pos > 0)
            pos--;

        mLayoutManager.scrollToPosition(pos);
    }

    private void setLastPersistedMessage(DTOMessage message)
    {
        AccessData accessData = new AccessData(getContext());
        accessData.setLastTransId(message.getIdTrans());
        accessData.setLastTs(message.getTs());
    }

    /*
        PROTECTED METHODS
     */

    protected void setRecyclerView(RecyclerView recyclerView)
    {
        mRecyclerview = recyclerView;
        // use this setting to improve performance if you know that changes
        // in content do not change the layout size of the RecyclerView
        mRecyclerview.setHasFixedSize(true);
        // use a linear layout manager
        mLayoutManager = new LinearLayoutManager(getActivity());
        mRecyclerview.setLayoutManager(mLayoutManager);

        // Set item draw decoration
        RecyclerView.ItemDecoration itemDecoration = new RecyclerView.ItemDecoration()
        {
            @Override
            public void onDraw(Canvas c, RecyclerView parent, RecyclerView.State state)
            {
                super.onDraw(c, parent, state);
                if (parent.getItemAnimator().isRunning() && mDeleting)
                {
                    int childCount = parent.getChildCount();
                    for (int i = 0; i < childCount; i++)
                    {
                        c.save();
                        final View child = parent.getChildAt(i);
                        Paint paint = new Paint();
                        paint.setColor(Color.RED);
                        c.drawRect(0, child.getTop(), child.getRight(), child.getBottom(), paint);
                        c.restore();
                    }
                }
                else if (!parent.getItemAnimator().isRunning() && mDeleting)
                    mDeleting = false;
            }
        };
        mRecyclerview.addItemDecoration(itemDecoration);

        mRecyclerview.setItemAnimator(new DefaultItemAnimator());
        mRecyclerview.getItemAnimator().setRemoveDuration(300);

        // Set item swipe behaviour
        ItemTouchHelper itemTouchHelper = new ItemTouchHelper(new MessageInboxItemTouchHelper(this));
        itemTouchHelper.attachToRecyclerView(mRecyclerview);

        if (mAdapter != null)
            mRecyclerview.setAdapter(mAdapter);
    }

    protected void setRecyclerViewAdapterData(DTOMessage[] messages)
    {
        if (messages != null)
        {
            /*PreferencesApp preferencesApp = new PreferencesApp(getContext());
            // Check preferences' messages limit
            if (messages.length > preferencesApp.getMessagesMaxLimit())
                messages = Arrays.copyOf(messages, preferencesApp.getMessagesMaxLimit());*/
            // Set adapter's data
            MessageListAdapter adapter = new MessageListAdapter(getContext(), messages, this);
            mAdapter = adapter;
            mRecyclerview.setAdapter(mAdapter);
        }
        else
        {
            mAdapter = null;
            mRecyclerview.setAdapter(mAdapter);
        }
    }

    protected void addNewMessagesToRecyclerView(DTOMessage[] messages)
    {
        /*PreferencesApp preferencesApp = new PreferencesApp(getContext());*/
        if (mAdapter == null || mAdapter.getItemCount() == 0)
        {
            // Add all items
            setRecyclerViewAdapterData(messages);
        }
        else if (messages != null && messages.length > 0)
        {
            // Add new messages
            for (int i=messages.length-1; i>=0; i--)
            {
                if (messages[i].getTs() > mAdapter.getItem(0).getTs()) {
                    mAdapter.add(0, messages[i]);
                    scrollToNewPosition();
                }
            }
            // Set last received messsage
            setLastPersistedMessage(mAdapter.getItem(0));

            // Delete old messagesif if it is necessary
            /*if (mAdapter.getItemCount() > preferencesApp.getMessagesMaxLimit())
                mAdapter.removeRange(preferencesApp.getMessagesMaxLimit(), mAdapter.getItemCount());*/
        }
    }

    protected void onActionModeCreated(ActionMode mode, Menu menu){};

    protected void onActionModeDestroyed(ActionMode mode){};

    protected void onActionDeleted(int newItemCount){};

    protected abstract void onMessageSelected(DMOMessage message);

    /*
        PUBLIC METHODS
     */

    public void messageRead(String trans_id, long tsRead)
    {
        mAdapter.updateItemTsRead(trans_id, tsRead);
    }

    public void messageDeleted(String trans_id)
    {
        mAdapter.remove(trans_id);
    }

    /*
        IMPLEMENTS
        MessageListAdapter.MessageListAdapterListener
     */

    @Override
    public void onMessageListAdapterItemClicked(View view, int position)
    {
        if (mActionMode == null)
        {
            DMOMessage message = new DMOMessage(mAdapter.getItem(position));
            onMessageSelected(message);
        }
        else
        {
            mAdapter.toggleSelection(position);
            updateActionBarItemSelectionCounter(mAdapter.getSelectedItemCount());
        }
    }

    @Override
    public void onMessageListAdapterItemLongClicked(View view, int position)
    {
        Vibrator v = (Vibrator) getContext().getSystemService(Context.VIBRATOR_SERVICE);
        v.vibrate(100);

        if (mActionMode == null)
            startContextActionBar();

        mAdapter.toggleSelection(position);
        updateActionBarItemSelectionCounter(mAdapter.getSelectedItemCount());
    }

     /*
        IMPLEMENTS
        MessageInboxItemTouchHelper.OnRecyclerItemTouchHelperListener
     */

    @Override
    public void onMessageInboxItemSwiped(int position)
    {
        UtilsLog.d(TAG, "mPositionToDelete 1: " + mPositionToDelete);
        if (mPositionToDelete > 0 && mDeleteAlertDialog != null)
        {
            mDeleting = true;
            mDeleteAlertDialog.dismiss();
            deleteMessage(mPositionToDelete - 1);
            UtilsLog.d(TAG, "mPositionToDelete 2: "+ mPositionToDelete );
            if (position >= mPositionToDelete)
                mPositionToDelete = position;
            else
                mPositionToDelete = position + 1;

            UtilsLog.d(TAG, "mPositionToDelete 3: "+ mPositionToDelete );
        }
        else
        mPositionToDelete = position + 1;

        UtilsLog.d(TAG,"mPositionToDelete 4: "+ mPositionToDelete );

        AlertDialog.Builder builder = new AlertDialog.Builder(getActivity());
        builder.setTitle(R.string.action_delete)
                .setMessage(mAdapter.getItem(mPositionToDelete - 1).getPublicText())
                .setCancelable(false)
                .setPositiveButton(
                        R.string.action_delete,
                        new DialogInterface.OnClickListener()
                        {
                            @Override
                            public void onClick(DialogInterface dialog, int which)
                            {
                                mDeleting = true;
                                deleteMessage(mPositionToDelete - 1);
                            }
                        }
                )
                .setNegativeButton(
                        R.string.action_cancel,
                        new DialogInterface.OnClickListener()
                        {
                            @Override
                            public void onClick(DialogInterface dialog, int which)
                            {
                                mAdapter.deleteSelection(mPositionToDelete - 1);
                                mAdapter.notifyItemChanged(mPositionToDelete - 1);
                            }
                        }
                )
                .setOnDismissListener(new DialogInterface.OnDismissListener()
                {
                    @Override
                    public void onDismiss(DialogInterface dialog)
                    {
                        mDeleteAlertDialog = null;
                        mPositionToDelete = 0;
                    }
                });
        mDeleteAlertDialog = builder.create();
        mDeleteAlertDialog.show();
    }

    /*
        IMPLEMENTS
        MezuPnsPCSService.OnDeleteListener
     */

    @Override
    public void onPostDeleteAsyncTask(DTOResponseSimple response, String[] trans_ids)
    {
        if (response.getResult().equals(DAOService.RESPONSE_RESULT_OK) && getActivity() != null)
        {
            DAOService daoService = new DAOService(getActivity());
            daoService.createDeleteMsgPendingOperationsCall(Constants.PNS_MESSAGE_OPERATION_DELETE, trans_ids ).start();
        }
    }
}