package com.ejgv.gvi;

import android.content.Context;
import android.os.AsyncTask;
import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.support.v7.app.ActionBar;
import android.support.v7.app.AppCompatActivity;
import android.view.ActionMode;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import com.ejgv.gvi.dao.DAOService;
import com.ejgv.gvi.dao.dto.DTOResponseGetMessages;
import com.ejgv.gvi.dmo.DMOMessage;
import com.ejgv.gvi.ui.EmptyRecyclerView;
import com.ejgv.gvi.utils.UtilsApp;

/**
 * Created by Batura Mobile Solutions (Koldo Calvo).
 */

/**
 * A simple {@link Fragment} subclass.
 * Activities that contain this fragment must implement the
 * {@link MessageSearchResultsFragment.OnFragmentInteractionListener} interface
 * to handle interaction events.
 * Use the {@link MessageSearchResultsFragment#newInstance} factory method to
 * create an instance of this fragment.
 */
public class MessageSearchResultsFragment extends MessageListFragment
                                    implements DAOService.OnGetMessagesListener
{
    private static final String TAG = "MessageListFragment";

    // the fragment initialization arguments
    private static final String ARG_FILTER_TAG = "FILTER_TAG";
    private String mFilterTag;

    // Functionallity variables
    private AsyncTask mGetMessagesAsyncTask;
    private boolean mRetained;

    // UI References
    private EmptyRecyclerView mRecyclerView;
    private TextView mTxtResultsCounter;

    // Interfaces
    private OnFragmentInteractionListener mListener;
    /**
     * This interface must be implemented by activities that contain this
     * fragment to allow an interaction in this fragment to be communicated
     * to the activity and potentially other fragments contained in that
     * activity.
     * <p/>
     * See the Android Training lesson <a href=
     * "http://developer.android.com/training/basics/fragments/communicating.html"
     * >Communicating with Other Fragments</a> for more information.
     */
    public interface OnFragmentInteractionListener
    {
        void onMessageSelected(DMOMessage message);
        void onActionMode(boolean isActive);
        void onTagsActionClicked();
        void onActionDeleted();
    }

    /**
     * Use this factory method to create a new instance of
     * this fragment using the provided parameters.
     *
     * @param strFilterTag Parameter 1.
     * @return A new instance of fragment MessageListFragment.
     */
    public static MessageSearchResultsFragment newInstance(String strFilterTag)
    {
        MessageSearchResultsFragment fragment = new MessageSearchResultsFragment();
        Bundle args = new Bundle();
        args.putString(ARG_FILTER_TAG, strFilterTag);
        fragment.setArguments(args);
        return fragment;
    }

    public MessageSearchResultsFragment()
    {
        // Required empty public constructor
    }

    @Override
    public void onAttach(Context context)
    {
        super.onAttach(context);
        try {
            mListener = (OnFragmentInteractionListener) context;
        } catch (ClassCastException e) {
            throw new ClassCastException(context.toString()
                    + " must implement MessageSearchResultsFragment.OnFragmentInteractionListener");
        }
    }

    @Override
    public void onCreate(Bundle savedInstanceState)
    {
        super.onCreate(savedInstanceState);

        if (getArguments() != null)
            mFilterTag = getArguments().getString(ARG_FILTER_TAG);


        ActionBar actionBar = ((AppCompatActivity) getActivity()).getSupportActionBar();
        String strTitle;
        if (mFilterTag != null && mFilterTag.indexOf('%') > 0)
            strTitle = mFilterTag.replace("%", getString(R.string.search_query_match_anything));
        else if (mFilterTag != null && !mFilterTag.isEmpty())
            strTitle = mFilterTag;
        else
            strTitle = getString(R.string.tag_empty);

        actionBar.setTitle(strTitle);
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState)
    {
        // Inflate the layout for this fragment
        View v = inflater.inflate(R.layout.fragment_search_results, container, false);

        mRecyclerView = (EmptyRecyclerView) v.findViewById(R.id.searchResults_recyclerView);
        View vEmpty = v.findViewById(R.id.searchResults_empty_lyt);
        ((TextView)vEmpty.findViewById(R.id.searchResults_empty_txt)).setText(getString(R.string.empty_search_result, mFilterTag.replace("%","")));
        mRecyclerView.setEmptyView(vEmpty);
        setRecyclerView(mRecyclerView);

        mTxtResultsCounter = (TextView) v.findViewById(R.id.searchResults_results_counter_txt);
        if (mRetained)
            mTxtResultsCounter.setText(String.valueOf(mRecyclerView.getAdapter().getItemCount()));

        return v;
    }

    @Override
    public void onActivityCreated(Bundle savedInstanceState)
    {
        super.onActivityCreated(savedInstanceState);
        if (!mRetained)
            init();
    }

    @Override
    public void onCreateOptionsMenu(final Menu menu, MenuInflater inflater)
    {
        super.onCreateOptionsMenu(menu, inflater);
        inflater.inflate(R.menu.menu_message_search_results, menu);
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item)
    {
        switch (item.getItemId())
        {
            case R.id.action_filter_messages_search_results:
                mListener.onTagsActionClicked();
                return true;
            default:
                return super.onOptionsItemSelected(item);
        }
    }

    @Override
    public void onPause()
    {
        mRetained = true;
        super.onPause();
    }

    @Override
    public void onDestroy()
    {
        if (mGetMessagesAsyncTask != null)
            mGetMessagesAsyncTask.cancel(true);
        super.onDestroy();
    }

    @Override
    public void onDetach()
    {
        mListener = null;
        mGetMessagesAsyncTask = null;
        super.onDetach();
    }

    /*
        EXTENDS
        MessageListFragment
     */

    @Override
    public void onMessageSelected(DMOMessage message)
    {
        mListener.onMessageSelected(message);
    }

    @Override
    protected void onActionModeCreated(ActionMode mode, Menu menu)
    {
        super.onActionModeCreated(mode, menu);
        mListener.onActionMode(true);
    }

    @Override
    protected void onActionModeDestroyed(ActionMode mode)
    {
        super.onActionModeDestroyed(mode);
        mListener.onActionMode(false);
    }

    @Override
    protected void onActionDeleted(int newItemCount)
    {
        super.onActionDeleted(newItemCount);
        mTxtResultsCounter.setText(String.valueOf(newItemCount));
        mListener.onActionDeleted();
    }

    /*
        PRIVATE METHODS
     */

    private void init()
    {
        DAOService daoService = new DAOService(getContext());
        mGetMessagesAsyncTask = daoService.createGetMessagesCall(this);
        mGetMessagesAsyncTask.execute(mFilterTag);
    }

    /*
        IMPLEMENTS
        DAOService.OnGetMessagesListener
     */

    @Override
    public void onPostGetMessagesAsyncTask(DTOResponseGetMessages response)
    {
        if (getContext() == null)
            return;

        if (response.getResult().equals(DAOService.RESPONSE_RESULT_OK))
        {
            mTxtResultsCounter.setText(String.valueOf(response.getMessages().length));
            // Set adapter's data
            setRecyclerViewAdapterData(response.getMessages());
        }
        else
            UtilsApp.showAlertDialogBasic(
                    getActivity(),
                    getActivity().getString(R.string.title_error_with_code, response.getCode()),
                    response.getDescription()
            );
    }
}