package com.ejgv.gvi;

/**
 * Created by Batura Mobile Solutions (Koldo Calvo).
 */

import android.app.Activity;
import android.os.AsyncTask;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ListView;

import com.ejgv.gvi.rss.RSSFeedItem;
import com.ejgv.gvi.rss.RSSFeedItemAdapter;
import com.ejgv.gvi.rss.RSSFeedXmlParser;
import com.ejgv.gvi.utils.Utils;

import org.xmlpull.v1.XmlPullParserException;

import java.io.IOException;
import java.io.InputStream;
import java.net.HttpURLConnection;
import java.net.SocketTimeoutException;
import java.net.URL;
import java.util.List;

/**
 * Created by Batura Mobile Solutions (Ivan Ornes).
 */

/**
 * A simple {@link PullToRefreshListFragment} subclass.
 * Activities that contain this fragment must implement the
 * {@link NewsFragment.OnRSSFeedListClickListener} interface
 * to handle interaction events.
 */
public class NewsFragment extends PullToRefreshListFragment 
{

    private static final String TAG = "NewsFragment";

    // Functionallity variables
    private DownloadXmlAsyncTask mDownloadXmlAsyncTask;
    private String mRSSFeedURL;
    private boolean mLoading = false;

    // Interfaces
    private OnRSSFeedListClickListener mListener;

    /**
     * This interface must be implemented by activities that contain this
     * fragment to allow an interaction in this fragment to be communicated
     * to the activity and potentially other fragments contained in that
     * activity.
     * <p/>
     * See the Android Training lesson <a href=
     * "http://developer.android.com/training/basics/fragments/communicating.html"
     * >Communicating with Other Fragments</a> for more information.
     */
    public interface OnRSSFeedListClickListener
    {
        public void onRSSFeedSelected(int position, RSSFeedItem item);
    }

    public NewsFragment()
    {
        // Required empty public constructor
    }

    /*
        EXTENDS
        Fragment
     */

    @Override
    public void onAttach(Activity activity)
    {
        super.onAttach(activity);
        try {
            mListener = (OnRSSFeedListClickListener) activity;
        } catch (ClassCastException e) {
            throw new ClassCastException("***************** " + activity.toString()
                    + " must implement onRSSFeedListClickListener *****************");
        }
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState)
    {

        if (getArguments() != null)
            mRSSFeedURL = getArguments().getString(Constants.PARAM_RSS_FEED);

        // Inflate the layout for this fragment
        return inflater.inflate(R.layout.fragment_news, container, false);
    }

    @Override
    public void onViewCreated(View view, Bundle savedInstanceState)
    {
        super.onViewCreated(view, savedInstanceState);

        getPullToRefreshLayout().setOnRefreshListener(this);

        init();
    }

    @Override
    public void onDestroy()
    {
        if (mDownloadXmlAsyncTask != null)
            mDownloadXmlAsyncTask.cancel(true);

        super.onDestroy();
    }

    @Override
    public void onDetach()
    {
        super.onDetach();
        mDownloadXmlAsyncTask = null;
    }

    /*
        EXTENDS
        PullToRefreshListFragment
     */
        
    @Override
    public void onListItemClick(ListView l, View v, int position, long id)
    {
        if (mListener != null) {
            RSSFeedItem item = (RSSFeedItem) getListAdapter().getItem(position);
            mListener.onRSSFeedSelected(position, item);
        }
    }

    @Override
    public void onRefresh()
    {
        init();
    }

    /*
        PRIVATE METHODS
     */
        
    private void init()
    {
        if (!mLoading) {

            if(Utils.isConnected(getActivity())) {

                mLoading = true;
                mDownloadXmlAsyncTask = new DownloadXmlAsyncTask();
                mDownloadXmlAsyncTask.execute(mRSSFeedURL);
            } else {
                getPullToRefreshLayout().setRefreshing(false);
                removeProgressBar();
            }
        } else {
            removeProgressBar();
        }
    }

    // Implementation of AsyncTask used to download XML feed
    private class DownloadXmlAsyncTask extends AsyncTask<String, Void, List<RSSFeedItem>>
    {
        private String errorText = null;

        @Override
        protected List<RSSFeedItem> doInBackground(String... urls) {
            try {
                return loadXmlFromNetwork(urls[0]);
            } catch (SocketTimeoutException e) {
                errorText = getActivity().getResources().getString(R.string.error_network_connection_timeout);
            } catch (XmlPullParserException e) {
                errorText = getString(R.string.error_xml_parse);
            } catch (Exception e) {
                errorText = getActivity().getResources().getString(R.string.error_general);
            }
            return null;
        }

        @Override
        protected void onPostExecute(List<RSSFeedItem> listRSSFeeds) {
            bindRSSFeedsToUI(listRSSFeeds, errorText);
            mLoading = false;
            getPullToRefreshLayout().setRefreshing(false);
            removeProgressBar();
        }
    }


    // Uploads XML, parses it, and combines it with
    // HTML markup. Returns HTML string.
    private List<RSSFeedItem> loadXmlFromNetwork(String urlString) throws XmlPullParserException, IOException
    {
        InputStream stream = null;
        // Instantiate the parser
        RSSFeedXmlParser xmlParser = new RSSFeedXmlParser();
        List<RSSFeedItem> items = null;

        try {
            stream = downloadUrl(urlString);
            items = xmlParser.parse(stream);
            // Makes sure that the InputStream is closed after the app is
            // finished using it.
        } finally {
            if (stream != null) {
                stream.close();
            }
        }

        return items;
    }

    // Given a string representation of a URL, sets up a connection and gets
    // an input stream.
    private InputStream downloadUrl(String urlString) throws IOException
    {
        URL url = new URL(urlString);
        HttpURLConnection conn = (HttpURLConnection) url.openConnection();
        conn.setReadTimeout(10000 /* milliseconds */);
        conn.setConnectTimeout(15000 /* milliseconds */);
        conn.setRequestMethod("GET");
        conn.setDoInput(true);
        // Starts the query
        conn.connect();
        return conn.getInputStream();
    }

    private void bindRSSFeedsToUI(List<RSSFeedItem> listRSSFeeds, String errorText)
    {
        if (getListAdapter() != null) {
            setListAdapter(null);
        }
        if (listRSSFeeds != null && listRSSFeeds.size() > 0) {
            hideEmptyView();
            RSSFeedItemAdapter adapter = new RSSFeedItemAdapter(
                    getActivity(),
                    R.layout.list_item_gvi_rss_feed_item,
                    listRSSFeeds);
            setListAdapter(adapter);
        } else if (listRSSFeeds != null && listRSSFeeds.size() == 0) {
            setEmptyView();
        } else {
            setEmptyView( (errorText != null) ? errorText : getString(R.string.error_ws_null) );
        }
    }
}