package com.ejgv.gvi;

/**
 * Copyright 2015 Google Inc. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import android.app.IntentService;
import android.content.Intent;

import com.ejgv.gvi.dao.DAOMsgPendingOperation;
import com.ejgv.gvi.dao.DDBBHelper;
import com.ejgv.gvi.dao.dto.DTOPnsMsgOperation;
import com.ejgv.gvi.restapipns.PNSHttpClient;
import com.ejgv.gvi.restapipns.dto.DTOResponseSimple;
import com.ejgv.gvi.restapipns.servicedataaccess.DeleteHttpClient;
import com.ejgv.gvi.restapipns.servicedataaccess.MarkAsReadHttpClient;
import com.ejgv.gvi.utils.UtilsApp;
import com.ejgv.gvi.utils.UtilsLog;
import com.google.gson.Gson;
import com.google.gson.GsonBuilder;

import org.json.JSONObject;

import java.util.ArrayList;

/**
 * Modified by Batura Mobile Solutions (Koldo Calvo).
 */
public class PendingOperationsIntentService extends IntentService
{
    private static final String TAG = "PendingOpeIntServ";

    public PendingOperationsIntentService()
    {
        super(TAG);
    }

    @Override
    protected void onHandleIntent(Intent intent)
    {
        AccessData accessData = new AccessData(this);
        DDBBHelper dbHelper = new DDBBHelper(this);
        DAOMsgPendingOperation daoMsgPendingOperation = new DAOMsgPendingOperation(dbHelper);
        try
        {
            // DDBB call
            DTOPnsMsgOperation[] operations = daoMsgPendingOperation.selectPendingOperations();
            ArrayList<String> delete_trans_ids = new ArrayList<>();

            DTOResponseSimple objResponse;
            for (int i=0, len=operations.length; i<len; i++)
            {
                if (operations[i].getMsgOperation() == Constants.PNS_MESSAGE_OPERATION_MARK_AS_READ)
                {
                    // Api call
                    MarkAsReadHttpClient markAsReadHttpClient = new MarkAsReadHttpClient(this);
                    JSONObject jsonResponse = markAsReadHttpClient.send(accessData.getUuidApp(), accessData.getUuidDevice(), operations[i].getIdTrans());
                    // Parse JSON response to DTO response
                    Gson gson = new Gson();
                    objResponse = gson.fromJson(jsonResponse.toString(), DTOResponseSimple.class);

                    Gson gson_p = new GsonBuilder().setPrettyPrinting().create();
                    UtilsLog.d(TAG, gson_p.toJson(objResponse));

                    if (objResponse != null && objResponse.getResult().equals(PNSHttpClient.RESPONSE_RESULT_OK))
                        daoMsgPendingOperation.deleteOperation(operations[i]);
                }
                else if (operations[i].getMsgOperation() == Constants.PNS_MESSAGE_OPERATION_DELETE)
                {
                    delete_trans_ids.add(operations[i].getIdTrans());
                }
            }

            if (delete_trans_ids.size() > 0)
            {
                String[] trans_ids = delete_trans_ids.toArray(new String[delete_trans_ids.size()]);
                // Api call
                DeleteHttpClient deleteHttpClient = new DeleteHttpClient(this);
                JSONObject jsonResponse = deleteHttpClient.send(accessData.getUuidApp(), accessData.getUuidDevice(), trans_ids );

                // Parse JSON response to DTO response
                Gson gson = new Gson();
                objResponse = gson.fromJson(jsonResponse.toString(), DTOResponseSimple.class);

                // Print
                Gson gson_p = new GsonBuilder().setPrettyPrinting().create();
                UtilsLog.d(TAG, gson_p.toJson(objResponse));

                if (objResponse != null && objResponse.getResult().equals(PNSHttpClient.RESPONSE_RESULT_OK))
                {
                    if (trans_ids.length == 1)
                    {
                        DTOPnsMsgOperation msgOperation = new DTOPnsMsgOperation();
                        msgOperation.setIdTrans(trans_ids[0]);
                        msgOperation.setMsgOperation(Constants.PNS_MESSAGE_OPERATION_DELETE);
                        daoMsgPendingOperation.deleteOperation(msgOperation);
                    }
                    else
                        daoMsgPendingOperation.deleteOperations(Constants.PNS_MESSAGE_OPERATION_DELETE, trans_ids);
                }
            }
            /*if (getApplicationContext() == null)
            {
                PreferencesApp preferencesApp = new PreferencesApp(this);
                DAOMessage daoMessage = new DAOMessage(dbHelper);
                long deleted_ids = daoMessage.deleteOver(preferencesApp.getMessagesMaxLimit());
                UtilsLog.d(TAG, "DelMsgsOverMaxLimit: "+ String.valueOf(deleted_ids));
            }*/
        }
        catch (Exception e)
        {
            UtilsLog.d(TAG, "Exception in onHandleIntent", e);
            UtilsApp.sendTraceToServer(UtilsApp.createStandardAddTraceParams(getClass().getName(), "onHandleIntent", e));
        }
    }
}