package com.ejgv.gvi;

import android.content.Context;
import android.content.SharedPreferences;
import android.support.v7.preference.PreferenceManager;

/**
 * Created by Batura Mobile Solutions (Koldo Calvo).
 */
public class PreferencesApp
{
    private SharedPreferences mSharedPreferences;
    public static final String SHAREDPREF_KEY_USER_LANGUAGE = "PREFERENCESAPP_USER_LANGUAGE";
    public static final String SHAREDPREF_VALUE_USER_LANGUAGE_DEFAULT = "es_ES";
    public static final String SHAREDPREF_KEY_MESSAGES_MAX_LIMIT = "PREFERENCESAPP_MESSAGES_MAX_LIMIT";
    public static final String SHAREDPREF_VALUE_MESSAGES_MAX_LIMIT_DEFAULT = "1000";
    public static final String SHAREDPREF_KEY_UNREGISTER = "PREFERENCESAPP_UNREGISTER";

    /**
     * Constructor
     *
     * @param context
     */
    public PreferencesApp(Context context)
    {
        mSharedPreferences = PreferenceManager.getDefaultSharedPreferences(context);
    }

    /**
     * Get app shared preferences
     *
     * @return SharedPreferences app shared preferences object
     */
    public SharedPreferences getPreferences()
    {
        return mSharedPreferences;
    }

    /**
     * Get user's app language
     *
     * @return user language setting
     */
    public String getUserLanguage()
    {
        return mSharedPreferences.getString(SHAREDPREF_KEY_USER_LANGUAGE, SHAREDPREF_VALUE_USER_LANGUAGE_DEFAULT);
    }

    /**
     *
     *
     * @param language the language to set as default locale
     */
    public void setUserLanguage(String language)
    {
        SharedPreferences.Editor editor = mSharedPreferences.edit();
        if (language != null && !language.isEmpty())
            editor.putString(SHAREDPREF_KEY_USER_LANGUAGE, language);
        else
            editor.remove(SHAREDPREF_KEY_USER_LANGUAGE);
        editor.commit();
    }

    /**
     *
     * @return int
     */
    public int getMessagesMaxLimit()
    {
        try
        {
            return Integer.parseInt(mSharedPreferences.getString(SHAREDPREF_KEY_MESSAGES_MAX_LIMIT, SHAREDPREF_VALUE_MESSAGES_MAX_LIMIT_DEFAULT));
        }
        catch (Exception e)
        {
            e.printStackTrace();
            return Integer.parseInt(SHAREDPREF_VALUE_MESSAGES_MAX_LIMIT_DEFAULT);
        }
    }

    /**
     *
     * @param intLimit
     */
    public void setMessagesMaxLimit(int intLimit)
    {
        SharedPreferences.Editor editor = mSharedPreferences.edit();
        if (intLimit > 0)
            editor.putString(SHAREDPREF_KEY_MESSAGES_MAX_LIMIT, String.valueOf(intLimit));
        else
            editor.remove(SHAREDPREF_KEY_MESSAGES_MAX_LIMIT);
        editor.commit();
    }

    /**
     * Delete session preferences
     */
    public void sessionExpired()
    {
        SharedPreferences.Editor editor = mSharedPreferences.edit();
        editor.remove(SHAREDPREF_KEY_USER_LANGUAGE);
        editor.remove(SHAREDPREF_KEY_MESSAGES_MAX_LIMIT);
        editor.commit();
    }

    /**
     * Remove all preferences
     */
    public void remove()
    {
        mSharedPreferences.edit().clear().commit();
    }

}