package com.ejgv.gvi;

import android.app.Activity;
import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.support.v7.app.AppCompatActivity;
import android.view.View;
import android.view.WindowManager;
import android.view.animation.Animation;
import android.view.animation.AnimationUtils;
import android.widget.Button;

import com.ejgv.gvi.utils.Utils;

/**
 * Created by Batura Mobile Solutions (Koldo Calvo).
 */
public class PresentationActivity extends AppCompatActivity implements View.OnClickListener
{
    // CUSTOM VARIABLES
    public static final int SPLASHSCREEN_TIMEOUT = 1500;

    @Override
    protected void onCreate(Bundle savedInstanceState)
    {
        super.onCreate(savedInstanceState);

        setContentView(R.layout.activity_presentation);

        final View lytPresentation = findViewById(R.id.presentation_wrapper_lyt);
        lytPresentation.setVisibility(View.GONE);

        Button btnStartUsingApp = (Button) findViewById(R.id.presentation_startUsingApp_btn);
        btnStartUsingApp.setOnClickListener(this);

        if (savedInstanceState == null)
        {
            new Handler().postDelayed(new Runnable() {
                @Override
                public void run() {
                    if (isUserLoggedIn())
                    {
                        startMainActivity();
                        Utils.clearTypefaces();
                    }
                    else
                    {
                        showStatusBar();
                        lytPresentation.setVisibility(View.VISIBLE);
                        Animation slide_down = AnimationUtils.loadAnimation(getApplicationContext(),
                                R.anim.slide_show_top_down);
                        lytPresentation.startAnimation(slide_down);

                    }
                }
            }, SPLASHSCREEN_TIMEOUT);
        }
    }

    @Override
    public void onClick(View v)
    {
        if (v.getId() == R.id.presentation_startUsingApp_btn)
            startRegisterActivity();
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data)
    {
        switch (requestCode)
        {
            case 0:
                if (resultCode == Activity.RESULT_OK)
                    finish();
                break;
            default:
                super.onActivityResult(requestCode, resultCode, data);
        }
    }

    /**
     * Check if the user is previously logged in
     * @return boolean
     */
    private boolean isUserLoggedIn()
    {
        AccessData accessData = new AccessData(this);
        if (accessData.isTokenPnsRegistered() && accessData.getUuidDevice() != null)
            return true;
        else
            return false;
    }

    /**
     * Start register process
     */
    private void startRegisterActivity()
    {
        Intent intent = new Intent(this, RegisterActivity.class);
        startActivityForResult(intent, 0);
        overridePendingTransition(0, 0);
    }

    /**
     * Start main activity
     */
    private void startMainActivity()
    {
        Intent intent = new Intent(this, MainActivity.class);
        startActivity(intent);
        finish();
        overridePendingTransition(android.R.anim.fade_in, android.R.anim.fade_out);
    }

    private void showStatusBar()
    {
        getWindow().addFlags(
                WindowManager.LayoutParams.FLAG_FORCE_NOT_FULLSCREEN);
        getWindow().clearFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN);
    }
}