package com.ejgv.gvi;

import android.app.Activity;
import android.content.Intent;
import android.content.SharedPreferences;
import android.os.Bundle;
import android.support.v4.app.NavUtils;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.Toolbar;
import android.view.MenuItem;
import android.view.View;

import com.ejgv.gvi.utils.Utils;

/**
 * Created by Batura Mobile Solutions (Koldo Calvo).
 */
public class RegisterActivity extends AppCompatActivity
                                implements RegisterStepOneFragment.OnFragmentInteractionListener,
                                            RegisterStepTwoFragment.OnFragmentInteractionListener
{
    public static final String TAG = "RegisterActivity";

    // Custom variables
    public static final String SHAREDPREF_REGISTER	= "REGISTER";
    public static final String SHAREDPREF_REGISTER_KEY_LAST_STEP = "REGISTER_LAST_STEP";
    public static final String SHAREDPREF_REGISTER_KEY_PHONE_NUMBER = "PHONE_NUMBER";
    private boolean mRegisterFinished;

    @Override
    protected void onCreate(Bundle savedInstanceState)
    {
        super.onCreate(savedInstanceState);

        setContentView(R.layout.activity_register);

        Toolbar toolbar = (Toolbar) findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);

        getSupportActionBar().setDisplayHomeAsUpEnabled(true);
        getSupportActionBar().setHomeAsUpIndicator(R.drawable.ic_back);
        getSupportActionBar().setTitle(null);

        View lytContent = findViewById(R.id.content_lyt);

        if (savedInstanceState == null)
        {
            lytContent.setVisibility(View.GONE);
            init();
            lytContent.setVisibility(View.VISIBLE);
        }
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item)
    {
        onBackPressed();
        return true;
    }

    @Override
    public void onBackPressed()
    {
        if (getSupportFragmentManager().getBackStackEntryCount() > 1)
        {
            getSupportFragmentManager().popBackStack();
        }
        else
        {
            NavUtils.navigateUpFromSameTask(this);
            overridePendingTransition(0, 0);
        }
    }

    @Override
    protected void onStop()
    {
        if (mRegisterFinished)
            clearRegisterPreferences();
        else
            putStringInRegisterSharedPreferences(SHAREDPREF_REGISTER_KEY_LAST_STEP, getActiveFragmentTag());
        super.onStop();
    }

    @Override
    protected void onDestroy()
    {
        Utils.clearTypefaces();
        super.onDestroy();
    }

    /*
        PRIVATE METHODS
     */

    private String getActiveFragmentTag()
    {
        return getSupportFragmentManager().findFragmentById(R.id.content_lyt).getTag();
    }

    private void init()
    {
        SharedPreferences sharedPreferences = getSharedPreferences(SHAREDPREF_REGISTER, MODE_PRIVATE);
        String  strRegisterLastStep = sharedPreferences.getString(SHAREDPREF_REGISTER_KEY_LAST_STEP, null);

        if (strRegisterLastStep != null && strRegisterLastStep.equals("2"))
        {
            String strPhoneNumber = sharedPreferences.getString(SHAREDPREF_REGISTER_KEY_PHONE_NUMBER, null);
            addRegisterStepOneFragment();
            addRegisterStepTwoFragment(strPhoneNumber);
        }
        else
        {
            addRegisterStepOneFragment();
        }
    }

    private void addRegisterStepOneFragment()
    {
        RegisterStepOneFragment registerStepOneFragment = new RegisterStepOneFragment();
        getSupportFragmentManager().beginTransaction().replace(R.id.content_lyt, registerStepOneFragment, "1").addToBackStack(null).commit();
    }

    private void addRegisterStepTwoFragment(String strPhoneNumber)
    {
        RegisterStepTwoFragment registerStepTwoFragment = RegisterStepTwoFragment.newInstance(strPhoneNumber);
        getSupportFragmentManager().beginTransaction().replace(R.id.content_lyt, registerStepTwoFragment, "2").addToBackStack(null).commit();
    }

    private void putStringInRegisterSharedPreferences(String strKey, String strValue)
    {
        SharedPreferences sharedPreferences = getSharedPreferences(SHAREDPREF_REGISTER, MODE_PRIVATE);
        SharedPreferences.Editor editor = sharedPreferences.edit();
        if (strValue != null)
            editor.putString(strKey, strValue);
        else
            editor.remove(strKey);
        editor.commit();
    }

    private void clearRegisterPreferences()
    {
        SharedPreferences sharedPreferences = getSharedPreferences(SHAREDPREF_REGISTER, MODE_PRIVATE);
        SharedPreferences.Editor editor = sharedPreferences.edit();
        editor.clear();
        editor.commit();
    }

    /*
        IMPLEMENTS
        RegisterStepOneFragment.OnFragmentInteractionListener
     */

    @Override
    public void onRegisterStepOneSuccess(String mPhoneNumberFormatted)
    {
        putStringInRegisterSharedPreferences(SHAREDPREF_REGISTER_KEY_PHONE_NUMBER, mPhoneNumberFormatted);
        addRegisterStepTwoFragment(mPhoneNumberFormatted);
    }

    /*
        IMPLEMENTS
        RegisterStepTwoFragment.OnFragmentInteractionListener
     */

    @Override
    public void onRegisterStepTwoSuccess()
    {
        mRegisterFinished = true;
        // Start app main activity
        setResult(Activity.RESULT_OK);
        Intent intent = new Intent(this, MainActivity.class);
        startActivity(intent);
        overridePendingTransition(android.R.anim.fade_in, android.R.anim.fade_out);
        finish();
    }
}