package com.ejgv.gvi;

import android.content.Context;
import android.os.AsyncTask;
import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.text.TextUtils;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.RelativeLayout;

import com.ejgv.gvi.restapipns.PNSHttpClient;
import com.ejgv.gvi.restapipns.dto.DTOResponseGetSmsToken;
import com.ejgv.gvi.restapipns.serviceappregister.MezuPnsPRSService;
import com.ejgv.gvi.ui.TypefaceEditText;
import com.ejgv.gvi.utils.UtilsApp;

/**
 * Created by Batura Mobile Solutions (Koldo Calvo).
 */
/**
 * A simple {@link Fragment} subclass.
 * Activities that contain this fragment must implement the
 * {@link RegisterStepOneFragment.OnFragmentInteractionListener} interface
 * to handle interaction events.
 * create an instance of this fragment.
 */
public class RegisterStepOneFragment extends Fragment
                                    implements View.OnClickListener,
                                                MezuPnsPRSService.OnGetSmsTokenListener
{
    // CUSTOM VARIABLES
    private TypefaceEditText mEtxtPhoneNumber;
    private String mPhoneNumberFormatted;
    private AsyncTask mAsyncTask;

    // CUSTOM INTERFACES
    private OnFragmentInteractionListener mListener;
    private RelativeLayout mRlyProgressBar;

    /**
     * This interface must be implemented by activities that contain this
     * fragment to allow an interaction in this fragment to be communicated
     * to the activity and potentially other fragments contained in that
     * activity.
     * <p/>
     * See the Android Training lesson <a href=
     * "http://developer.android.com/training/basics/fragments/communicating.html"
     * >Communicating with Other Fragments</a> for more information.
     */
    public interface OnFragmentInteractionListener
    {
        void onRegisterStepOneSuccess(String mPhoneNumberFormatted);
    }

    public RegisterStepOneFragment()
    {
        // Required empty public constructor
    }

    @Override
    public void onCreate(Bundle savedInstanceState)
    {
        super.onCreate(savedInstanceState);
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState)
    {
        // Inflate the layout for this fragment
        View v =  inflater.inflate(R.layout.fragment_register_step_one, container, false);

        mEtxtPhoneNumber = (TypefaceEditText) v.findViewById(R.id.registerStepOne_phoneNumber_etxt);

        mRlyProgressBar = (RelativeLayout) v.findViewById(R.id.layoutProgressbar_main_lyt);

        Button btnRegisterPhoneNumber = (Button) v.findViewById(R.id.registerStepOne_registerPhoneNumber_btn);
        btnRegisterPhoneNumber.setOnClickListener(this);

        return v;
    }

    @Override
    public void onAttach(Context context)
    {
        super.onAttach(context);
        try {
            mListener = (OnFragmentInteractionListener) context;
        } catch (ClassCastException e) {
            throw new ClassCastException(context.toString()
                    + " must implement RegisterStepOneFragment.OnFragmentInteractionListener");
        }
    }

    @Override
    public void onDestroy()
    {
        if (mAsyncTask != null)
            mAsyncTask.cancel(true);
        super.onDestroy();
    }

    @Override
    public void onDetach()
    {
        super.onDetach();
        mListener = null;
        mAsyncTask = null;
    }

    @Override
    public void onClick(View v)
    {
        String strPhoneNumber = mEtxtPhoneNumber.getText().toString().trim();
        if (v.getId() == R.id.registerStepOne_registerPhoneNumber_btn && mListener != null && isValidPhoneNumber(strPhoneNumber))
        {
            mRlyProgressBar.setVisibility(View.VISIBLE);
            mPhoneNumberFormatted = strPhoneNumber;

            MezuPnsPRSService mezuPnsPRSService = new MezuPnsPRSService(getContext());
            mAsyncTask = mezuPnsPRSService.createGetSmsTokenCall(
                    this,
                    mPhoneNumberFormatted,
                    Constants.APPLICATION_REFERENCE
            );
            mAsyncTask.execute();
        }
    }

    /**
     * Check if a string is a phone number
     *
     * @param strPhoneNumber
     * @return boolean
     */
    private boolean isValidPhoneNumber(String strPhoneNumber)
    {
        if (strPhoneNumber != null && !strPhoneNumber.isEmpty() && TextUtils.isDigitsOnly(strPhoneNumber) && strPhoneNumber.length() <= 15)
        {
            return true;
        }
        else
        {
            UtilsApp.showAlertDialogBasic(
                    getActivity(),
                    getString(R.string.title_info),
                    getString(R.string.error_validation_phone_number)
            );
            return false;
        }
    }


    /*
        IMPLEMENTS
        MezuPnsPRSService.OnGetSmsTokenListener
     */

    @Override
    public void onPostGetSmsTokenAsyncTask(DTOResponseGetSmsToken response)
    {
        if (getContext() == null)
            return;

        if (response.getResult().equals(PNSHttpClient.RESPONSE_RESULT_OK))
        {
            AccessData accessData = new AccessData(getActivity());
            accessData.setUuidApp(response.getUuidApp());

            mListener.onRegisterStepOneSuccess(mPhoneNumberFormatted);
        }
        else
        {
            UtilsApp.showAlertDialogBasic(
                    getActivity(),
                    getActivity().getString(R.string.title_error_with_code, response.getCode()),
                    response.getDescription()
            );
        }
        if (mRlyProgressBar != null)
            mRlyProgressBar.setVisibility(View.GONE);
    }

}