package com.ejgv.gvi;

import android.app.SearchManager;
import android.content.Intent;
import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.Toolbar;
import android.view.View;

import com.ejgv.gvi.dmo.DMOMessage;

public class SearchResultsActivity extends AppCompatActivity
                                    implements MessageSearchResultsFragment.OnFragmentInteractionListener,
                                                TagsFilterFragment.OnFragmentInteractionListener,
                                                MessagePrivateFragment.OnFragmentInteractionListener
{
    private boolean mActionPerformed;
    private String mTitle;

    /*
        EXTENDED METHODS
     */

    @Override
    protected void onCreate(Bundle savedInstanceState)
    {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_search_results);

        Toolbar toolbar = (Toolbar) findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);
        toolbar.setNavigationOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                onBackPressed();
            }
        });

        getSupportActionBar().setDisplayShowHomeEnabled(true);
        getSupportActionBar().setDisplayHomeAsUpEnabled(true);
        getSupportActionBar().setDisplayShowTitleEnabled(true);

        handleIntent(getIntent());
    }

    @Override
    protected void onNewIntent(Intent intent)
    {
        setIntent(intent);
        handleIntent(intent);
    }

    @Override
    public void onBackPressed()
    {
        if (getSupportFragmentManager().getBackStackEntryCount() > 0)
            getSupportFragmentManager().popBackStackImmediate();
        else
        {
            if (mActionPerformed)
                setResult(RESULT_OK);
            else
                setResult(RESULT_CANCELED);
            super.onBackPressed();
        }
    }

    /*
        PRIVATE METHODS
     */

    /**
     * Check if a search is handled to show results
     *
     * @param intent
     */
    private void handleIntent(Intent intent)
    {
        // Get the intent, verify the action and get the query
        if (intent != null && Intent.ACTION_SEARCH.equals(intent.getAction()))
        {
            String query = intent.getStringExtra(SearchManager.QUERY);
            searchResults(query);
        }
    }


    /**
     * Search and show results
     *
     * @param query
     */
    private void searchResults(String query)
    {
        MessageSearchResultsFragment fragment = MessageSearchResultsFragment.newInstance(query);
        getSupportFragmentManager().beginTransaction().replace(R.id.content_lyt, fragment, "MessageSearchResultsFragment").commit();
    }

    private void hideTitle() {
        mTitle = getSupportActionBar().getTitle().toString();
        getSupportActionBar().setTitle("");
    }

    private void showTitle() {
        getSupportActionBar().setTitle(mTitle);
    }

    /*
        IMPLEMENTS
        MessageSearchResultsFragment.OnFragmentInteractionListener
     */

    @Override
    public void onMessageSelected(DMOMessage message)
    {
        MessagePrivateFragment messagePrivateFragment = MessagePrivateFragment.newInstance(message);
        getSupportFragmentManager().beginTransaction().replace(R.id.content_lyt, messagePrivateFragment).addToBackStack(null).commit();
    }

    @Override
    public void onActionMode(boolean isActive)
    {
    }

    @Override
    public void onTagsActionClicked()
    {
        TagsFilterFragment tagsFilterFragment = new TagsFilterFragment();
        getSupportFragmentManager().beginTransaction().add(R.id.content_lyt, tagsFilterFragment).addToBackStack(null).commit();
        hideTitle();
    }

    @Override
    public void onActionDeleted()
    {
        mActionPerformed = true;
    }

    /*
        IMPLEMENTS
        TagsFilterFragment.OnFragmentInteractionListener
     */

    @Override
    public void onQueryTextSubmit(String query)
    {
        onBackPressed();
        Intent intent = new Intent(this, SearchResultsActivity.class);
        intent.setAction(Intent.ACTION_SEARCH);
        intent.putExtra(SearchManager.QUERY, query);
        startActivity(intent);
    }

    @Override
    public void onTagFilterDetach() {
        showTitle();
    }

    /*
        IMPLEMENTS
        MessagePrivateFragment.OnFragmentInteractionListener
     */

    @Override
    public void onMarkAsRead(String trans_id, long tsRead)
    {
        mActionPerformed = true;
        Fragment fragment = getSupportFragmentManager().findFragmentByTag("MessageSearchResultsFragment");
        if (fragment != null)
            ((MessageSearchResultsFragment) fragment).messageRead(trans_id, tsRead);
    }

    @Override
    public void onPnsMessageDelete(String trans_id)
    {
        mActionPerformed = true;
        Fragment fragment = getSupportFragmentManager().findFragmentByTag("MessageSearchResultsFragment");
        if (fragment != null)
        {
            ((MessageSearchResultsFragment) fragment).messageDeleted(trans_id);
            onBackPressed();
        }
    }
}