package com.ejgv.gvi;

import android.content.Context;
import android.content.DialogInterface;
import android.content.SharedPreferences;
import android.os.AsyncTask;
import android.os.Bundle;
import android.support.design.widget.Snackbar;
import android.support.v14.preference.MultiSelectListPreference;
import android.support.v7.app.ActionBar;
import android.support.v7.app.AlertDialog;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.preference.EditTextPreference;
import android.support.v7.preference.ListPreference;
import android.support.v7.preference.Preference;
import android.support.v7.preference.PreferenceFragmentCompat;
import android.support.v7.preference.PreferenceGroup;
import android.support.v7.preference.PreferenceManager;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import com.ejgv.gvi.interfaces.OnDeviceUnregisterListener;
import com.ejgv.gvi.restapipns.PNSHttpClient;
import com.ejgv.gvi.restapipns.dto.DTOResponseSimple;
import com.ejgv.gvi.restapipns.serviceappregister.MezuPnsPRSService;
import com.ejgv.gvi.utils.Utils;
import com.ejgv.gvi.utils.UtilsApp;

/**
 * Created by Batura Mobile Solutions (Koldo Calvo).
 */
/**
 * A simple {@link PreferenceFragmentCompat} subclass.
 * Activities that contain this fragment must implement the
 * {@link SettingsFragment.OnFragmentInteractionListener} interface
 * to handle interaction events.
 */
public class SettingsFragment extends PreferenceFragmentCompat
                            implements SharedPreferences.OnSharedPreferenceChangeListener,
                                        MezuPnsPRSService.OnUnregisterListener
{
    // Variables
    private AsyncTask mUnregisterAsyncTask;

    // Interfaces
    private OnDeviceUnregisterListener mDeviceUnregisterListener;
    private OnFragmentInteractionListener mListener;
    /**
     * This interface must be implemented by activities that contain this
     * fragment to allow an interaction in this fragment to be communicated
     * to the activity and potentially other fragments contained in that
     * activity.
     * <p/>
     * See the Android Training lesson <a href=
     * "http://developer.android.com/training/basics/fragments/communicating.html"
     * >Communicating with Other Fragments</a> for more information.
     */
    public interface OnFragmentInteractionListener
    {
        void onSettingLanguageChanged();
        void onMessageMaxLimitChanged(int newLimit);
    }

    public SettingsFragment()
    {
        // Required empty public constructor
    }

    /*
        EXTENDS PreferenceFragmentCompat
     */

    @Override
    public void onCreatePreferences(Bundle bundle, String s)
    {
        PreferenceManager.setDefaultValues(getContext(), R.xml.preferences, false);

        // Load the preferences from an XML resource
        addPreferencesFromResource(R.xml.preferences);

        initSummary(getPreferenceScreen());
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState)
    {
        ActionBar actionBar = ((AppCompatActivity) getActivity()).getSupportActionBar();
        actionBar.setDisplayShowTitleEnabled(true);
        actionBar.setTitle(R.string.title_fragment_settings);

        Preference prefUnregister = findPreference(PreferencesApp.SHAREDPREF_KEY_UNREGISTER);
        prefUnregister.setOnPreferenceClickListener(new Preference.OnPreferenceClickListener()
        {
            @Override
            public boolean onPreferenceClick(Preference preference)
            {
                AlertDialog.Builder builder = new AlertDialog.Builder(getActivity());
                builder.setTitle(R.string.title_info)
                        .setMessage(R.string.confirm_unregister)
                        .setPositiveButton(R.string.action_yes, new DialogInterface.OnClickListener()
                        {
                            @Override
                            public void onClick(DialogInterface dialog, int which)
                            {
                                unregisterDevice();
                            }
                        })
                        .setNegativeButton(R.string.action_no, null);
                builder.create().show();

                return true;
            }
        });

        Preference prefMessagesMaxLimit = findPreference(PreferencesApp.SHAREDPREF_KEY_MESSAGES_MAX_LIMIT);
        prefMessagesMaxLimit.setOnPreferenceChangeListener(new Preference.OnPreferenceChangeListener()
        {
            @Override
            public boolean onPreferenceChange(Preference preference, Object obj_value)
            {
                int intOldLimit = 0;
                int intNewLimit = 0;
                try
                {
                    intNewLimit = Integer.parseInt(String.valueOf(obj_value));
                    intOldLimit = Integer.parseInt(preference.getSummary().toString());
                }
                catch (Exception e)
                {
                    e.printStackTrace();
                }

                if (intOldLimit > intNewLimit)
                {
                    messagesMaxLimitReduced(preference, intNewLimit);
                    return false;
                }
                else
                    return true;
            }
        });

        return super.onCreateView(inflater, container, savedInstanceState);
    }

    @Override
    public void onAttach(Context context)
    {
        super.onAttach(context);
        try {
            mDeviceUnregisterListener = (OnDeviceUnregisterListener) context;
            mListener = (OnFragmentInteractionListener) context;
        } catch (ClassCastException e) {
            throw new ClassCastException(context.toString()
                    + " must implement OnFragmentInteractionListener");
        }
    }

    @Override
    public void onResume() {
        super.onResume();
        getPreferenceScreen().getSharedPreferences()
                .registerOnSharedPreferenceChangeListener(this);
    }

    @Override
    public void onPause()
    {
        getPreferenceScreen().getSharedPreferences()
                .unregisterOnSharedPreferenceChangeListener(this);
        super.onPause();
    }

    @Override
    public void onDestroy()
    {
        if (mUnregisterAsyncTask != null)
            mUnregisterAsyncTask.cancel(true);

        super.onDestroy();
    }

    @Override
    public void onDetach()
    {
        super.onDetach();
        mListener = null;
        mDeviceUnregisterListener = null;
        mUnregisterAsyncTask = null;
    }

    /*
        PRIVATE METHODS
     */

    /**
     * Add summary to each preference
     *
     * @param p
     */
    private void initSummary(Preference p)
    {
        if (p instanceof PreferenceGroup)
        {
            PreferenceGroup pGrp = (PreferenceGroup) p;
            for (int i = 0; i < pGrp.getPreferenceCount(); i++)
            {
                initSummary(pGrp.getPreference(i));
            }
        }
        else {
            updatePrefSummary(p);
        }
    }

    /**
     * Update preference summary with the new value set
     *
     * @param p
     */
    private void updatePrefSummary(Preference p)
    {
        if (p instanceof ListPreference)
        {
            ListPreference listPref = (ListPreference) p;
            CharSequence str_summary = listPref.getEntry();
            p.setSummary(str_summary);
        }
        if (p instanceof EditTextPreference)
        {
            EditTextPreference editTextPref = (EditTextPreference) p;
            if (p.getTitle().toString().contains("assword"))
            {
                p.setSummary("******");
            } else {
                p.setSummary(editTextPref.getText());
            }
        }
        if (p instanceof MultiSelectListPreference)
        {
            EditTextPreference editTextPref = (EditTextPreference) p;
            p.setSummary(editTextPref.getText());
        }
    }

    /**
     * Unregister app from system
     */
    private void unregisterDevice()
    {
        if (Utils.isConnected(getContext()))
        {
            AccessData accessData = new AccessData(getContext());
            MezuPnsPRSService mezuPnsPRSService = new MezuPnsPRSService(getContext());
            mUnregisterAsyncTask = mezuPnsPRSService.createUnregisterCall(this, accessData.getUuidApp(), accessData.getUuidDevice());
            mUnregisterAsyncTask.execute();
        }
        else
        {
            Snackbar mySnackbar = Snackbar.make(getActivity().findViewById(R.id.appBar_coordinatorLayout),
                    R.string.error_connection_no_connected, Snackbar.LENGTH_LONG);
            mySnackbar.setAction(R.string.action_retry, new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    unregisterDevice();
                }
            });
            mySnackbar.show();
        }
    }

    private void messagesMaxLimitReduced(final Preference preference, final int newPrefValue )
    {

        AlertDialog alertDialog = new AlertDialog.Builder(getContext())
                .setTitle(R.string.title_info)
                .setMessage(getString(R.string.confirm_message_limit, newPrefValue))
                .setPositiveButton(R.string.action_accept, new DialogInterface.OnClickListener()
                {
                    @Override
                    public void onClick(DialogInterface dialog, int which)
                    {
                        ((ListPreference) preference).setValue(String.valueOf(newPrefValue));
                    }
                })
                .setNegativeButton(R.string.action_cancel, null)
                .create();
        alertDialog.show();
    }


    /*
        IMPLEMENTS
        SharedPreferences.OnSharedPreferenceChangeListener
     */

    @Override
    public void onSharedPreferenceChanged(SharedPreferences sharedPreferences, String key)
    {
        Preference preference = findPreference(key);
        updatePrefSummary(preference);

        if ( key.equals(PreferencesApp.SHAREDPREF_KEY_USER_LANGUAGE) )
            mListener.onSettingLanguageChanged();
        if ( key.equals(PreferencesApp.SHAREDPREF_KEY_MESSAGES_MAX_LIMIT) )
            mListener.onMessageMaxLimitChanged(Integer.parseInt(preference.getSummary().toString()));
    }

    /*
        IMPLEMENTS
        MezuPnsPRSService.OnUnregisterListener
     */

    @Override
    public void onPostUnregisterAsyncTask(DTOResponseSimple response)
    {
        if (response.getResult().equals(PNSHttpClient.RESPONSE_RESULT_OK) && mDeviceUnregisterListener != null)
        {
            mDeviceUnregisterListener.onDeviceUnregistered();
        }
        else
        {
            UtilsApp.showAlertDialogBasic(
                    getActivity(),
                    getActivity().getString(R.string.title_error_with_code, response.getCode()),
                    UtilsApp.getTextLocale(getActivity(), response.getDescription())
            );
        }
    }

}