package com.ejgv.gvi;

import android.app.SearchManager;
import android.content.Context;
import android.graphics.Rect;
import android.os.AsyncTask;
import android.os.Build;
import android.os.Bundle;
import android.support.v4.app.ListFragment;
import android.support.v4.view.MenuItemCompat;
import android.support.v7.widget.SearchView;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.view.ViewTreeObserver;
import android.view.inputmethod.InputMethodManager;
import android.widget.ArrayAdapter;
import android.widget.FrameLayout;
import android.widget.ListView;

import com.ejgv.gvi.adapters.TagsFilterAdapter;
import com.ejgv.gvi.dao.DAOService;
import com.ejgv.gvi.dao.dto.DTOResponseTagList;
import com.ejgv.gvi.restapipns.dto.DTOResponseSimple;

/**
 * Created by Batura Mobile Solutions (Koldo Calvo).
 */
/**
 * A simple {@link ListFragment} subclass.
 * Activities that contain this fragment must implement the
 * {@link TagsFilterFragment.OnFragmentInteractionListener} interface
 * to handle interaction events.
 */
public class TagsFilterFragment extends ListFragment implements
                                                                ViewTreeObserver.OnGlobalLayoutListener,
                                                                View.OnClickListener,
                                                                SearchView.OnQueryTextListener,
                                                                DAOService.OnPopulateTagsListener,
                                                                DAOService.OnGetTagsListener
{
    private static final String TAG = "TagsFilterFragment";

    // Functionality variables
    private AsyncTask mGetTagsAsyncTask;
    private AsyncTask mPopulateAsyncTask;
    private boolean mbKeyboardHidden;
    private boolean mbMinHeightExceeded;
    private int mReduceHeight;

    // Custom interfaces
    private OnFragmentInteractionListener mListener;
    /**
     * This interface must be implemented by activities that contain this
     * fragment to allow an interaction in this fragment to be communicated
     * to the activity and potentially other fragments contained in that
     * activity.
     * <p/>
     * See the Android Training lesson <a href=
     * "http://developer.android.com/training/basics/fragments/communicating.html"
     * >Communicating with Other Fragments</a> for more information.
     */
    public interface OnFragmentInteractionListener
    {
        void onQueryTextSubmit(String query);

        void onTagFilterDetach();
    }

    /**
     * Constructor
     */
    public TagsFilterFragment()
    {
        // Required empty public constructor
    }

    /*
        EXTENDED METHODS
     */

    @Override
    public void onCreate(Bundle savedInstanceState)
    {
        super.onCreate(savedInstanceState);
        setHasOptionsMenu(true);


    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState)
    {
        // Inflate the layout for this fragment
        View v = inflater.inflate(R.layout.fragment_tags_filter, container, false);

        View main = v.findViewById(R.id.tagsFilter_main_lyt);
        main.setOnClickListener(this);

        return v;
    }

    @Override
    public void onViewCreated(View view, Bundle savedInstanceState)
    {
        super.onViewCreated(view, savedInstanceState);
        getActivity().getWindow().getDecorView().getViewTreeObserver().addOnGlobalLayoutListener(this);
        init();
    }

    @Override
    public void onAttach(Context context)
    {
        super.onAttach(context);
        try {
            mListener = (OnFragmentInteractionListener) context;
        } catch (ClassCastException e) {
            throw new ClassCastException(context.toString()
                    + " must implement TagsFilterFragment.OnFragmentInteractionListener");
        }
    }

    @Override
    public void onCreateOptionsMenu(Menu menu, MenuInflater inflater)
    {
        inflater.inflate(R.menu.menu_search, menu);
        MenuItem menuItem = menu.findItem(R.id.action_filter_messages_search);
        MenuItem menuItemInbox = menu.findItem(R.id.action_filter_messages_message_inbox);
        if (menuItemInbox != null)
            menuItemInbox.setVisible(false);

        MenuItem menuItemSearchResults = menu.findItem(R.id.action_filter_messages_search_results);
        if (menuItemSearchResults != null)
            menuItemSearchResults.setVisible(false);

        initSearchView(menuItem);
    }

    @Override
    public void onDetach()
    {
        if (mListener!= null) {
            mListener.onTagFilterDetach();
        }

        mListener = null;

        if (Build.VERSION.SDK_INT < 16)
            getActivity().getWindow().getDecorView().getViewTreeObserver().removeGlobalOnLayoutListener(this);
        else
            getActivity().getWindow().getDecorView().getViewTreeObserver().removeOnGlobalLayoutListener(this);

        hideKeyboard();
        super.onDetach();
    }

    @Override
    public void onListItemClick(ListView l, View v, int position, long id)
    {
        String filter = (getListAdapter().getItem(position) != null) ? String.valueOf(getListAdapter().getItem(position)) : "";
        mListener.onQueryTextSubmit(filter);
    }

    /*
        PRIVATE METHODS
     */

    /**
     * Initialization of the fragment
     */
    private void init()
    {
        DAOService daoService = new DAOService(getContext());
        PreferencesApp preferencesApp = new PreferencesApp(getContext());
        String strLanguageName = preferencesApp.getUserLanguage().split("_")[0];
        if (mPopulateAsyncTask == null)
        {
            mPopulateAsyncTask = daoService.createPopulateTagsCall(this, strLanguageName);
            mPopulateAsyncTask.execute();
        }
    }

    /**
     * Initialization of the search view
     */
    private void initSearchView(MenuItem searchItem)
    {
        // Get the SearchView and set the searchable configuration
        SearchView searchView = (SearchView) MenuItemCompat.getActionView(searchItem);
        SearchManager searchManager = (SearchManager) getContext().getSystemService(Context.SEARCH_SERVICE);
        searchView.setSearchableInfo(searchManager.getSearchableInfo(getActivity().getComponentName()));
        searchView.setQueryHint(getString(R.string.hint_search_send_by));
        searchView.setFocusable(true);
        searchView.requestFocus();
        searchView.requestFocusFromTouch();
        searchView.setOnQueryTextListener(this);
        searchView.onActionViewExpanded();
    }

    /**
     * Get height of the tag listview
     *
     * @return int
     */
    private int getListViewItemsHeight()
    {
        if (getListAdapter().getCount() == 0)
            return 0;
        View mView = getListAdapter().getView(0, null, getListView());
        mView.measure(
                View.MeasureSpec.makeMeasureSpec(0, View.MeasureSpec.UNSPECIFIED),
                View.MeasureSpec.makeMeasureSpec(0, View.MeasureSpec.UNSPECIFIED));

        return getListAdapter().getCount() * mView.getMeasuredHeight();
    }

    /**
     * Detect focused view and hide keyboard if it is shown
     */
    private void hideKeyboard()
    {
        // Check if no view has focus:
        View view = getActivity().getCurrentFocus();
        if (view != null) {
            InputMethodManager imm = (InputMethodManager) getActivity().getSystemService(Context.INPUT_METHOD_SERVICE);
            imm.hideSoftInputFromWindow(view.getWindowToken(), 0);
        }
    }

    /*
        IMPLEMENTS
        ViewTreeObserver.OnGlobalLayoutListener
     */

    @Override
    public void onGlobalLayout()
    {
        int height = getView().getHeight();
        Rect rect = new Rect();
        getView().getWindowVisibleDisplayFrame(rect);
        int displayHeight = rect.bottom - rect.top;
        boolean bKeyboardHiddenTemp = ((double) displayHeight / height > 0.8);
        boolean bMinHeightExceededTemp = (getListView() != null && getListAdapter() != null) ? getListViewItemsHeight() < displayHeight - mReduceHeight : false;

        if (bKeyboardHiddenTemp != mbKeyboardHidden)
        {
            mbKeyboardHidden = bKeyboardHiddenTemp;

            if (!mbKeyboardHidden && !bMinHeightExceededTemp)
            {
                mbMinHeightExceeded = bMinHeightExceededTemp;
                FrameLayout.LayoutParams mParam = new FrameLayout.LayoutParams(FrameLayout.LayoutParams.MATCH_PARENT, displayHeight - mReduceHeight);
                getListView().setLayoutParams(mParam);
                getListView().requestLayout();
            }
            else
            {
                mReduceHeight = displayHeight - height;
                FrameLayout.LayoutParams mParam = new FrameLayout.LayoutParams(FrameLayout.LayoutParams.MATCH_PARENT, FrameLayout.LayoutParams.WRAP_CONTENT);
                getListView().setLayoutParams(mParam);
                getListView().requestLayout();
            }
        }
        else if (bMinHeightExceededTemp != mbMinHeightExceeded)
        {
            mbMinHeightExceeded = bMinHeightExceededTemp;
            FrameLayout.LayoutParams mParam = new FrameLayout.LayoutParams(FrameLayout.LayoutParams.MATCH_PARENT, FrameLayout.LayoutParams.WRAP_CONTENT);
            getListView().setLayoutParams(mParam);
            getListView().requestLayout();
        }
    }

    /*
        IMPLEMENTS
        View.OnClickListener
     */

    @Override
    public void onClick(View v)
    {
        getActivity().onBackPressed();
    }

    /*
        IMPLEMENTS
        SearchView.OnQueryTextListener
     */

    @Override
    public boolean onQueryTextSubmit(String query)
    {
        mListener.onQueryTextSubmit(query + "%");
        return true;
    }

    @Override
    public boolean onQueryTextChange(String newText)
    {
        if (mGetTagsAsyncTask !=null)
            mGetTagsAsyncTask.cancel(true);
        DAOService daoService = new DAOService(getContext());
        mGetTagsAsyncTask = daoService.createGetTagsCall(this);
        mGetTagsAsyncTask.execute(newText);

        return true;
    }

    /*
        IMPLEMENTS
         DAOService.OnPopulateTagsListener
     */

    @Override
    public void onPostPopulateTagsAsyncTask(DTOResponseSimple response)
    {
        mPopulateAsyncTask = null;

        if (mGetTagsAsyncTask != null)
            mGetTagsAsyncTask.cancel(true);

        if (getContext() != null)
        {
            DAOService daoService = new DAOService(getContext());
            mGetTagsAsyncTask = daoService.createGetTagsCall(this);
            mGetTagsAsyncTask.execute();
        }
    }

    /*
        IMPLEMENTS
        DAOService.OnGetTagsListener
     */

    @Override
    public void onPostGetTagsAsyncTask(DTOResponseTagList response)
    {
        if ( response.getTags() != null && getContext() != null)
        {
            ArrayAdapter<String> adapter = new TagsFilterAdapter(
                    getContext(), android.R.layout.simple_list_item_1,
                    response.getTags());
            setListAdapter(adapter);
        }
    }


}
