package com.ejgv.gvi.adapters;

import android.content.Context;
import android.support.v7.widget.RecyclerView;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import com.ejgv.gvi.R;
import com.ejgv.gvi.dmo.DMOContact;
import com.ejgv.gvi.utils.UtilsApp;

import java.util.ArrayList;
import java.util.List;

/**
 * Created by ivanbatura on 27/4/16.
 */
public class ContactsListAdapter extends RecyclerView.Adapter<ContactsListAdapter.ViewHolder>
{

    private static final String TAG = "ContactsListAdapter";

    private Context mContext;
    private List<DMOContact> mDataSet;

    private ContactsListAdapterListener mListener;

    public interface ContactsListAdapterListener
    {
        void onContactListAdapterItemClicked(View view, int position);
    }


    // BEGIN_INCLUDE(recyclerViewSampleViewHolder)
    /**
     * Provide a reference to the type of views that you are using (custom ViewHolder)
     */
    public static class ViewHolder extends RecyclerView.ViewHolder implements View.OnClickListener
    {
        private TextView mCityTxt;
        private TextView mAddressLine1Txt;
        private TextView mAddressLine2Txt;
        private ContactsListAdapterListener mListener;

        public ViewHolder(View v, ContactsListAdapterListener listener)
        {
            super(v);
            v.setOnClickListener(this);
            mCityTxt = (TextView) v.findViewById(R.id.rowItemContact_city_txt);
            mAddressLine1Txt = (TextView) v.findViewById(R.id.rowItemContact_addressLine1_txt);
            mAddressLine2Txt = (TextView) v.findViewById(R.id.rowItemContact_addressLine2_txt);
            mListener = listener;
        }

        public TextView getmCityTxt() {
            return mCityTxt;
        }

        public TextView getmAddressLine1Txt() {
            return mAddressLine1Txt;
        }

        public TextView getmAddressLine2Txt() {
            return mAddressLine2Txt;
        }

        @Override
        public void onClick(View v)
        {
            if ( mListener != null)
                mListener.onContactListAdapterItemClicked(v, getAdapterPosition());
        }
    }

    // END_INCLUDE(recyclerViewSampleViewHolder)

    @Override
    public int getItemCount()
    {
        return mDataSet.size();
    }

    /**
     * Constructor
     * Initialize the dataset of the Adapter.
     *
     * @param context
     * @param dataSet DTOMessage[] containing the data to populate views to be used by RecyclerView.
     * @param listener
     */
    public ContactsListAdapter(Context context, ArrayList<DMOContact> dataSet, ContactsListAdapterListener listener)
    {
        mContext = context;
        mDataSet = dataSet;
        mListener = listener;
    }

    // BEGIN_INCLUDE(recyclerViewOnCreateViewHolder)
    // Create new views (invoked by the layout manager)
    @Override
    public ViewHolder onCreateViewHolder(ViewGroup viewGroup, int viewType)
    {
        // Create a new view.
        View v = LayoutInflater.from(viewGroup.getContext())
                .inflate(R.layout.row_item_contact, viewGroup, false);

        return new ViewHolder(v, mListener);
    }
    // END_INCLUDE(recyclerViewOnCreateViewHolder)

    // BEGIN_INCLUDE(recyclerViewOnBindViewHolder)
    // Replace the contents of a view (invoked by the layout manager)
    @Override
    public void onBindViewHolder(ViewHolder viewHolder, int position)
    {
        // Get element from the dataset at this position and replace the contents of the view
        // with that element
        DMOContact contact = mDataSet.get(position);
        viewHolder.getmCityTxt().setText(UtilsApp.getTextLocale(mContext, contact.getName()));
        viewHolder.getmAddressLine1Txt().setText(UtilsApp.getTextLocale(mContext, contact.getAddressLine1()));
        viewHolder.getmAddressLine2Txt().setText(UtilsApp.getTextLocale(mContext, contact.getAddressLine2()));
    }

    // END_INCLUDE(recyclerViewOnBindViewHolder)
}
