package com.ejgv.gvi.adapters;

import android.content.Context;
import android.support.v7.widget.RecyclerView;
import android.text.format.DateUtils;
import android.util.SparseBooleanArray;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.CheckBox;
import android.widget.RelativeLayout;
import android.widget.TextView;

import com.ejgv.gvi.R;
import com.ejgv.gvi.dao.dto.DTOMessage;
import com.ejgv.gvi.utils.Utils;
import com.ejgv.gvi.utils.UtilsApp;
import com.ejgv.gvi.utils.UtilsLog;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * Created by Batura Mobile Solutions (Koldo Calvo).
 */
/**
 * Provide views to RecyclerView with data from mDataSet.
 */
public class MessageListAdapter extends RecyclerView.Adapter<MessageListAdapter.ViewHolder>
{
    private static final String TAG = "MessageListAdapter";

    private Context mContext;
    private List<DTOMessage> mDataSet;
    private SparseBooleanArray mSelectedItems;

    private MessageListAdapterListener mListener;

    public interface MessageListAdapterListener
    {
        void onMessageListAdapterItemClicked(View view, int position);
        void onMessageListAdapterItemLongClicked(View view, int position);
    }

    // BEGIN_INCLUDE(recyclerViewSampleViewHolder)
    /**
     * Provide a reference to the type of views that you are using (custom ViewHolder)
     */
    public static class ViewHolder extends RecyclerView.ViewHolder implements View.OnClickListener, View.OnLongClickListener
    {
        private TextView mActionDeleteTxt;
        private RelativeLayout mRemoveableView;
        private final TextView txtDate;
        private final TextView txtTag;
        private final TextView txtPublicText;
        private final CheckBox chkSelection;
        private MessageListAdapterListener mListener;

        public ViewHolder(View v, MessageListAdapterListener listener)
        {
            super(v);
            v.setOnClickListener(this);
            v.setOnLongClickListener(this);
            mActionDeleteTxt = (TextView) v.findViewById(R.id.rowItemMessageRemoveable_noRemoveableActionDelete_txt);
            mRemoveableView = (RelativeLayout) v.findViewById(R.id.rowItemMessage_removeable_lyt);
            txtDate = (TextView) v.findViewById(R.id.rowItemMessage_date_txt);
            txtTag = (TextView) v.findViewById(R.id.rowItemMessage_tag_txt);
            txtPublicText = (TextView) v.findViewById(R.id.rowItemMessage_publicText_txt);
            chkSelection = (CheckBox) v.findViewById(R.id.rowItemMessage_selection_chk);
            mListener = listener;
        }

        public TextView getNoSwipableView() {
            return mActionDeleteTxt;
        }

        public View getSwipableView() {
            return mRemoveableView;
        }

        public TextView getTxtDate() {
            return txtDate;
        }

        public TextView getTxtTag() {
            return txtTag;
        }

        public TextView getTxtPublicText() {
            return txtPublicText;
        }

        public CheckBox getChkSelection() {
            return chkSelection;
        }

        @Override
        public void onClick(View v)
        {
            chkSelection.toggle();
            if ( mListener != null)
                mListener.onMessageListAdapterItemClicked(v, getAdapterPosition());
        }

        @Override
        public boolean onLongClick(View v)
        {
            chkSelection.toggle();
            if ( mListener != null)
                mListener.onMessageListAdapterItemLongClicked(v, getAdapterPosition());
            return true;
        }
    }

    // END_INCLUDE(recyclerViewSampleViewHolder)

    /**
     * Constructor
     * Initialize the dataset of the Adapter.
     *
     * @param context
     * @param dataSet DTOMessage[] containing the data to populate views to be used by RecyclerView.
     * @param listener
     */
    public MessageListAdapter(Context context, DTOMessage[] dataSet, MessageListAdapterListener listener)
    {
        mContext = context;
        mDataSet = new ArrayList<>(Arrays.asList(dataSet));
        mListener = listener;
    }

    // BEGIN_INCLUDE(recyclerViewOnCreateViewHolder)
    // Create new views (invoked by the layout manager)
    @Override
    public ViewHolder onCreateViewHolder(ViewGroup viewGroup, int viewType)
    {
        // Create a new view.
        View v = LayoutInflater.from(viewGroup.getContext())
                .inflate(R.layout.row_item_message, viewGroup, false);

        return new ViewHolder(v, mListener);
    }
    // END_INCLUDE(recyclerViewOnCreateViewHolder)

    // BEGIN_INCLUDE(recyclerViewOnBindViewHolder)
    // Replace the contents of a view (invoked by the layout manager)
    @Override
    public void onBindViewHolder(ViewHolder viewHolder, int position)
    {
        // Get element from the dataset at this position and replace the contents of the view
        // with that element
        DTOMessage item = mDataSet.get(position);

        CharSequence chsDate = DateUtils.getRelativeTimeSpanString(mContext, item.getTs(), false);

        if (item.getPrivateMsg() != null && UtilsApp.getTextLocale(mContext, item.getPrivateMsg().getTag()) != null)
        {
            viewHolder.getTxtTag().setText(UtilsApp.getTextLocale(mContext, item.getPrivateMsg().getTag()));
            viewHolder.getTxtTag().setVisibility(View.VISIBLE);
        }
        else
            viewHolder.getTxtTag().setVisibility(View.GONE);


        String strPublicText = item.getPublicText();
        if (item.getTsRead() == 0)
        {
            viewHolder.getTxtDate().setText(Utils.bold(chsDate));
            viewHolder.getTxtPublicText().setText(Utils.bold(strPublicText));
        }
        else
        {
            viewHolder.getTxtDate().setText(chsDate);
            viewHolder.getTxtPublicText().setText(strPublicText);
        }

        if (mSelectedItems != null)
        {
            viewHolder.getChkSelection().setVisibility(View.VISIBLE);
            viewHolder.getChkSelection().setChecked(mSelectedItems.get(position, false));
        }
        else
            viewHolder.getChkSelection().setVisibility(View.INVISIBLE);
    }

    // END_INCLUDE(recyclerViewOnBindViewHolder)

    @Override
    public int getItemCount()
    {
        return mDataSet.size();
    }

    /*
        PUBLIC METHODS
     */

    public DTOMessage getItem(int position)
    {
        return mDataSet.get(position);
    }

    public void add(int position, DTOMessage message)
    {
        mDataSet.add(position, message);
        notifyItemInserted(position);
    }

    public void startSelection()
    {
        mSelectedItems = new SparseBooleanArray(0);
        notifyDataSetChanged();
    }

    public void remove(String trans_id)
    {
        for (int i=0; i<mDataSet.size(); i++)
        {
            if(mDataSet.get(i).getIdTrans() != null && mDataSet.get(i).getIdTrans().equals(trans_id))
            {
                mDataSet.remove(i);
                notifyItemRemoved(i);
            }
        }
    }
    public void removeRange(int start, int end)
    {
        mDataSet.subList(start, end).clear();
        notifyItemRangeRemoved(start, end);
    }


    public void removeAt(int position)
    {
        UtilsLog.d("****", String.valueOf(position));

        if (mSelectedItems != null && mSelectedItems.get(position, false))
        {
            mSelectedItems.delete(position);
        }

        if (mDataSet.get(position) != null)
            mDataSet.remove(position);

        notifyItemRemoved(position);
    }

    public void toggleSelection(int position)
    {
        if (mSelectedItems != null)
        {
            if (mSelectedItems.get(position, false))
                mSelectedItems.delete(position);
            else
                mSelectedItems.put(position, true);
        }
    }

    public void clearSelections()
    {
        if (mSelectedItems != null)
        {
            mSelectedItems.clear();
            mSelectedItems = null;
        }
        notifyDataSetChanged();
    }

    public int getSelectedItemCount()
    {
        return (mSelectedItems != null) ? mSelectedItems.size() : 0;
    }

    public List<Integer> getSelectedItems()
    {
        List<Integer> items = new ArrayList<>();
        int len = (mSelectedItems != null) ? mSelectedItems.size() : 0;
        for (int i=0; i<len; i++)
            items.add(mSelectedItems.keyAt(i));
        return items;
    }

    public void deleteSelection(int position)
    {
        if (mSelectedItems != null && mSelectedItems.get(position, false))
                mSelectedItems.delete(position);
    }

    public void updateItemTsRead(String trans_id, long tsRead)
    {
        for (int i=0, len=mDataSet.size(); i<len; i++)
        {
            if (mDataSet.get(i).getIdTrans().equals(trans_id))
            {
                mDataSet.get(i).setTsRead(tsRead);
                notifyItemChanged(i);
                break;
            }
        }
    }
}