package com.ejgv.gvi.dao;

import android.database.sqlite.SQLiteOpenHelper;
import android.os.AsyncTask;

import com.ejgv.gvi.dao.dto.DTOMessage;
import com.ejgv.gvi.dao.dto.DTOResponseGetMessages;
import com.ejgv.gvi.restapipns.dto.DTOPnsMessage;
import com.ejgv.gvi.utils.UtilsApp;
import com.ejgv.gvi.utils.UtilsLog;

import java.util.ArrayList;

/**
 * Created by Batura Mobile Solutions (Koldo Calvo).
 */
class AddMessagesAsyncTask extends AsyncTask<Object, Void, DTOResponseGetMessages>
{
    private static final String TAG = "AddMessagesAsyncTask";

    private SQLiteOpenHelper mDatabaseHelper;
    private DAOService.OnAddMessagesListener mListener;

    private DTOPnsMessage[] mPnsMessages;
    private int mMessagesMaxLimit;

    /**
     *  @param databaseHelper
     * @param listener
     * @param pnsMessages
     */
    public AddMessagesAsyncTask(SQLiteOpenHelper databaseHelper, DAOService.OnAddMessagesListener listener, DTOPnsMessage[] pnsMessages, int messagesMaxLimit)
    {
        mDatabaseHelper = databaseHelper;
        mListener = listener;
        mPnsMessages = pnsMessages;
        mMessagesMaxLimit = messagesMaxLimit;
    }

    /**
     * AddMessagesAsyncTask UI background processing
     *
     * @param params
     * @return DTOResponseSimple
     */
    @Override
    protected DTOResponseGetMessages doInBackground(Object... params)
    {
        DTOResponseGetMessages objResponse = new DTOResponseGetMessages();
        DAOMessage daoMessage = new DAOMessage(mDatabaseHelper);
        try
        {
            if (mPnsMessages != null && mPnsMessages.length > 0)
            {
                // DDBB call
                long[] insert_ids = daoMessage.insert(mPnsMessages);

                ArrayList<DTOMessage> messages = new ArrayList<>();

                int cont = 0;
                for (int i=0,len=insert_ids.length; i<len; i++)
                {
                    if (insert_ids[i] != -1)
                    {
                        messages.add(cont, new DTOMessage(mPnsMessages[i]));
                        cont++;
                    }
                }
                objResponse.setResult(DAOService.RESPONSE_RESULT_OK);
                objResponse.setMessages(messages.toArray(new DTOMessage[messages.size()]));
            }
            else
            {
                objResponse.setResult(DAOService.RESPONSE_RESULT_ERROR);
                objResponse.setCode(DAOService.DDBB_ERROR_CODE + DAOService.DDBB_ERROR_EMPTY_DAO);
            }
        }
        catch (Exception e)
        {
            UtilsLog.d(TAG, "Exception in doInBackground", e);
            UtilsApp.sendTraceToServer(UtilsApp.createStandardAddTraceParams(getClass().getName(), "doInBackground", e));
            objResponse.setResult(DAOService.RESPONSE_RESULT_ERROR);
            objResponse.setCode(DAOService.DDBB_ERROR_CODE + DAOService.DDBB_ERROR);

        } finally
        {
            return objResponse;
        }
    }

    /**
     * AddMessagesAsyncTask UI main processing after its execution
     *
     * @param response
     */
    @Override
    protected void onPostExecute(DTOResponseGetMessages response)
    {
        if (mListener != null && !isCancelled())
            mListener.onPostAddMessagesAsyncTask(response);
    }

}