package com.ejgv.gvi.dao;

import android.database.Cursor;
import android.database.sqlite.SQLiteOpenHelper;
import android.database.sqlite.SQLiteQueryBuilder;

import com.ejgv.gvi.dao.dto.DTOMessage;
import com.ejgv.gvi.restapipns.dto.DTOPnsMsgPrivate;
import com.ejgv.gvi.restapipns.dto.DTOPnsMsgPrivateResponse;
import com.ejgv.gvi.restapipns.dto.DTOPnsMsgPrivateResponseValues;
import com.ejgv.gvi.restapipns.dto.DTOText;

/**
 * Created by koldo on 20/01/16.
 */
public class DAOJoin implements DAOService.DAOJoinInterface
{
    private SQLiteOpenHelper mDatabaseHelper;

    public DAOJoin(SQLiteOpenHelper databaseHelper)
    {
        mDatabaseHelper = databaseHelper;
    }

    @Override
    public DTOMessage selectPrivateMessage(String trans_id)
    {
        String tables = new StringBuilder()
                .append(TableMessage.Table.NAME)
                .append(" NATURAL LEFT OUTER JOIN ")
                .append(TableMsgResponsed.Table.NAME)
                .append(" NATURAL LEFT OUTER JOIN ")
                .append(TableMsgResponseValue.Table.NAME)
                .toString();

        String[] projection = {
                TableMessage.Table.NAME +"."+ TableMessage.Columns.ID_TRANS,
                TableMessage.Table.NAME +"."+ TableMessage.Columns.TS_READ,
                TableMsgResponsed.Table.NAME +"."+ TableMsgResponsed.Columns.TS_RESPONSE,
                TableMsgResponsed.Table.NAME +"."+ TableMsgResponsed.Columns.CODE,
                TableMsgResponseValue.Table.NAME +"."+ TableMsgResponseValue.Columns.VALUE_EU,
                TableMsgResponseValue.Table.NAME +"."+ TableMsgResponseValue.Columns.VALUE_ES,
                TableMessage.Table.NAME +"."+ TableMessage.Columns.PRIVATE_EU,
                TableMessage.Table.NAME +"."+ TableMessage.Columns.PRIVATE_ES
        };

        String selection = new StringBuilder()
                .append(TableMessage.Table.NAME).append(".").append(TableMessage.Columns.ID_TRANS).append("=? ")
                .toString();

        String[] selectionArgs = { trans_id };

        SQLiteQueryBuilder qBuilder = new SQLiteQueryBuilder();

        qBuilder.setTables(tables);

        Cursor cursor = qBuilder.query(
                mDatabaseHelper.getReadableDatabase(),
                projection,
                selection,
                selectionArgs,
                null,
                null,
                null,
                "1"
        );

        if (cursor == null)
        {
            return null;
        }
        else if (!cursor.moveToFirst())
        {
            cursor.close();
            return null;
        }
        else
        {
            DTOMessage message = new DTOMessage();
            while (cursor.isAfterLast() == false)
            {
                message.setIdTrans(cursor.getString(0));
                message.setTsRead(cursor.getLong(1));
                message.setTsResponse(cursor.getLong(2));
                message.setResponseCode(cursor.getString(3));
                DTOText responseText = new DTOText();
                responseText.setEu(cursor.getString(4));
                responseText.setEs(cursor.getString(5));
                message.setResponseText(responseText);
                DTOText privateText = new DTOText();
                privateText.setEu(cursor.getString(6));
                privateText.setEs(cursor.getString(7));
                DTOPnsMsgPrivate privateMsg = new DTOPnsMsgPrivate();
                privateMsg.setText(privateText);
                message.setPrivateMsg(privateMsg);

                cursor.moveToNext();
            }
            cursor.close();
            return message;
        }
    }

    @Override
    public DTOMessage selectMsgResponseValues(String trans_id) throws Exception
    {

        String tables = new StringBuilder()
                .append(TableMessage.Table.NAME)
                .append(", ")
                .append(TableMsgResponseValue.Table.NAME)
                .toString();

        String[] projection = {
                TableMessage.Table.NAME +"."+ TableMessage.Columns.ID_TRANS,
                TableMessage.Table.NAME +"."+ TableMessage.Columns.ALIAS,
                TableMsgResponseValue.Table.NAME +"."+ TableMsgResponseValue.Columns.CODE,
                TableMsgResponseValue.Table.NAME +"."+ TableMsgResponseValue.Columns.VALUE_EU,
                TableMsgResponseValue.Table.NAME +"."+ TableMsgResponseValue.Columns.VALUE_ES
        };

        String selection = new StringBuilder()
                .append(TableMessage.Table.NAME).append(".").append(TableMessage.Columns.ID_TRANS)
                .append("=")
                .append(TableMsgResponseValue.Table.NAME).append(".").append(TableMsgResponseValue.Columns.ID_TRANS)
                .append(" AND ")
                .append(TableMessage.Table.NAME).append(".").append(TableMessage.Columns.ID_TRANS).append("=? ")
                .toString();

        String[] selectionArgs = { trans_id };

        SQLiteQueryBuilder qBuilder = new SQLiteQueryBuilder();

        qBuilder.setTables(tables);

        Cursor cursor = qBuilder.query(
                mDatabaseHelper.getReadableDatabase(),
                projection,
                selection,
                selectionArgs,
                null,
                null,
                null
        );

        if (cursor == null)
        {
            return null;
        }
        else if (!cursor.moveToFirst())
        {
            cursor.close();
            return null;
        }
        else
        {
            DTOMessage message = new DTOMessage();
            message.setIdTrans(cursor.getString(0));
            DTOPnsMsgPrivate msgPrivate = new DTOPnsMsgPrivate();
            DTOPnsMsgPrivateResponse msgPrivateResponse = new DTOPnsMsgPrivateResponse();
            msgPrivateResponse.setAlias(cursor.getString(1));
            DTOPnsMsgPrivateResponseValues[] responseValues = new DTOPnsMsgPrivateResponseValues[cursor.getCount()];

            int cont = 0;
            while (cursor.isAfterLast() == false)
            {
                DTOPnsMsgPrivateResponseValues value = new DTOPnsMsgPrivateResponseValues();
                value.setCode(cursor.getString(2));
                value.setEu(cursor.getString(3));
                value.setEs(cursor.getString(4));
                responseValues[cont] = value;

                cursor.moveToNext();
                cont++;
            }

            msgPrivateResponse.setValues(responseValues);
            msgPrivate.setResponse(msgPrivateResponse);
            message.setPrivateMsg(msgPrivate);

            cursor.close();
            return message;
        }
    }
}