package com.ejgv.gvi.dao;

import android.content.ContentValues;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;
import android.text.TextUtils;

import com.ejgv.gvi.dao.dto.DTOMessage;
import com.ejgv.gvi.restapipns.dto.DTOPnsMessage;
import com.ejgv.gvi.restapipns.dto.DTOPnsMsgPrivate;
import com.ejgv.gvi.restapipns.dto.DTOText;

import java.util.Date;

/**
 * Created by koldo on 22/12/15.
 */
public class DAOMessage implements DAOService.DAOMessageInterface
{
    private SQLiteOpenHelper mDatabaseHelper;

    public DAOMessage(SQLiteOpenHelper databaseHelper)
    {
        mDatabaseHelper = databaseHelper;
    }

    /**
     * Insert new message
     *
     * @param pnsMessage
     * @return inserted id
     * @throws Exception
     */
    @Override
    public long insert(DTOPnsMessage pnsMessage) throws Exception
    {
        ContentValues values = new ContentValues();
        values.put(TableMessage.Columns.ID_TRANS, pnsMessage.getIdTrans());
        values.put(TableMessage.Columns.PUBLIC, pnsMessage.getPublicText());
        if (pnsMessage.getPrivateMsg() != null )
        {
            if (pnsMessage.getPrivateMsg().getTag() != null)
            {
                values.put(TableMessage.Columns.TAG_EU, pnsMessage.getPrivateMsg().getTag().getEu());
                values.put(TableMessage.Columns.TAG_ES, pnsMessage.getPrivateMsg().getTag().getEs());
            }
            values.put(TableMessage.Columns.PRIVATE_EU, pnsMessage.getPrivateMsg().getText().getEu());
            values.put(TableMessage.Columns.PRIVATE_ES, pnsMessage.getPrivateMsg().getText().getEs());
            if (pnsMessage.getPrivateMsg().getResponse() != null )
                values.put(TableMessage.Columns.ALIAS, pnsMessage.getPrivateMsg().getResponse().getAlias());
        }

        values.put(TableMessage.Columns.TS, pnsMessage.getTs());


        long insertId = mDatabaseHelper.getWritableDatabase().insertWithOnConflict(
                TableMessage.Table.NAME,
                null,
                values,
                SQLiteDatabase.CONFLICT_IGNORE
        );
        return insertId;
    }

    /**
     * Insert new messages
     *
     * @param pnsMessages
     * @return inserted ids
     * @throws Exception
     */
    @Override
    public long[] insert(DTOPnsMessage[] pnsMessages) throws Exception
    {
        long[] alng_insertId = new long[pnsMessages.length];

        DAOMsgResponseValue daoMsgResponseValue = new DAOMsgResponseValue(mDatabaseHelper);
        for (int i=0, len=alng_insertId.length; i<len; i++)
        {
            alng_insertId[i] = insert(pnsMessages[i]);
            if (pnsMessages[i].getPrivateMsg() != null && pnsMessages[i].getPrivateMsg().getResponse() != null)
                daoMsgResponseValue.insert(pnsMessages[i].getIdTrans(), pnsMessages[i].getPrivateMsg().getResponse().getValues());
        }

        return alng_insertId;
    }

    /**
     * set message read timestamp
     *
     * @param trans_id
     * @return
     * @throws Exception
     */
    @Override
    public long updateTsRead(String trans_id) throws Exception
    {
        ContentValues values = new ContentValues();
        values.put(TableMessage.Columns.TS_READ, new Date().getTime());


        String selection = new StringBuilder()
                .append(TableMessage.Columns.ID_TRANS)
                .append("=?")
                .toString();

        String[] selectionArgs = {trans_id };

        long updateId = mDatabaseHelper.getWritableDatabase().update(
                TableMessage.Table.NAME,
                values,
                selection,
                selectionArgs
        );
        return updateId;
    }

    /**
     *
     *
     * @return
     * @throws Exception
     */
    @Override
    public long delete() throws Exception
    {
        long deleteId = mDatabaseHelper.getWritableDatabase().delete(
                TableMessage.Table.NAME,
                null,
                null
        );
        return deleteId;
    }

    /**
     *
     * @param trans_id
     * @return
     * @throws Exception
     */
    @Override
    public long delete(String trans_id) throws Exception
    {
        String selection = new StringBuilder()
                .append(TableMessage.Columns.ID_TRANS)
                .append(" =?")
                .toString();

        String[] selectionArgs = { trans_id };

        long deleteId = mDatabaseHelper.getWritableDatabase().delete(
                TableMessage.Table.NAME,
                selection,
                selectionArgs
        );
        return deleteId;
    }

    /**
     *
     * @param trans_ids
     * @return
     * @throws Exception
     */
    @Override
    public long delete(String[] trans_ids) throws Exception
    {
        String selection = new StringBuilder()
                .append(TableMessage.Columns.ID_TRANS)
                .append(" IN ('")
                .append(TextUtils.join(",", trans_ids).replace(",", "','"))
                .append("')")
                .toString();

        long deleteId = mDatabaseHelper.getWritableDatabase().delete(
                TableMessage.Table.NAME,
                selection,
                null
        );
        return deleteId;
    }

    /**
     *
     * @param limit
     * @return
     * @throws Exception
     */
    @Override
    public long deleteOver(int limit) throws Exception
    {
        String selection = new StringBuilder()
                .append(TableMessage.Columns.ID_TRANS)
                .append(" IN (")
                .append(" SELECT ").append(TableMessage.Columns.ID_TRANS).append(" FROM ").append(TableMessage.Table.NAME)
                .append(" ORDER BY ").append(TableMessage.Columns.TS).append(" DESC LIMIT -1 OFFSET ?")
                .append(")")
                .toString();

        String[] selectionArgs = { String.valueOf(limit) };

        long deleteId = mDatabaseHelper.getWritableDatabase().delete(
                TableMessage.Table.NAME,
                selection,
                selectionArgs
        );
        return deleteId;
    }

    /**
     *
     * @return
     * @throws Exception
     */
    @Override
    public DTOMessage[] selectMessages() throws Exception
    {
        String orderBy = new StringBuilder()
                .append(TableMessage.Columns.TS)
                .append(" DESC")
                .toString();

        return query(null, null, orderBy);
    }

    /**
     *
     * @return
     * @throws Exception
     */
    @Override
    public DTOMessage[] selectMessagesFrom(String trans_id, long ts) throws Exception
    {

        String selection = new StringBuilder()
                .append(TableMessage.Columns.ID_TRANS).append("!=?")
                .append(" AND ")
                .append(TableMessage.Columns.TS).append(">=?")
                .toString();

        String[] selectionArgs = { trans_id, String.valueOf(ts)};

        String orderBy = new StringBuilder()
                .append(TableMessage.Columns.TS)
                .append(" DESC")
                .toString();

        return query(selection, selectionArgs, orderBy);
    }

    /**
     *
     * @param query
     * @return
     * @throws Exception
     */
    @Override
    public DTOMessage[] selectMessagesTagsLike(String query) throws Exception
    {
        String selection;
        String[] selectionArgs = null;

        if (query != null && !query.isEmpty())
        {
            selection = new StringBuilder()
                    .append(TableMessage.Columns.TAG_EU)
                    .append(" LIKE ? OR ")
                    .append(TableMessage.Columns.TAG_ES)
                    .append(" LIKE ? ")
                    .toString();
            selectionArgs = new String[]{ query, query };
        }
        else
        {
            selection = new StringBuilder()
                    .append(TableMessage.Columns.TAG_EU)
                    .append(" IS NULL OR ")
                    .append(TableMessage.Columns.TAG_ES)
                    .append(" IS NULL")
                    .toString();
        }

        String orderBy = new StringBuilder()
                .append(TableMessage.Columns.TAG_EU)
                .append(", ")
                .append(TableMessage.Columns.TAG_ES)
                .toString();

        return query(selection, selectionArgs, orderBy);
    }

    private DTOMessage[] query(String selection, String[] selectionArgs, String orderBy) throws Exception
    {
        String[] projection = {
                TableMessage.Columns.ID_TRANS,
                TableMessage.Columns.PUBLIC,
                TableMessage.Columns.TAG_EU,
                TableMessage.Columns.TAG_ES,
                TableMessage.Columns.TS,
                TableMessage.Columns.TS_READ
        };

        Cursor cursor = mDatabaseHelper.getReadableDatabase().query(
                TableMessage.Table.NAME,
                projection,
                selection,
                selectionArgs,
                null,
                null,
                orderBy
        );

        if (cursor == null)
        {
            return null;
        }
        else if (!cursor.moveToFirst())
        {
            cursor.close();
            return new DTOMessage[0];
        }
        else
        {
            DTOMessage[] messages = new DTOMessage[cursor.getCount()];
            int count = 0;
            while (cursor.isAfterLast() == false)
            {
                DTOMessage message = new DTOMessage();
                message.setIdTrans(cursor.getString(0));
                message.setPublicText(cursor.getString(1));
                DTOText tag = new DTOText();
                tag.setEu(cursor.getString(2));
                tag.setEs(cursor.getString(3));
                DTOPnsMsgPrivate privateMsg = new DTOPnsMsgPrivate();
                privateMsg.setTag(tag);
                message.setPrivateMsg(privateMsg);
                message.setTs(cursor.getLong(4));
                message.setTsRead(cursor.getLong(5));

                messages[count] = message;
                count++;

                cursor.moveToNext();
            }
            cursor.close();
            return messages;
        }
    }
}