package com.ejgv.gvi.dao;

import android.content.ContentValues;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;
import android.text.TextUtils;

import com.ejgv.gvi.dao.dto.DTOPnsMsgOperation;

/**
 * Created by koldo on 20/01/16.
 */
public class DAOMsgPendingOperation implements DAOService.DAOMsgPendingOperationInterface
{
    private SQLiteOpenHelper mDatabaseHelper;

    public DAOMsgPendingOperation(SQLiteOpenHelper databaseHelper)
    {
        mDatabaseHelper = databaseHelper;
    }

    /**
     *
     * @param msgOperation
     * @return
     * @throws Exception
     */
    @Override
    public long insert(DTOPnsMsgOperation msgOperation) throws Exception
    {
        ContentValues values = new ContentValues();
        values.put(TableMsgPendingOperation.Columns.ID_TRANS, msgOperation.getIdTrans());
        values.put(TableMsgPendingOperation.Columns.OPERATION, String.valueOf(msgOperation.getMsgOperation()));

        long insertId = mDatabaseHelper.getWritableDatabase().insert(
                TableMsgPendingOperation.Table.NAME,
                null,
                values
        );
        return insertId;
    }

    /**
     * Insert new messages operations
     * @param msgOperation
     * @param trans_ids
     * @return
     * @throws Exception
     */
    @Override
    public long[] insertOperations(char msgOperation, String[] trans_ids) throws Exception
    {
        long[] alng_insertId = new long[trans_ids.length];

        for (int i=0, len=alng_insertId.length; i<len; i++)
        {
            DTOPnsMsgOperation msgOpertion = new DTOPnsMsgOperation();
            msgOpertion.setIdTrans(trans_ids[i]);
            msgOpertion.setMsgOperation(msgOperation);
            alng_insertId[i] = insert(msgOpertion);
        }

        return alng_insertId;
    }

    /**
     *
     * @return
     * @throws Exception
     */
    @Override
    public long delete() throws Exception
    {
        long deleteId = mDatabaseHelper.getWritableDatabase().delete(
                TableMsgPendingOperation.Table.NAME,
                null,
                null
        );
        return deleteId;
    }

    /**
     * 
     * @param msgOperation
     * @return
     * @throws Exception
     */
    @Override
    public long deleteOperation(DTOPnsMsgOperation msgOperation) throws Exception
    {
        String selection = new StringBuilder()
                .append(TableMsgPendingOperation.Columns.ID_TRANS).append("=? ")
                .append("AND ")
                .append(TableMsgPendingOperation.Columns.OPERATION).append("=?")
                .toString();

        String[] selectionArgs = { msgOperation.getIdTrans(), String.valueOf(msgOperation.getMsgOperation())};

        long deleteId = mDatabaseHelper.getWritableDatabase().delete(
                TableMsgPendingOperation.Table.NAME,
                selection,
                selectionArgs
        );
        return deleteId;
    }

    /**
     *
     *
     * @param operation
     * @param trans_ids
     * @return
     * @throws Exception
     */
    @Override
    public long deleteOperations(char operation, String[] trans_ids) throws Exception
    {
        String selection = new StringBuilder()
                .append(TableMsgPendingOperation.Columns.ID_TRANS)
                .append(" IN ('")
                .append(TextUtils.join(",", trans_ids).replace(",", "','"))
                .append("')")
                .append(" AND ")
                .append(TableMsgPendingOperation.Columns.OPERATION).append("=?")
                .toString();

        String[] selectionArgs = { 
                String.valueOf(operation)
        };

        long deleteId = mDatabaseHelper.getWritableDatabase().delete(
                TableMsgPendingOperation.Table.NAME,
                selection,
                selectionArgs
        );
        return deleteId;
    }

    /**
     *
     * @return
     * @throws Exception
     */
    @Override
    public DTOPnsMsgOperation[] selectPendingOperations() throws Exception
    {
        return query(null, null, null);
    }

    private DTOPnsMsgOperation[] query(String selection, String[] selectionArgs, String orderBy) throws Exception
    {
        String[] projection = {
                TableMsgPendingOperation.Columns.ID_TRANS,
                TableMsgPendingOperation.Columns.OPERATION
        };

        Cursor cursor = mDatabaseHelper.getReadableDatabase().query(
                TableMsgPendingOperation.Table.NAME,
                projection,
                selection,
                selectionArgs,
                null,
                null,
                orderBy
        );

        if (cursor == null)
        {
            return null;
        }
        else if (!cursor.moveToFirst())
        {
            cursor.close();
            return new DTOPnsMsgOperation[0];
        }
        else
        {
            DTOPnsMsgOperation[] operations = new DTOPnsMsgOperation[cursor.getCount()];
            int count = 0;
            while (cursor.isAfterLast() == false)
            {
                DTOPnsMsgOperation operation = new DTOPnsMsgOperation();
                operation.setIdTrans(cursor.getString(0));
                operation.setMsgOperation(cursor.getString(1).charAt(0));

                operations[count] = operation;
                count++;

                cursor.moveToNext();
            }
            cursor.close();
            return operations;
        }
    }
}
