package com.ejgv.gvi.dao;

import android.database.sqlite.SQLiteOpenHelper;
import android.os.AsyncTask;

import com.ejgv.gvi.dao.dto.DTOMessage;
import com.ejgv.gvi.dao.dto.DTOResponseGetMessages;
import com.ejgv.gvi.utils.UtilsLog;
import com.google.gson.Gson;
import com.google.gson.GsonBuilder;

/**
 * Created by Batura Mobile Solutions (Koldo Calvo).
 */
class GetMessagesAsyncTask extends AsyncTask<Object, Void, DTOResponseGetMessages>
{
    private static final String TAG = "GetMessagesAsyncTask";

    private SQLiteOpenHelper mDatabaseHelper;
    private DAOService.OnGetMessagesListener mListener;

    private boolean mDeleteOverLimitFirst;
    private int mMessagesLimit;

    /**
     * Constructor
     *
     * @param databaseHelper
     * @param listener
     */
    public GetMessagesAsyncTask(SQLiteOpenHelper databaseHelper, DAOService.OnGetMessagesListener listener)
    {
        mDatabaseHelper = databaseHelper;
        mListener = listener;
    }

    public GetMessagesAsyncTask(SQLiteOpenHelper databaseHelper, DAOService.OnGetMessagesListener listener, boolean deleteOverLimitFirst, int messagesLimit)
    {
        mDatabaseHelper = databaseHelper;
        mListener = listener;
        mDeleteOverLimitFirst = deleteOverLimitFirst;
        mMessagesLimit = messagesLimit;
    }

    public static GetMessagesAsyncTask createGetMessagesAfterDeleteOnesOverLimit(SQLiteOpenHelper databaseHelper, DAOService.OnGetMessagesListener listener, int messagesLimit)
    {
        return new GetMessagesAsyncTask(databaseHelper, listener, true, messagesLimit);
    }

    /**
     * GetMessagesAsyncTask UI background processing
     *
     * @param params
     * @return DTOResponseSimple
     */
    @Override
    protected DTOResponseGetMessages doInBackground(Object... params)
    {
        DTOResponseGetMessages objResponse = new DTOResponseGetMessages();
        try
        {
            // DDBB call
            DAOMessage daoMessage = new DAOMessage(mDatabaseHelper);

            if (mDeleteOverLimitFirst)
                daoMessage.deleteOver(mMessagesLimit);

            DTOMessage[] messages;
            String query = (params != null && params.length > 0) ? String.valueOf(params[0]) : null;
            if (query != null)
                messages = daoMessage.selectMessagesTagsLike(query);
            else
                messages = daoMessage.selectMessages();

            objResponse.setResult(DAOService.RESPONSE_RESULT_OK);
            objResponse.setMessages(messages);

            Gson gson_p = new GsonBuilder().setPrettyPrinting().create();
            UtilsLog.d(TAG, gson_p.toJson(objResponse));
        }
        catch (Exception e)
        {
            objResponse.setResult(DAOService.RESPONSE_RESULT_ERROR);
            objResponse.setCode(DAOService.DDBB_ERROR_CODE + DAOService.DDBB_ERROR);
            e.printStackTrace();
        }
        finally
        {
            return objResponse;
        }
    }

    /**
     * GetMessagesAsyncTask UI main processing after its execution
     *
     * @param response
     */
    @Override
    protected void onPostExecute(DTOResponseGetMessages response)
    {
        if (mListener != null && !isCancelled())
            mListener.onPostGetMessagesAsyncTask(response);
    }

}