package com.ejgv.gvi.dao;

import android.database.sqlite.SQLiteOpenHelper;
import android.os.AsyncTask;

import com.ejgv.gvi.dao.dto.DTOMessage;
import com.ejgv.gvi.dao.dto.DTOResponseGetMessages;
import com.ejgv.gvi.utils.UtilsApp;
import com.ejgv.gvi.utils.UtilsLog;

/**
 * Created by Batura Mobile Solutions (Koldo Calvo).
 */
class GetMessagesFromAsyncTask extends AsyncTask<Object, Void, DTOResponseGetMessages>
{
    private static final String TAG = "GetMsgFromAsyncTask";

    private SQLiteOpenHelper mDatabaseHelper;
    private DAOService.OnGetMessagesFromListener mListener;

    private String mTransId;
    private long mTs;

    /**
     * Constructor
     *
     * @param databaseHelper
     * @param listener
     */
    public GetMessagesFromAsyncTask(SQLiteOpenHelper databaseHelper, DAOService.OnGetMessagesFromListener listener,
                                    String trans_id, long ts)
    {
        mDatabaseHelper = databaseHelper;
        mListener = listener;
        mTransId = trans_id;
        mTs = ts;
    }

    /**
     * GetMessagesAsyncTask UI background processing
     *
     * @param params
     * @return DTOResponseSimple
     */
    @Override
    protected DTOResponseGetMessages doInBackground(Object... params)
    {
        DTOResponseGetMessages objResponse = new DTOResponseGetMessages();
        try
        {
            // DDBB call
            DAOMessage daoMessage = new DAOMessage(mDatabaseHelper);
            DTOMessage[] messages;
            if (mTransId != null && mTs > 0)
                messages = daoMessage.selectMessagesFrom(mTransId, mTs);
            else
                messages = daoMessage.selectMessages();

            objResponse.setResult(DAOService.RESPONSE_RESULT_OK);
            objResponse.setMessages(messages);
        }
        catch (Exception e)
        {
            UtilsLog.d(TAG, "Exception in doInBackground", e);
            UtilsApp.sendTraceToServer(UtilsApp.createStandardAddTraceParams(getClass().getName(), "doInBackground", e));
            objResponse.setResult(DAOService.RESPONSE_RESULT_ERROR);
            objResponse.setCode(DAOService.DDBB_ERROR_CODE + DAOService.DDBB_ERROR);
        }
        finally
        {
            return objResponse;
        }
    }

    /**
     * GetMessagesAsyncTask UI main processing after its execution
     *
     * @param response
     */
    @Override
    protected void onPostExecute(DTOResponseGetMessages response)
    {
        if (mListener != null && !isCancelled())
            mListener.onPostGetMessagesFromAsyncTask(response);
    }

}