package com.ejgv.gvi.dao;

import android.database.sqlite.SQLiteOpenHelper;
import android.os.AsyncTask;

import com.ejgv.gvi.dao.dto.DTOMessage;
import com.ejgv.gvi.dao.dto.DTOResponseGetMessages;
import com.ejgv.gvi.utils.UtilsApp;
import com.ejgv.gvi.utils.UtilsLog;

/**
 * Created by Batura Mobile Solutions (Koldo Calvo).
 */
public class GetMsgResponseValuesAsyncTask extends AsyncTask<Object, Void, DTOResponseGetMessages>
{
    private static final String TAG = "GetMsgRespValAsyncTask";

    private SQLiteOpenHelper mDatabaseHelper;
    private DAOService.OnGetMsgResponseValuesListener mListener;

    private String mTransId;

    /**
     *
     * @param databaseHelper
     * @param listener
     */
    public GetMsgResponseValuesAsyncTask(SQLiteOpenHelper databaseHelper, DAOService.OnGetMsgResponseValuesListener listener,
                                         String trans_id)
    {
        mDatabaseHelper = databaseHelper;
        mListener = listener;
        mTransId = trans_id;
    }

    /**
     * GetMessagesAsyncTask UI background processing
     *
     * @param params
     * @return DTOResponseSimple
     */
    @Override
    protected DTOResponseGetMessages doInBackground(Object... params)
    {
        DTOResponseGetMessages objResponse = new DTOResponseGetMessages();
        try
        {
            DTOMessage[] messages;
            if (mTransId != null)
            {
                // DDBB call
                DAOJoin daoJoin = new DAOJoin(mDatabaseHelper);
                DTOMessage message = daoJoin.selectMsgResponseValues(mTransId);
                if (message != null)
                    messages = new DTOMessage[]{message};
                else
                    messages = new DTOMessage[0];
            }
            else
                messages = new DTOMessage[0];

            objResponse.setResult(DAOService.RESPONSE_RESULT_OK);
            objResponse.setMessages(messages);
        }
        catch (Exception e)
        {
            UtilsLog.d(TAG, "Exception in doInBackground", e);
            UtilsApp.sendTraceToServer(UtilsApp.createStandardAddTraceParams(getClass().getName(), "doInBackground", e));
            objResponse.setResult(DAOService.RESPONSE_RESULT_ERROR);
            objResponse.setCode(DAOService.DDBB_ERROR_CODE + DAOService.DDBB_ERROR);
        }
        finally
        {
            return objResponse;
        }
    }

    /**
     * GetMessagesAsyncTask UI main processing after its execution
     *
     * @param response
     */
    @Override
    protected void onPostExecute(DTOResponseGetMessages response)
    {
        if (mListener != null && !isCancelled())
            mListener.onPostGetMsgResponseValuesAsyncTask(response);
    }

}