package com.ejgv.gvi.dao;

import android.database.sqlite.SQLiteOpenHelper;
import android.os.AsyncTask;

import com.ejgv.gvi.dao.dto.DTOResponseTagList;
import com.ejgv.gvi.utils.UtilsApp;
import com.ejgv.gvi.utils.UtilsLog;

/**
 * Created by Batura Mobile Solutions (Koldo Calvo).
 */
class GetTagsAsyncTask extends AsyncTask<Object, Void, DTOResponseTagList>
{
    private static final String TAG = "GetTagsAsyncTask";

    private SQLiteOpenHelper mDatabaseHelper;
    private DAOService.OnGetTagsListener mListener;

    /**
     *
     * @param databaseHelper
     * @param listener
     */
    public GetTagsAsyncTask(SQLiteOpenHelper databaseHelper, DAOService.OnGetTagsListener listener)
    {
        mDatabaseHelper = databaseHelper;
        mListener = listener;
    }

    /**
     * DeleteAsyncTask UI background processing
     * @param params
     * @return DTOResponseSimple
     */
    @Override
    protected DTOResponseTagList doInBackground(Object... params)
    {
        DTOResponseTagList objResponse = new DTOResponseTagList();
        try
        {

            // DDBB call
            DAOVirtualTags daoVirtualTags = new DAOVirtualTags(mDatabaseHelper);
            String[] tags;
            String query = (params != null && params.length > 0) ? String.valueOf(params[0]) : null;
            if (query != null && !query.isEmpty())
                tags = daoVirtualTags.selectTagsMatch(query);
            else
                tags = daoVirtualTags.selectTags();

            objResponse.setResult(DAOService.RESPONSE_RESULT_OK);
            objResponse.setTags(tags);
        }
        catch (Exception e)
        {
            UtilsLog.d(TAG, "Exception in doInBackground", e);
            UtilsApp.sendTraceToServer(UtilsApp.createStandardAddTraceParams(getClass().getName(), "doInBackground", e));
            objResponse.setResult(DAOService.RESPONSE_RESULT_ERROR);
            objResponse.setCode(DAOService.DDBB_ERROR_CODE + DAOService.DDBB_ERROR);
        }
        finally
        {
            return objResponse;
        }
    }

    /**
     * DeleteAsyncTask UI main processing after its execution
     * @param response
     */
    @Override
    protected void onPostExecute(DTOResponseTagList response)
    {
        if (mListener != null && !isCancelled())
            mListener.onPostGetTagsAsyncTask(response);
    }

}