package com.ejgv.gvi.restapipns;

import android.content.ContentValues;
import android.content.Context;
import android.util.Base64;

import com.ejgv.gvi.restapipns.dto.DTOResponseSimple;
import com.ejgv.gvi.restapipns.dto.DTOText;

import org.json.JSONException;
import org.json.JSONObject;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.security.KeyManagementException;
import java.security.NoSuchAlgorithmException;
import java.security.SecureRandom;
import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;
import java.util.Locale;

import javax.net.ssl.HostnameVerifier;
import javax.net.ssl.HttpsURLConnection;
import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLSession;
import javax.net.ssl.TrustManager;
import javax.net.ssl.X509TrustManager;


/**
 * Created by Batura Mobile Solutions (Koldo Calvo).
 */
public class PNSHttpClient
{
    public static final String REQUEST_HEADER_CLICONTEXT = "w43taid";

    public static final String RESPONSE_RESULT = "result";
    public static final String RESPONSE_RESULT_OK = "ok";
    public static final String RESPONSE_RESULT_ERROR = "error";
    public static final String RESPONSE_CODE = "code";
    public static final String RESPONSE_DESCRIPTION = "description";
    public static final String RESPONSE_DESCRIPTION_EU = "eu";
    public static final String RESPONSE_DESCRIPTION_ES = "es";

    public static final String HTTP_CLI_ERROR_TAG = "SRV";

    protected static final int HTTP_CLI_READ_TIMEOUT 	    = 10000;
    protected static final int HTTP_CLI_CONNECT_TIMEOUT 	= 15000;

    protected class JsonBody
    {
        private String operation;
        protected JsonParams params;

        public JsonBody(String operation, JsonParams params)
        {
            this.operation = operation;
            this.params = params;
        }
    }

    protected abstract class JsonParams{};

    private Context mContext;

    /**
     * Constructor
     * @param context
     */
    public PNSHttpClient(Context context)
    {
        this.mContext = context;
    }

    /**
     * mContext getter
     * @return Context
     */
    protected Context getHttpClientContext()
    {
        return mContext;
    }

    /**
     * Get language from current configuration
     * @return String
     */
    protected String getCurrentLocale()
    {
        Locale current_locale = mContext.getResources().getConfiguration().locale;
        return current_locale.getLanguage();
    }

    /**
     * Set SSL Certificate
     * @param connection
     */
    protected void setCertificate(final HttpsURLConnection connection)
    {
        try
        {
            HostnameVerifier hostnameVerifier = new HostnameVerifier() {
                @Override
                public boolean verify(String hostname, SSLSession session) {
                    return true;
                }
            };
            connection.setHostnameVerifier(hostnameVerifier);
            SSLContext sc = SSLContext.getInstance("TLS");
            sc.init(null, new TrustManager[]{new X509TrustManager()
            {
                @Override
                public void checkClientTrusted(X509Certificate[] chain, String authType) throws CertificateException
                {

                }

                @Override
                public void checkServerTrusted(X509Certificate[] chain, String authType) throws CertificateException
                {

                }

                @Override
                public X509Certificate[] getAcceptedIssuers()
                {
                    return new X509Certificate[0];
                }
            }}, new SecureRandom());
            connection.setSSLSocketFactory(sc.getSocketFactory());
        }
        catch (NoSuchAlgorithmException e)
        {
            e.printStackTrace();
        }
        catch (KeyManagementException e)
        {
            e.printStackTrace();
        }
    }

    /**
     * Create the default basic authentication header
     * @return String
     */
    protected String getDefaultAuthorizationBasic()
    {
        return encodeAuthBasic(PNSConfig.getBasicauthUser(), PNSConfig.getBasicauthPwd());
    }

    /**
     * Create and encode a basic authentication header
     * @param user
     * @param password
     * @return String
     */
    private String encodeAuthBasic(String user, String password)
    {
        return "Basic "+ Base64.encodeToString((user + ":" + password).getBytes(), Base64.URL_SAFE | Base64.NO_WRAP);
    }

    /**
     * Create a query string
     * @param values
     * @return String
     * @throws UnsupportedEncodingException
     */
    protected String getQuery(ContentValues values) throws UnsupportedEncodingException
    {
        StringBuilder result = new StringBuilder();
        boolean first = true;

        for ( String key : values.keySet() )
        {
            if (first)
                first = false;
            else
                result.append("&");

            if ( key != null )
            {
                result.append(URLEncoder.encode(key, "UTF-8"));
                result.append("=");
                result.append(URLEncoder.encode(values.getAsString(key), "UTF-8"));
            }
        }

        return result.toString();
    }

    /**
     * Set post params
     * @param connection
     * @param post_data
     * @throws IOException
     */
    protected void setPostParams(HttpsURLConnection connection, String post_data) throws IOException
    {
        // Handle POST parameters
        OutputStream out = connection.getOutputStream();
        BufferedWriter writer = new BufferedWriter(new OutputStreamWriter(out));
        writer.write(post_data);
        writer.flush();
        writer.close();
    }

    /**
     * Read server response data
     * @param connection
     * @return
     * @throws IOException
     * @throws JSONException
     */
    protected JSONObject readData (HttpsURLConnection connection) throws Exception
    {
        JSONObject json_response;

        // Read output
        InputStream in = connection.getInputStream();
        BufferedReader reader = new BufferedReader(new InputStreamReader(in));
        String inputLine;
        StringBuffer response = new StringBuffer();
        while ((inputLine = reader.readLine()) != null)
        {
            response.append(new String(inputLine.getBytes("ISO-8859-1"), "UTF-8"));
        }
        reader.close();
        in.close();

        // Prepare response
        json_response = new JSONObject(response.toString());

        return json_response;
    }

    /**
     * Prepare error response
     * @param errorMessage
     * @param
     * @return
     */
    protected JSONObject createJSONError(String errorMessage )
    {
        JSONObject json_response = new JSONObject();
        JSONObject json_description = new JSONObject();
        try
        {
            json_response.put(RESPONSE_RESULT, RESPONSE_RESULT_ERROR);
            json_response.put(RESPONSE_CODE, HTTP_CLI_ERROR_TAG);
            json_description.put(RESPONSE_DESCRIPTION_EU, errorMessage);
            json_description.put(RESPONSE_DESCRIPTION_ES, errorMessage);
            json_response.put(RESPONSE_DESCRIPTION, json_description);
        }
        catch (JSONException e)
        {
            e.printStackTrace();
        }

        return json_response;
    }

    public static DTOResponseSimple createErrorDTOResponseSimple(String description)
    {
        DTOResponseSimple dtoResponseSimple = new DTOResponseSimple();
        dtoResponseSimple.setResult(RESPONSE_RESULT_ERROR);
        dtoResponseSimple.setCode(HTTP_CLI_ERROR_TAG);
        DTOText dtoText = new DTOText();
        dtoText.setEu(description);
        dtoText.setEs(description);
        dtoResponseSimple.setDescription(dtoText);

        return dtoResponseSimple;
    }
}