package com.ejgv.gvi.restapipns.serviceappregister;

import android.content.Context;
import android.os.AsyncTask;

import com.ejgv.gvi.R;
import com.ejgv.gvi.restapipns.PNSHttpClient;
import com.ejgv.gvi.restapipns.dto.DTOResponseGetSmsToken;
import com.ejgv.gvi.restapipns.dto.DTOResponseSimple;
import com.ejgv.gvi.utils.UtilsApp;
import com.ejgv.gvi.utils.UtilsLog;
import com.google.gson.Gson;
import com.google.gson.GsonBuilder;

import org.json.JSONObject;

/**
 * Created by Batura Mobile Solutions (Koldo Calvo).
 */
class GetSmsTokenAsyncTask extends AsyncTask<Object, Void, DTOResponseGetSmsToken>
{
    private static final String TAG = "GetSmsTokenAsyncTask";

    private Context mContext;
    private MezuPnsPRSService.OnGetSmsTokenListener mListener;

    private String mGsm;
    private String mAppRef;

    /**
     * Constructor
     * @param context
     * @param listener
     */
    public GetSmsTokenAsyncTask(Context context, MezuPnsPRSService.OnGetSmsTokenListener listener,
                                String gsm, String app_ref)
    {
        mContext = context;
        mListener = listener;
        mGsm = gsm;
        mAppRef = app_ref;
    }

    /**
     * GetSmsTokenAsyncTask UI background processing
     *
     * @param params
     * @return DTOResponseGetSmsToken
     */
    @Override
    protected DTOResponseGetSmsToken doInBackground(Object... params)
    {
        DTOResponseGetSmsToken objResponse = null;
        try
        {
            // Api call
            GetSmsTokenHttpClient httpGetSmsToken = new GetSmsTokenHttpClient(mContext);
            JSONObject jsonResponse = httpGetSmsToken.send(mGsm, mAppRef);
            // Parse JSON response to DTO response
            Gson gson = new Gson();
            objResponse = gson.fromJson(jsonResponse.toString(), DTOResponseGetSmsToken.class);

            Gson gson_p = new GsonBuilder().setPrettyPrinting().create();
            UtilsLog.d(TAG, gson_p.toJson(objResponse));
        }
        catch (Exception e)
        {
            UtilsLog.d(TAG, "Exception in doInBackground", e);
            UtilsApp.sendTraceToServer(UtilsApp.createStandardAddTraceParams(getClass().getName(), "doInBackground", e));
            objResponse = setAndGetGeneralError(objResponse);
        }
        finally
        {
            return objResponse;
        }
    }

    /**
     * GetSmsTokenAsyncTask UI main processing after its execution
     *
     * @param response
     */
    @Override
    protected void onPostExecute(DTOResponseGetSmsToken response)
    {
        if (mListener != null && !isCancelled())
            mListener.onPostGetSmsTokenAsyncTask(response);
    }

    private DTOResponseGetSmsToken setAndGetGeneralError(DTOResponseGetSmsToken objResponse)
    {
        DTOResponseSimple dtoResponseSimple = PNSHttpClient.createErrorDTOResponseSimple(mContext.getString(R.string.error_httpclient_prs_getsmstoken));
        objResponse.setResult(dtoResponseSimple.getResult());
        objResponse.setCode(dtoResponseSimple.getCode());
        objResponse.setDescription(dtoResponseSimple.getDescription());
        return  objResponse;
    }

}