package com.ejgv.gvi.restapipns.serviceappregister;

import android.content.Context;

import com.ejgv.gvi.R;
import com.ejgv.gvi.restapipns.PNSHttpClient;
import com.ejgv.gvi.utils.UtilsApp;
import com.ejgv.gvi.utils.UtilsLog;
import com.google.gson.Gson;
import com.google.gson.GsonBuilder;

import org.json.JSONObject;

import java.net.URL;

import javax.net.ssl.HttpsURLConnection;

/**
 * Created by Batura Mobile Solutions (Koldo Calvo).
 */
class GetSmsTokenHttpClient extends PNSHttpClient
{
    private static final String TAG = "GetSmsTokenHttpCli";

    // Service operation
    private static final String operation = "get_sms_token";

    // Service Params
    public class JsonParams extends PNSHttpClient.JsonParams
    {
        private String gsm;
        private String app_ref;

        public JsonParams(String gsm, String app_ref)
        {
            this.gsm = gsm;
            this.app_ref = app_ref;
        }
    }

    /**
     * Constructor
     * @param context
     */
    public GetSmsTokenHttpClient(Context context) {
        super(context);
    }

    /**
     * get_sms_token
     * @param gsm
     * @param app_ref
     * @return JSONObject
     */
    public JSONObject send( String gsm, String app_ref ) throws Exception
    {
        JSONObject json_response;

        // Create URL
        URL url = new URL(MezuPnsPRSConfig.getWsUrl());

        // Create connection
        HttpsURLConnection conn = (HttpsURLConnection) url.openConnection();

        // Add certificate for SSL connection
        setCertificate(conn);

        // Set Connection params
        conn.setReadTimeout(HTTP_CLI_READ_TIMEOUT);
        conn.setConnectTimeout(HTTP_CLI_CONNECT_TIMEOUT);

        // Set Connection properties
        conn.setRequestProperty("Authorization", getDefaultAuthorizationBasic());
        conn.setRequestProperty("Content-Type", "application/json");
        conn.setRequestMethod("POST");
        conn.setDoOutput(true);
        conn.setUseCaches(false);

        // write POST params
        JsonBody jsonBody = new JsonBody(operation, new JsonParams(gsm, app_ref));
        Gson gson = new Gson();
        setPostParams(conn, gson.toJson(jsonBody));

        UtilsLog.d(TAG, url.toString());
        Gson gson_p = new GsonBuilder().setPrettyPrinting().create();
        UtilsLog.d(TAG, gson_p.toJson(jsonBody));

        conn.connect();

        // Handle issues
        int statusCode = conn.getResponseCode();
        if ( statusCode == HttpsURLConnection.HTTP_OK )
            json_response = readData(conn);
        else
        {
            UtilsLog.d(TAG, UtilsApp.createResponseErrorMessage("send", statusCode, conn.getResponseMessage()));
            UtilsApp.sendTraceToServer(
                    UtilsApp.createHttpClientAddTraceParams(
                            getClass().getName(),
                            "send",
                            UtilsApp.createResponseErrorMessage("send", statusCode, conn.getResponseMessage())
                    )
            );
            json_response = createJSONError(getHttpClientContext().getString(R.string.error_httpclient_prs_getsmstoken));
        }

        conn.disconnect();
        return json_response;
    }
}