package com.ejgv.gvi.restapipns.serviceappregister;

import android.content.Context;

import com.ejgv.gvi.R;
import com.ejgv.gvi.restapipns.PNSHttpClient;
import com.ejgv.gvi.utils.UtilsApp;
import com.ejgv.gvi.utils.UtilsLog;
import com.google.gson.Gson;
import com.google.gson.GsonBuilder;

import org.json.JSONObject;

import java.net.URL;

import javax.net.ssl.HttpsURLConnection;

/**
 * Created by Batura Mobile Solutions (Koldo Calvo).
 */
public class RegisterHttpClient extends PNSHttpClient
{
    private static final String TAG = "RegisterHttpClient";

    // Service operation
    private static final String operation = "register";

    // Service Params
    private class JsonParams extends PNSHttpClient.JsonParams
    {
        private String token_sms;
        private String token_pns;
        private String uuid_app;
        private String uuid_device;
        private String chn_provider;
        private String app_version;
        private String dev_vendor;
        private String dev_model;
        private String os_vendor;
        private String os_version;

        public JsonParams(String token_sms, String token_pns, String uuid_app, String uuid_device, String chn_provider,
                          String app_version, String dev_vendor, String dev_model, String os_vendor, String os_version)
        {
            this.token_sms = token_sms;
            this.token_pns = token_pns;
            this.uuid_app = uuid_app;
            this.uuid_device = uuid_device;
            this.chn_provider = chn_provider;
            this.app_version = app_version;
            this.dev_vendor = dev_vendor;
            this.dev_model = dev_model;
            this.os_vendor = os_vendor;
            this.os_version = os_version;
        }
    }

    /**
     * Constructor
     * @param context
     */
    public RegisterHttpClient(Context context) {
        super(context);
    }

    /**
     * register
     * @param token_sms
     * @param token_pns
     * @param uuid_device
     * @param uuid_app
     * @param chn_provider
     * @param app_version
     * @param dev_vendor
     * @param dev_model
     * @param os_vendor
     * @param os_version
     * @return JSONObject
     */
    public JSONObject send( String token_sms, String token_pns, String uuid_app, String uuid_device, String chn_provider,
                            String app_version, String dev_vendor, String dev_model, String os_vendor, String os_version ) throws Exception
    {
        JSONObject json_response;

        // Create URL
        URL url = new URL(MezuPnsPRSConfig.getWsUrl());

        // Create connection
        HttpsURLConnection conn = (HttpsURLConnection) url.openConnection();

        // Add certificate for SSL connection
        setCertificate(conn);

        // Set Connection params
        conn.setReadTimeout(HTTP_CLI_READ_TIMEOUT);
        conn.setConnectTimeout(HTTP_CLI_CONNECT_TIMEOUT);

        // Set Connection properties
        conn.setRequestProperty("Authorization", getDefaultAuthorizationBasic());
        conn.setRequestProperty("Content-Type", "application/json");
        conn.setRequestProperty(PNSHttpClient.REQUEST_HEADER_CLICONTEXT, uuid_app);
        conn.setRequestMethod("POST");
        conn.setDoOutput(true);
        conn.setUseCaches(false);

        // write POST params
        JsonBody jsonBody = new JsonBody(operation, new JsonParams(token_sms, token_pns, uuid_app, uuid_device, chn_provider,
                                            app_version, dev_vendor, dev_model, os_vendor, os_version));
        Gson gson = new Gson();
        setPostParams(conn, gson.toJson(jsonBody));

        UtilsLog.d(TAG, url.toString());
        Gson gson_p = new GsonBuilder().setPrettyPrinting().create();
        UtilsLog.d(TAG, gson_p.toJson(jsonBody));

        conn.connect();

        // Handle issues
        int statusCode = conn.getResponseCode();
        if ( statusCode == HttpsURLConnection.HTTP_OK )
            json_response = readData(conn);
        else
        {
            UtilsLog.d(TAG, UtilsApp.createResponseErrorMessage("send", statusCode, conn.getResponseMessage()));
            UtilsApp.sendTraceToServer(
                    UtilsApp.createHttpClientAddTraceParams(
                            getClass().getName(),
                            "send",
                            UtilsApp.createResponseErrorMessage("send", statusCode, conn.getResponseMessage())
                    )
            );
            json_response = createJSONError(getHttpClientContext().getString(R.string.error_httpclient_prs_register));
        }

        conn.disconnect();

        return json_response;
    }
}