package com.ejgv.gvi.restapipns.serviceappregister;

import android.content.Context;
import android.os.AsyncTask;

import com.ejgv.gvi.R;
import com.ejgv.gvi.restapipns.PNSHttpClient;
import com.ejgv.gvi.restapipns.dto.DTOResponseSimple;
import com.ejgv.gvi.utils.UtilsApp;
import com.ejgv.gvi.utils.UtilsLog;
import com.google.gson.Gson;
import com.google.gson.GsonBuilder;

import org.json.JSONObject;

/**
 * Created by Batura Mobile Solutions (Koldo Calvo).
 */
class UnregisterAsyncTask extends AsyncTask<Object, Void, DTOResponseSimple>
{
    private static final String TAG = "UnregisterAsyncTask";

    private Context mContext;
    private MezuPnsPRSService.OnUnregisterListener mListener;
    private String mUuidApp;
    private String mUuidDevice;

    /**
     * Constructor
     * @param context
     * @param listener
     * @param uuid_app
     * @param uuid_device
     */
    public UnregisterAsyncTask(Context context, MezuPnsPRSService.OnUnregisterListener listener,
                               String uuid_app, String uuid_device)
    {
        mContext = context;
        mListener = listener;
        mUuidApp = uuid_app;
        mUuidDevice = uuid_device;
    }

    /**
     * UnregisterAsyncTask UI background processing
     * @param params
     * @return DTOResponseSimple
     */
    @Override
    protected DTOResponseSimple doInBackground(Object... params)
    {
        DTOResponseSimple objResponse = null;
        try
        {
            // Api call
            UnregisterHttpClient unregisterHttpClient = new UnregisterHttpClient(mContext);
            JSONObject jsonResponse = unregisterHttpClient.send(mUuidApp, mUuidDevice);
            // Parse JSON response to DTO response
            Gson gson = new Gson();
            objResponse = gson.fromJson(jsonResponse.toString(), DTOResponseSimple.class);

            Gson gson_p = new GsonBuilder().setPrettyPrinting().create();
            UtilsLog.d(TAG, gson_p.toJson(objResponse));
        }
        catch (Exception e)
        {
            UtilsLog.d(TAG, "Exception in doInBackground", e);
            UtilsApp.sendTraceToServer(UtilsApp.createStandardAddTraceParams(getClass().getName(), "doInBackground", e));
            objResponse = PNSHttpClient.createErrorDTOResponseSimple(mContext.getString(R.string.error_httpclient_prs_unregister));
        }
        finally
        {
            return objResponse;
        }
    }

    /**
     * UnregisterAsyncTask UI main processing after its execution
     * @param response
     */
    @Override
    protected void onPostExecute(DTOResponseSimple response)
    {
        if (mListener != null && !isCancelled())
            mListener.onPostUnregisterAsyncTask(response);
    }
}