package com.ejgv.gvi.restapipns.servicedataaccess;

import android.content.Context;
import android.os.AsyncTask;

import com.ejgv.gvi.R;
import com.ejgv.gvi.restapipns.PNSHttpClient;
import com.ejgv.gvi.restapipns.dto.DTOResponseSimple;
import com.ejgv.gvi.utils.UtilsApp;
import com.ejgv.gvi.utils.UtilsLog;
import com.google.gson.Gson;
import com.google.gson.GsonBuilder;

import org.json.JSONObject;

/**
 * Created by Batura Mobile Solutions (Koldo Calvo).
 */
class DeleteAsyncTask extends AsyncTask<Object, Void, DTOResponseSimple>
{
    private static final String TAG = "DeleteAsyncTask";

    private Context mContext;
    private MezuPnsPCSService.OnDeleteListener mListener;

    private String mUuidApp;
    private String mUuidDevice;
    private String[] mTransIds;

    /**
     * Constructor
     * @param context
     * @param listener
     */
    public DeleteAsyncTask(Context context, MezuPnsPCSService.OnDeleteListener listener,
                           String uuid_app, String uuid_device, String[] trans_ids)
    {
        this.mContext = context;
        this.mListener = listener;
        mUuidApp = uuid_app;
        mUuidDevice = uuid_device;
        mTransIds = trans_ids;
    }

    /**
     * DeleteAsyncTask UI background processing
     * @param params
     * @return DTOResponseSimple
     */
    @Override
    protected DTOResponseSimple doInBackground(Object... params)
    {
        DTOResponseSimple objResponse = null;
        try
        {
            // Api call
            DeleteHttpClient deleteHttpClient = new DeleteHttpClient(mContext);
            JSONObject jsonResponse = deleteHttpClient.send(mUuidApp, mUuidDevice, mTransIds);
            // Parse JSON response to DTO response
            Gson gson = new Gson();
            objResponse = gson.fromJson(jsonResponse.toString(), DTOResponseSimple.class);
            Gson gson_p = new GsonBuilder().setPrettyPrinting().create();
            UtilsLog.d(TAG, gson_p.toJson(objResponse));
        }
        catch (Exception e)
        {
            UtilsLog.d(TAG, "Exception in doInBackground", e);
            UtilsApp.sendTraceToServer(UtilsApp.createStandardAddTraceParams(getClass().getName(), "doInBackground", e));
            objResponse = PNSHttpClient.createErrorDTOResponseSimple(mContext.getString(R.string.error_httpclient_pcs_delete));
        }
        finally
        {
            return objResponse;
        }
    }

    /**
     * DeleteAsyncTask UI main processing after its execution
     * @param response
     */
    @Override
    protected void onPostExecute(DTOResponseSimple response)
    {
        if (mListener != null && !isCancelled())
            mListener.onPostDeleteAsyncTask(response, mTransIds);
    }

}