package com.ejgv.gvi.restapipns.servicedataaccess;

import android.content.Context;
import android.os.AsyncTask;

import com.ejgv.gvi.R;
import com.ejgv.gvi.restapipns.PNSHttpClient;
import com.ejgv.gvi.restapipns.dto.DTOPnsMessage;
import com.ejgv.gvi.restapipns.dto.DTOPnsMsgPrivate;
import com.ejgv.gvi.restapipns.dto.DTOResponseGetPendingPns;
import com.ejgv.gvi.restapipns.dto.DTOResponseSimple;
import com.ejgv.gvi.restapipns.dto.DTOText;
import com.ejgv.gvi.utils.UtilsApp;
import com.ejgv.gvi.utils.UtilsLog;
import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.google.gson.JsonDeserializationContext;
import com.google.gson.JsonDeserializer;
import com.google.gson.JsonElement;
import com.google.gson.JsonObject;
import com.google.gson.JsonParseException;

import org.json.JSONException;
import org.json.JSONObject;

import java.lang.reflect.Type;

/**
 * Created by Batura Mobile Solutions (Koldo Calvo).
 */
public class GetPendingPnsAsyncTask extends AsyncTask<Object, Void, DTOResponseGetPendingPns>
{
    private static final String TAG = "GetPendingPnsAsyncTask";

    private Context mContext;
    private MezuPnsPCSService.OnGetPendingPnsListener mListener;

    private String mUuidApp;
    private String mUuidDevice;
    private String mTransId;
    private long mTsFrom;

    /**
     * Constructor
     * @param context
     * @param listener
     */
    public GetPendingPnsAsyncTask(Context context, MezuPnsPCSService.OnGetPendingPnsListener listener,
                                  String uuid_app, String uuid_device, String trans_id, long ts_from)
    {
        this.mContext = context;
        this.mListener = listener;
        mUuidApp = uuid_app;
        mUuidDevice = uuid_device;
        mTransId = (trans_id == null) ? "0" : trans_id;
        mTsFrom = ts_from;
    }

    /**
     * GetPendingPnsAsyncTask UI background processing
     * @param params
     * @return DTOResponseGetPendingPns
     */
    @Override
    protected DTOResponseGetPendingPns doInBackground(Object... params)
    {
        DTOResponseGetPendingPns objResponse = new DTOResponseGetPendingPns();
        try
        {
            // Api call
            GetPendingPnsHttpClient getPendingPnsHttpClient = new GetPendingPnsHttpClient(mContext);
            JSONObject jsonResponse = getPendingPnsHttpClient.send(mUuidApp, mUuidDevice, mTransId, mTsFrom);
            // Parse JSON response to DTO response
            Gson gson = new GsonBuilder()
                    .registerTypeAdapter(DTOPnsMessage.class, new DateDeserializer())
                    .create();
            objResponse = gson.fromJson(jsonResponse.toString(), DTOResponseGetPendingPns.class);

            Gson gson_p = new GsonBuilder().setPrettyPrinting().create();
            UtilsLog.d(TAG, gson_p.toJson(objResponse));
        }
        catch (Exception e)
        {
            UtilsLog.d(TAG, "Exception in doInBackground", e);
            UtilsApp.sendTraceToServer(UtilsApp.createStandardAddTraceParams(getClass().getName(), "doInBackground", e));
            objResponse = setAndGetGeneralError(objResponse);
        }
        finally
        {
            return objResponse;
        }
    }

    /**
     * GetPendingPnsAsyncTask UI main processing after its execution
     * @param response
     */
    @Override
    protected void onPostExecute(DTOResponseGetPendingPns response)
    {
        if (mListener != null && !isCancelled())
            mListener.onPostGetPendingPnsAsyncTask(response);
    }

    static public class DateDeserializer implements JsonDeserializer<DTOPnsMessage>
    {
        @Override
        public DTOPnsMessage deserialize(JsonElement je, Type type, JsonDeserializationContext jdc) throws JsonParseException
        {
            if (je == null)
                return null;

            JsonObject jsonMessage = je.getAsJsonObject();

            DTOPnsMessage message = new DTOPnsMessage();
            DTOPnsMsgPrivate pnsMsgPrivate = new DTOPnsMsgPrivate();

            message.setIdTrans(jsonMessage.get("trans_id").getAsString());

            if (jsonMessage.get("public") != null && !jsonMessage.get("public").getAsString().equals("null"))
                message.setPublicText(jsonMessage.get("public").getAsString());

            message.setTs(jsonMessage.get("ts").getAsLong());

            JSONObject jsonPrimitive = null;
            try
            {
                jsonPrimitive = new JSONObject(jsonMessage.get("private").getAsString());
                if (jsonPrimitive.get("private") != null && !jsonMessage.get("private").getAsString().equals("null"))
                    jsonPrimitive = jsonPrimitive.getJSONObject("private");
            }
            catch (JSONException e)
            {

                // Only the Text is available
                DTOText dtoText = new DTOText();
                if(!jsonMessage.get("private").getAsString().equals("null"))
                {
                    dtoText.setEs(jsonMessage.get("private").getAsString());
                    dtoText.setEu(jsonMessage.get("private").getAsString());
                }
                pnsMsgPrivate.setText(dtoText);
                message.setPrivateMsg(pnsMsgPrivate);
                return message;
            }
            catch (Exception e)
            {
                jsonPrimitive = null;
            }
            finally
            {
                if(jsonPrimitive != null)
                {
                    // The whole object is available
                    Gson gson = new Gson();
                    message.setPrivateMsg(gson.fromJson(jsonPrimitive.toString(), DTOPnsMsgPrivate.class));
                }
                return message;
            }
        }
    }

    private DTOResponseGetPendingPns setAndGetGeneralError(DTOResponseGetPendingPns objResponse)
    {
        DTOResponseSimple dtoResponseSimple = PNSHttpClient.createErrorDTOResponseSimple(mContext.getString(R.string.error_httpclient_pcs_getpendingpns));
        objResponse.setResult(dtoResponseSimple.getResult());
        objResponse.setCode(dtoResponseSimple.getCode());
        objResponse.setDescription(dtoResponseSimple.getDescription());
        return  objResponse;
    }

}