package com.ejgv.gvi.restapipns.servicedataaccess;

import android.content.Context;

import com.ejgv.gvi.R;
import com.ejgv.gvi.restapipns.PNSHttpClient;
import com.ejgv.gvi.utils.Utils;
import com.ejgv.gvi.utils.UtilsApp;
import com.ejgv.gvi.utils.UtilsLog;
import com.google.gson.Gson;
import com.google.gson.GsonBuilder;

import org.json.JSONObject;

import java.net.URL;
import java.util.Date;

import javax.net.ssl.HttpsURLConnection;

/**
 * Created by Batura Mobile Solutions (Koldo Calvo).
 */
public class GetPendingPnsHttpClient extends PNSHttpClient
{
    private static final String TAG = "GetPendingPnsHttpClient";

    // Service operation
    private static final String operation = "get_pending_pns";

    // Service Params
    private class JsonParams extends PNSHttpClient.JsonParams
    {
        private String uuid_app;
        private String uuid_device;
        private String trans_id;
        private String ts_from;

        public JsonParams(String uuid_app, String uuid_device, String trans_id, String ts_from)
        {
            this.uuid_app = uuid_app;
            this.uuid_device = uuid_device;
            this.trans_id = trans_id;
            this.ts_from = ts_from;
        }
    }

    /**
     * Constructor
     * @param context
     */
    public GetPendingPnsHttpClient(Context context) {
        super(context);
    }

    /**
     * get_pending_pns
     * @param uuid_app
     * @param uuid_device
     * @param trans_id
     * @param ts_from
     * @return JSONObject
     */
    public JSONObject send( String uuid_app, String uuid_device, String trans_id, long ts_from ) throws Exception
    {
        JSONObject json_response;

        // Create URL
        URL url = new URL(MezuPnsPCSConfig.getWsUrl());

        // Create connection
        HttpsURLConnection conn = (HttpsURLConnection) url.openConnection();

        // Add certificate for SSL connection
        setCertificate(conn);

        // Set Connection params
        conn.setReadTimeout(HTTP_CLI_READ_TIMEOUT);
        conn.setConnectTimeout(HTTP_CLI_CONNECT_TIMEOUT);

        // Set Connection properties
        conn.setRequestProperty("Authorization", getDefaultAuthorizationBasic());
        conn.setRequestProperty("Content-Type", "application/json");
        conn.setRequestProperty(PNSHttpClient.REQUEST_HEADER_CLICONTEXT, uuid_app);
        conn.setRequestMethod("POST");
        conn.setDoOutput(true);
        conn.setUseCaches(false);

        // write POST params
        String tsFrom = (ts_from == 0) ? "0" : Utils.getDateFormatted(new Date(ts_from), "yy/MM/dd HH:mm:ss");
        JsonBody jsonBody = new JsonBody(operation, new JsonParams(uuid_app, uuid_device, trans_id, tsFrom));
        Gson gson = new Gson();
        setPostParams(conn, gson.toJson(jsonBody));

        UtilsLog.d(TAG, url.toString());
        Gson gson_p = new GsonBuilder().setPrettyPrinting().create();
        UtilsLog.d(TAG, gson_p.toJson(jsonBody));

        conn.connect();

        // Handle issues
        int statusCode = conn.getResponseCode();
        if ( statusCode == HttpsURLConnection.HTTP_OK )
            json_response = readData(conn);
        else
        {
            UtilsLog.d(TAG, UtilsApp.createResponseErrorMessage("send", statusCode, conn.getResponseMessage()));
            UtilsApp.sendTraceToServer(
                    UtilsApp.createHttpClientAddTraceParams(
                            getClass().getName(),
                            "send",
                            UtilsApp.createResponseErrorMessage("send", statusCode, conn.getResponseMessage())
                    )
            );
            json_response = createJSONError(getHttpClientContext().getString(R.string.error_httpclient_pcs_getpendingpns));
        }

        conn.disconnect();

        return json_response;
    }
}