package com.ejgv.gvi.restapipns.servicedataaccess;

import android.content.Context;
import android.os.AsyncTask;

import com.ejgv.gvi.restapipns.PNSHttpClient;
import com.ejgv.gvi.restapipns.dto.DTOResponseSimple;
import com.ejgv.gvi.utils.UtilsApp;
import com.ejgv.gvi.utils.UtilsLog;
import com.google.gson.Gson;
import com.google.gson.GsonBuilder;

import org.json.JSONObject;

/**
 * Created by Batura Mobile Solutions (Koldo Calvo).
 */
class MarkAsReadAsyncTask extends AsyncTask<Object, Void, DTOResponseSimple>
{
    private static final String TAG = "MarkAsReadAsyncTask";

    private Context mContext;
    private MezuPnsPCSService.OnMarkAsReadListener mListener;

    private String mUuidApp;
    private String mUuidDevice;
    private String mTransId;

    /**
     * Constructor
     *
     * @param context
     * @param listener
     */
    public MarkAsReadAsyncTask(Context context, MezuPnsPCSService.OnMarkAsReadListener listener,
                               String uuid_app, String uuid_device, String trans_id)
    {
        this.mContext = context;
        this.mListener = listener;
        mUuidApp = uuid_app;
        mUuidDevice = uuid_device;
        mTransId = trans_id;
    }

    /**
     * MarkAsReadAsyncTask UI background processing
     * @param params
     * @return DTOResponseSimple
     */
    @Override
    protected DTOResponseSimple doInBackground(Object... params)
    {
        DTOResponseSimple objResponse = null;
        try
        {
            // Api call
            MarkAsReadHttpClient markAsReadHttpClient = new MarkAsReadHttpClient(mContext);
            JSONObject jsonResponse = markAsReadHttpClient.send(mUuidApp, mUuidDevice, mTransId);
            // Parse JSON response to DTO response
            Gson gson = new Gson();
            objResponse = gson.fromJson(jsonResponse.toString(), DTOResponseSimple.class);

            Gson gson_p = new GsonBuilder().setPrettyPrinting().create();
            UtilsLog.d(TAG, gson_p.toJson(objResponse));
        }
        catch (Exception e)
        {
            UtilsLog.d(TAG, "Exception in doInBackground", e);
            UtilsApp.sendTraceToServer(UtilsApp.createStandardAddTraceParams(getClass().getName(), "doInBackground", e));
            objResponse = PNSHttpClient.createErrorDTOResponseSimple(null);
        }
        finally
        {
            return objResponse;
        }
    }

    /**
     * MarkAsReadAsyncTask UI main processing after its execution
     * @param response
     */
    @Override
    protected void onPostExecute(DTOResponseSimple response)
    {
        if (mListener != null && !isCancelled())
            mListener.onPostMarkAsReadAsyncTask(response);
    }

}