package com.ejgv.gvi.restapipns.servicedataaccess;

import android.content.Context;

import com.ejgv.gvi.restapipns.PNSHttpClient;
import com.ejgv.gvi.utils.UtilsApp;
import com.ejgv.gvi.utils.UtilsLog;
import com.google.gson.Gson;
import com.google.gson.GsonBuilder;

import org.json.JSONObject;

import java.net.URL;

import javax.net.ssl.HttpsURLConnection;

/**
 * Created by Batura Mobile Solutions (Koldo Calvo).
 */
public class MarkAsReadHttpClient extends PNSHttpClient
{
    private static final String TAG = "MarkAsReadHttpClient";

    // Service operation
    private static final String operation = "mark_as_read";

    // Service Params
    private class JsonParams extends PNSHttpClient.JsonParams
    {
        private String uuid_app;
        private String uuid_device;
        private String trans_id;

        public JsonParams(String uuid_app, String uuid_device, String trans_id)
        {
            this.uuid_app = uuid_app;
            this.uuid_device = uuid_device;
            this.trans_id = trans_id;
        }
    }

    /**
     * Constructor
     * @param context
     */
    public MarkAsReadHttpClient(Context context) {
        super(context);
    }

    /**
     * mark_as_read
     *
     * @param uuid_app
     * @param uuid_device
     * @param trans_id
     * @return JSONObject
     */
    public JSONObject send(  String uuid_app, String uuid_device, String trans_id ) throws Exception
    {
        JSONObject json_response;

        // Create URL
        URL url = new URL(MezuPnsPCSConfig.getWsUrl());

        // Create connection
        HttpsURLConnection conn = (HttpsURLConnection) url.openConnection();

        // Add certificate for SSL connection
        setCertificate(conn);

        // Set Connection params
        conn.setReadTimeout(HTTP_CLI_READ_TIMEOUT);
        conn.setConnectTimeout(HTTP_CLI_CONNECT_TIMEOUT);

        // Set Connection properties
        conn.setRequestProperty("Authorization", getDefaultAuthorizationBasic());
        conn.setRequestProperty("Content-Type", "application/json");
        conn.setRequestProperty(PNSHttpClient.REQUEST_HEADER_CLICONTEXT, uuid_app);
        conn.setRequestMethod("POST");
        conn.setDoOutput(true);
        conn.setUseCaches(false);

        // write POST params
        JsonBody jsonBody = new JsonBody(operation, new JsonParams(uuid_app, uuid_device, trans_id));
        Gson gson = new Gson();
        setPostParams(conn, gson.toJson(jsonBody));

        UtilsLog.d(TAG, url.toString());
        Gson gson_p = new GsonBuilder().setPrettyPrinting().create();
        UtilsLog.d(TAG, gson_p.toJson(jsonBody));

        conn.connect();

        // Handle issues
        int statusCode = conn.getResponseCode();
        if ( statusCode == HttpsURLConnection.HTTP_OK )
            json_response = readData(conn);
        else
        {
            UtilsLog.d(TAG, UtilsApp.createResponseErrorMessage("send", statusCode, conn.getResponseMessage()));
            UtilsApp.sendTraceToServer(
                    UtilsApp.createHttpClientAddTraceParams(
                            getClass().getName(),
                            "send",
                            UtilsApp.createResponseErrorMessage("send", statusCode, conn.getResponseMessage())
                    )
            );
            json_response = createJSONError(null);
        }

        conn.disconnect();

        return json_response;
    }
}