package com.ejgv.gvi.restapipns.servicedataaccess;

import android.content.Context;
import android.os.AsyncTask;

import com.ejgv.gvi.restapipns.PNSHttpClient;
import com.ejgv.gvi.restapipns.dto.DTOResponseSimple;
import com.ejgv.gvi.utils.UtilsApp;
import com.ejgv.gvi.utils.UtilsLog;
import com.google.gson.Gson;
import com.google.gson.GsonBuilder;

import org.json.JSONObject;

/**
 * Created by Batura Mobile Solutions (Koldo Calvo).
 */
class UpdateTokenPnsAsyncTask extends AsyncTask<Object, Void, DTOResponseSimple>
{
    private static final String TAG = "UpdateTokenPnsAsyncTask";

    private Context mContext;
    private MezuPnsPCSService.OnUpdateTokenPnsListener mListener;

    private String mTokenPns;
    private String mUuidApp;
    private String mUuidDevice;
    private String mChnProvider;
    private String mAppVersion;
    private String mDevVendor;
    private String mDevModel;
    private String mOsVendor;
    private String mOsVersion;

    /**
     * Constructor
     * @param context
     * @param listener
     */
    public UpdateTokenPnsAsyncTask(Context context, MezuPnsPCSService.OnUpdateTokenPnsListener listener,
                                   String token_pns, String uuid_app, String uuid_device, String chn_provider,
                                   String app_version, String dev_vendor, String dev_model, String os_vendor, String os_version)
    {
        mContext = context;
        mListener = listener;
        mTokenPns = token_pns;
        mUuidApp = uuid_app;
        mUuidDevice = uuid_device;
        mChnProvider = chn_provider;
        mAppVersion = app_version;
        mDevVendor = dev_vendor;
        mDevModel = dev_model;
        mOsVendor = os_vendor;
        mOsVersion = os_version;
    }

    /**
     * UpdateTokenPnsAsyncTask UI background processing
     * @param params
     * @return DTOResponseSimple
     */
    @Override
    protected DTOResponseSimple doInBackground(Object... params)
    {
        DTOResponseSimple objResponse = null;
        try
        {
            // Api call
            UpdateTokenPnsHttpClient updateTokenPnsHttpClient = new UpdateTokenPnsHttpClient(mContext);
            JSONObject jsonResponse = updateTokenPnsHttpClient.send(mTokenPns, mUuidApp, mUuidDevice, mChnProvider,
                                                                mAppVersion, mDevVendor, mDevModel, mOsVendor, mOsVersion);
            // Parse JSON response to DTO response
            Gson gson = new Gson();
            objResponse = gson.fromJson(jsonResponse.toString(), DTOResponseSimple.class);

            Gson gson_p = new GsonBuilder().setPrettyPrinting().create();
            UtilsLog.d(TAG, gson_p.toJson(objResponse));
        }
        catch (Exception e)
        {
            UtilsLog.d(TAG, "Exception in doInBackground", e);
            UtilsApp.sendTraceToServer(UtilsApp.createStandardAddTraceParams(getClass().getName(), "doInBackground", e));
            objResponse = PNSHttpClient.createErrorDTOResponseSimple(null);
        }
        finally
        {
            return objResponse;
        }
    }

    /**
     * UpdateTokenPnsAsyncTask UI main processing after its execution
     * @param response
     */
    @Override
    protected void onPostExecute(DTOResponseSimple response)
    {
        if (mListener != null && !isCancelled())
            mListener.onPostUpdateTokenPnsAsyncTask(response);
    }

}