package com.ejgv.gvi.restapipns.servicesendingresponses;

import android.content.Context;
import android.os.AsyncTask;

import com.ejgv.gvi.restapipns.PNSHttpClient;
import com.ejgv.gvi.restapipns.dto.DTOResponseSimple;
import com.ejgv.gvi.utils.UtilsApp;
import com.ejgv.gvi.utils.UtilsLog;
import com.google.gson.Gson;
import com.google.gson.GsonBuilder;

import org.json.JSONObject;

/**
 * Created by Batura Mobile Solutions (Koldo Calvo).
 */
class MsgInAsyncTask extends AsyncTask<Object, Void, DTOResponseSimple>
{
    private static final String TAG = "MsgInAsyncTask";

    private Context mContext;
    private MezuPnsRSPService.OnMsgInListener mListener;

    private String mAlias;
    private String mCode;
    private String mUuidApp;
    private String mUuidDevice;
    private String mTransId;
    private long mTsSend;

    /**
     * Constructor
     * @param context
     * @param listener
     */
    public MsgInAsyncTask(Context context, MezuPnsRSPService.OnMsgInListener listener,
                          String alias, String codigo, String uuid_app, String uuid_device, String trans_id, long ts_send)
    {
        this.mContext = context;
        this.mListener = listener;
        mAlias = alias;
        mCode = codigo;
        mUuidApp = uuid_app;
        mUuidDevice = uuid_device;
        mTransId = trans_id;
        mTsSend = ts_send;
    }

    /**
     * MsgInAsyncTask UI background processing
     * @param params
     * @return DTOResponseSimple
     */
    @Override
    protected DTOResponseSimple doInBackground(Object... params)
    {
        DTOResponseSimple objResponse = null;
        try
        {
            // Api call
            MsgInHttpClient msgInHttpClient = new MsgInHttpClient(mContext);
            JSONObject jsonResponse = msgInHttpClient.send(mAlias, mCode, mUuidApp, mUuidDevice, mTransId, mTsSend);
            // Parse JSON response to DTO response
            Gson gson = new Gson();
            objResponse = gson.fromJson(jsonResponse.toString(), DTOResponseSimple.class);

            Gson gson_p = new GsonBuilder().setPrettyPrinting().create();
            UtilsLog.d(TAG, gson_p.toJson(objResponse));
        }
        catch (Exception e)
        {
            UtilsLog.d(TAG, "Exception in doInBackground", e);
            UtilsApp.sendTraceToServer(UtilsApp.createStandardAddTraceParams(getClass().getName(), "doInBackground", e));
            objResponse = PNSHttpClient.createErrorDTOResponseSimple(null);
        }
        finally
        {
            return objResponse;
        }
    }

    /**
     * MsgInAsyncTask UI main processing after its execution
     * @param response
     */
    @Override
    protected void onPostExecute(DTOResponseSimple response)
    {
        if (mListener != null && !isCancelled())
            mListener.onPostMsgInAsyncTask(response);
    }

}