package com.ejgv.gvi.restapitrace;

import android.content.ContentValues;
import android.util.Base64;

import com.ejgv.gvi.utils.UtilsLog;

import org.json.JSONException;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.net.URLEncoder;
import java.security.KeyManagementException;
import java.security.NoSuchAlgorithmException;
import java.security.SecureRandom;
import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;

import javax.net.ssl.HostnameVerifier;
import javax.net.ssl.HttpsURLConnection;
import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLSession;
import javax.net.ssl.TrustManager;
import javax.net.ssl.X509TrustManager;


/**
 * Created by Batura Mobile Solutions (Koldo Calvo).
 */
public class TraceHttpClient
{
    private static final String TAG = "TraceHttpClient";
    public static final String RESPONSE_RESULT_OK = "ok";
    public static final String RESPONSE_RESULT_ERROR = "error";

    protected static final int HTTP_CLI_READ_TIMEOUT 				= 10000;
    protected static final int HTTP_CLI_CONNECT_TIMEOUT 			= 15000;

    /**
     * Empty Constructor
     */
    public TraceHttpClient()
    {
    }

    /**
     * Set SSL Certificate
     * @param connection
     */
    protected void setCertificate(final HttpsURLConnection connection)
    {
        try
        {
            HostnameVerifier hostnameVerifier = new HostnameVerifier() {
                @Override
                public boolean verify(String hostname, SSLSession session) {
                    return true;
                }
            };
            connection.setHostnameVerifier(hostnameVerifier);
            SSLContext sc = SSLContext.getInstance("TLS");
            sc.init(null, new TrustManager[]{new X509TrustManager()
            {
                @Override
                public void checkClientTrusted(X509Certificate[] chain, String authType) throws CertificateException
                {

                }

                @Override
                public void checkServerTrusted(X509Certificate[] chain, String authType) throws CertificateException
                {

                }

                @Override
                public X509Certificate[] getAcceptedIssuers()
                {
                    return new X509Certificate[0];
                }
            }}, new SecureRandom());
            connection.setSSLSocketFactory(sc.getSocketFactory());
        }
        catch (NoSuchAlgorithmException e)
        {
            e.printStackTrace();
        }
        catch (KeyManagementException e)
        {
            e.printStackTrace();
        }
    }

    /**
     * Create the default basic authentication header
     * @return String
     */
    protected String getDefaultAuthorizationBasic()
    {
        return encodeAuthBasic(TraceConfig.getBasicauthUser(), TraceConfig.getBasicauthPwd());
    }

    /**
     * Create and encode a basic authentication header
     * @param user
     * @param password
     * @return String
     */
    private String encodeAuthBasic(String user, String password)
    {
        return "Basic "+ Base64.encodeToString((user + ":" + password).getBytes(), Base64.URL_SAFE | Base64.NO_WRAP);
    }

    /**
     * Create a query string
     * @param values
     * @return String
     * @throws Exception
     */
    protected String getQuery(ContentValues values) throws Exception
    {
        StringBuilder result = new StringBuilder();
        boolean first = true;

        for ( String key : values.keySet() )
        {
            if (first)
                first = false;
            else
                result.append("&");

            if ( key != null )
            {
                result.append(URLEncoder.encode(key, "UTF-8"));
                result.append("=");
                result.append(URLEncoder.encode(values.getAsString(key), "UTF-8"));
            }
        }

        UtilsLog.d(TAG, result.toString());
        return result.toString();
    }

    /**
     * Set post params
     * @param connection
     * @param post_data
     * @throws IOException
     */
    protected void setPostParams(HttpsURLConnection connection, String post_data) throws IOException
    {
        // Handle POST parameters
        OutputStream out = connection.getOutputStream();
        BufferedWriter writer = new BufferedWriter(new OutputStreamWriter(out));
        writer.write(post_data);
        writer.flush();
        writer.close();
    }

    /**
     * Set post params
     * @param connection
     * @param values
     * @throws IOException
     */
    protected void setPostParams(HttpsURLConnection connection, ContentValues values) throws Exception
    {
        // Handle POST parameters
        OutputStream out = connection.getOutputStream();
        BufferedWriter writer = new BufferedWriter(new OutputStreamWriter(out));
        writer.write(getQuery(values));
        writer.flush();
        writer.close();
    }

    /**
     * Read server response data
     * @param connection
     * @return
     * @throws IOException
     * @throws JSONException
     */
    protected String readData (HttpsURLConnection connection) throws Exception
    {
        // Read output
        InputStream in = connection.getInputStream();
        BufferedReader reader = new BufferedReader(new InputStreamReader(in));
        String inputLine;
        StringBuffer response = new StringBuffer();
        while ((inputLine = reader.readLine()) != null)
        {
            response.append(new String(inputLine));
        }
        reader.close();
        in.close();

        return response.toString();
    }
}