package com.ejgv.gvi.ui;

/**
 * Created by koldo on 13/12/15.
 */

import android.content.Context;
import android.support.v7.widget.RecyclerView;
import android.util.AttributeSet;
import android.view.View;

/**
 * Simple RecyclerView subclass that supports providing an empty view (which
 * is displayed when the adapter has no data and hidden otherwise).
 */
public class EmptyRecyclerView extends RecyclerView
{
    private View mEmptyView;

    private AdapterDataObserver mDataObserver = new AdapterDataObserver() {
                                                        @Override
                                                        public void onItemRangeRemoved(int positionStart, int itemCount)
                                                        {
                                                            super.onItemRangeRemoved(positionStart, itemCount);
                                                            updateEmptyView();
                                                        }
    };

    public EmptyRecyclerView(Context context)
    {
        super(context);
    }

    public EmptyRecyclerView(Context context, AttributeSet attrs)
    {
        super(context, attrs);
    }

    public EmptyRecyclerView(Context context, AttributeSet attrs, int defStyle)
    {
        super(context, attrs, defStyle);
    }

    /**
     * Designate a view as the empty view. When the backing adapter has no
     * data this view will be made visible and the recycler view hidden.
     *
     */
    public void setEmptyView(View emptyView)
    {
        mEmptyView = emptyView;
    }

    @Override
    public void setAdapter(RecyclerView.Adapter adapter)
    {
        if (getAdapter() != null)
            getAdapter().unregisterAdapterDataObserver(mDataObserver);

        if (adapter != null)
            adapter.registerAdapterDataObserver(mDataObserver);

        super.setAdapter(adapter);

        updateEmptyView();
    }

    private void updateEmptyView()
    {
        if (mEmptyView != null && getAdapter() != null)
        {
            int intItemCount = getAdapter().getItemCount();
            if ( intItemCount > 0 )
                mEmptyView.setVisibility(INVISIBLE);
            else
                mEmptyView.setVisibility(VISIBLE);
        }
        else if (mEmptyView != null)
            mEmptyView.setVisibility(VISIBLE);
    }
}