package com.ejgv.gvi.utils;

import android.content.Context;
import android.content.Intent;
import android.content.res.Configuration;
import android.content.res.Resources;
import android.graphics.Typeface;
import android.graphics.drawable.Drawable;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.net.Uri;
import android.os.Build;
import android.text.Html;
import android.text.Spannable;
import android.text.SpannableStringBuilder;
import android.text.Spanned;
import android.text.style.ForegroundColorSpan;
import android.text.style.StyleSpan;
import android.util.Log;

import java.io.PrintWriter;
import java.io.StringWriter;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;

/**
 * Created by Batura Mobile Solutions (Koldo Calvo).
 */
public class Utils
{

    /**
     * Convert statc trace output to String
     *
     * @param t
     * @return
     */
    public static String getStackTraceString(Throwable t)
    {
        StringWriter stackTraceStringWriter = new StringWriter();
        PrintWriter stackTracePrintWriter = new PrintWriter(stackTraceStringWriter);
        t.printStackTrace(stackTracePrintWriter);
        return stackTraceStringWriter.toString();
    }

    /**
     * Log info
     *
     * @param tag
     * @param message
     * @param t
     */
    public static void logInfo(String tag, String message, Throwable t)
    {
        if (tag.length() > 23)
            tag = tag.substring(0, 23);

        Log.i(tag, message, t);
    }

    /**
     * Log warning
     *
     * @param tag
     * @param message
     * @param t
     */
    public static void logWarning(String tag, String message, Throwable t)
    {
        if (tag.length() > 23)
            tag = tag.substring(0, 23);

        Log.w(tag, message, t);
    }

    /**
     * Log error
     *
     * @param tag
     * @param message
     * @param t
     */
    public static void logError(String tag, String message, Throwable t)
    {
        if (tag.length() > 23)
            tag = tag.substring(0, 23);

        Log.e(tag, message, t);
    }



    /**
     * Make empty string for concatenations of null value
     *
     * @param strNullable
     * @return
     */
    public static String getNullableString(String strNullable)
    {
        return (strNullable != null) ? strNullable : "";
    }

    /**
     * Update current app configuration default locale
     * @param context
     * @param language
     */
    public static void updateLanguage(Context context, String language)
    {
        Locale locale;
        
        if ( language == null || language.isEmpty())
        {
            locale = new Locale("en");
        }
        else
        {
            locale = new Locale(language);
        }
        updateLocale(context, locale);
    }

    /**
     * Update current app configuration default locale
     * @param context
     * @param language
     */
    public static void updateLanguage(Context context, String language, String country)
    {
        Locale locale;

        if ( language == null || language.isEmpty() || country == null || country.isEmpty())
        {
            locale = new Locale("en", "US");
        }
        else
        {
            locale = new Locale(language, country);
        }
        updateLocale(context, locale);
    }

    private static void updateLocale(Context context, Locale locale)
    {
        Configuration config = context.getResources().getConfiguration();
        config.locale = locale;
        context.getResources().updateConfiguration(config, context.getResources().getDisplayMetrics());

        Configuration systemConf = Resources.getSystem().getConfiguration();
        systemConf.locale = locale;
        Resources.getSystem().updateConfiguration(systemConf, Resources.getSystem().getDisplayMetrics());

        Locale.setDefault(locale);
    }

    private static final Map<String, Typeface> typefaces = new HashMap<>();
    public static Typeface getTypeface(Context ctx, String fontName)
    {
        Map<String, Typeface> typefaces = new HashMap<>();

        Typeface typeface = typefaces.get(fontName);
        if (typeface == null)
        {
            typeface = Typeface.createFromAsset(ctx.getAssets(), fontName);
            typefaces.put(fontName, typeface);
        }
        return typeface;
    }
    public static void clearTypefaces()
    {
        typefaces.clear();
    }

    /**
     * Launch compose email intent
     * @param context
     * @param addresses
     * @param subject
     */
    public static void composeEmail(Context context, String[] addresses, String subject)
    {
        Intent intent = new Intent(Intent.ACTION_SENDTO);
        intent.setData(Uri.parse("mailto:")); // only email apps should handle this
        intent.putExtra(Intent.EXTRA_EMAIL, addresses);
        intent.putExtra(Intent.EXTRA_SUBJECT, subject);
        if (intent.resolveActivity(context.getPackageManager()) != null) {
            context.startActivity(intent);
        }
    }

    /**
     * Check if the app has Internet connectivity
     * @param context
     * @return
     */
    public static boolean isConnected(Context context)
    {
        // Check network connection
        ConnectivityManager connMgr = (ConnectivityManager) context.getSystemService(Context.CONNECTIVITY_SERVICE);
        NetworkInfo networkInfo = connMgr.getActiveNetworkInfo();
        if (networkInfo != null && networkInfo.isConnected())
        {
            return true;
        }
        else
        {
            return false;
        }
    }

    /**
     * Get drawable from context in a API save context
     * @param context
     * @param drawable_id
     * @return
     */
    public static Drawable getDrawableAPISafe(Context context, int drawable_id)
    {
        Drawable drawable;

        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP)
        {
            drawable = context.getResources().getDrawable(drawable_id, null);
        }
        else {
            drawable = context.getResources().getDrawable(drawable_id);
        }
        return drawable;
    }

    /**
     * Get color from context in a API save context
     * @param context
     * @param color_id
     * @return int
     */
    public static int getColorAPISafe(Context context, int color_id)
    {
        int color;

        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M)
        {
            color = context.getResources().getColor(color_id, null);
        }
        else {
            color = context.getResources().getColor(color_id);
        }
        return color;
    }

    /**
     * Function to format a date
     * @param date
     * @param format_out
     * @return String
     * @throws ParseException
     */
    public static String getDateFormatted(Date date, int format_out) throws ParseException
    {
        return SimpleDateFormat.getDateInstance(
                format_out, Locale.getDefault())
                .format(date);
    }

    /**
     * Function to format a date
     * @param value
     * @param format_in
     * @param format_out
     * @return String
     * @throws ParseException
     */
    public static String getDateFormatted(String value, String format_in, int format_out) throws ParseException
    {
        DateFormat fmt = new SimpleDateFormat(format_in);
        Date date = fmt.parse(value);

        return SimpleDateFormat.getDateInstance(
                format_out, Locale.getDefault())
                .format(date);
    }

    /**
     * Function to format a date
     * @param locale
     * @param value
     * @param format_in
     * @param format_out
     * @return String
     * @throws ParseException
     */
    public static String getDateFormatted(Locale locale, String value, String format_in, int format_out) throws ParseException
    {
        DateFormat fmt = new SimpleDateFormat(format_in);
        Date date = fmt.parse(value);

        return SimpleDateFormat.getDateInstance(
                format_out, locale)
                .format(date);
    }

    /**
     * Function to format a date
     * @param date
     * @param format_out
     * @return
     * @throws ParseException
     */
    public static String getDateFormatted(Date date, String format_out) throws ParseException
    {
        Calendar cal = GregorianCalendar.getInstance(Locale.getDefault());
        cal.setTime(date);

        SimpleDateFormat fmt_out = new SimpleDateFormat(format_out);

        return fmt_out.format(cal.getTime());
    }

    /**
     * Function to format a date
     * @param date
     * @param format_out
     * @return
     * @throws ParseException
     */
    public static String getDateFormatted(Locale locale, Date date, String format_out) throws ParseException
    {
        Calendar cal = GregorianCalendar.getInstance(locale);
        cal.setTime(date);

        SimpleDateFormat fmt_out = new SimpleDateFormat(format_out);

        return fmt_out.format(cal.getTime());
    }

    /**
     * Function to format a date
     * @param value
     * @param format_in
     * @param format_out
     * @return
     * @throws ParseException
     */
    public static String getDateFormatted(String value, String format_in, String format_out) throws ParseException
    {
        DateFormat fmt_in = new SimpleDateFormat(format_in);
        Date date = fmt_in.parse(value);

        Calendar cal = GregorianCalendar.getInstance(Locale.getDefault());
        cal.setTime(date);

        SimpleDateFormat fmt_out = new SimpleDateFormat(format_out);

        return fmt_out.format(cal.getTime());
    }

    /**
     * Function to format a date
     * @param locale
     * @param value
     * @param format_in
     * @param format_out
     * @return
     * @throws ParseException
     */
    public static String getDateFormatted(Locale locale, String value, String format_in, String format_out) throws ParseException
    {
        DateFormat fmt_in = new SimpleDateFormat(format_in);
        Date date = fmt_in.parse(value);

        Calendar cal = GregorianCalendar.getInstance(locale);
        cal.setTime(date);

        SimpleDateFormat fmt_out = new SimpleDateFormat(format_out);

        return fmt_out.format(cal.getTime());
    }

    /**
     * Returns a CharSequence that concatenates the specified array of CharSequence
     * objects and then applies a list of zero or more tags to the entire range.
     *
     * @param content an array of character sequences to apply a style to
     * @param tags the styled span objects to apply to the content
     *        such as android.text.style.StyleSpan
     *
     */
    private static SpannableStringBuilder apply(CharSequence[] content, Object... tags) {
        SpannableStringBuilder text = new SpannableStringBuilder();
        openTags(text, tags);
        for (CharSequence item : content) {
            text.append(item);
        }
        closeTags(text, tags);
        Spanned spanned = text;
        return text;
    }

    /**
     * Iterates over an array of tags and applies them to the beginning of the specified
     * Spannable object so that future text appended to the text will have the styling
     * applied to it. Do not call this method directly.
     */
    private static void openTags(Spannable text, Object[] tags) {
        for (Object tag : tags) {
            text.setSpan(tag, 0, 0, Spannable.SPAN_MARK_MARK);
        }
    }

    /**
     * "Closes" the specified tags on a Spannable by updating the spans to be
     * endpoint-exclusive so that future text appended to the end will not take
     * on the same styling. Do not call this method directly.
     */
    private static void closeTags(Spannable text, Object[] tags) {
        int len = text.length();
        for (Object tag : tags) {
            if (len > 0) {
                text.setSpan(tag, 0, len, Spanned.SPAN_EXCLUSIVE_EXCLUSIVE);
            } else {
                text.removeSpan(tag);
            }
        }
    }

    /**
     * Returns a CharSequence that applies boldface to the concatenation
     * of the specified CharSequence objects.
     */
    public static CharSequence bold(CharSequence... content) {
        return apply(content, new StyleSpan(Typeface.BOLD));
    }

    /**
     * Returns a purged html String that applies boldface to the concatenation
     * of the specified CharSequence objects.
     */
    public static String boldPurged(CharSequence... content) {
        return purgeToHtmlOfSpannedCast(Html.toHtml(apply(content, new StyleSpan(Typeface.BOLD))));
    }

    /**
     * Returns a CharSequence that applies italics to the concatenation
     * of the specified CharSequence objects.
     */
    public static CharSequence italic(CharSequence... content) {
        return apply(content, new StyleSpan(Typeface.ITALIC));
    }

    /**
     * Returns a purged String that applies italics to the concatenation
     * of the specified CharSequence objects.
     */
    public static String italicPurged(CharSequence... content) {
        return purgeToHtmlOfSpannedCast(apply(content, new StyleSpan(Typeface.ITALIC)));
    }

    /**
     * Returns a CharSequence that applies a foreground color to the
     * concatenation of the specified CharSequence objects.
     */
    public static CharSequence color(int color, CharSequence... content) {
        return apply(content, new ForegroundColorSpan(color));
    }

    /**
     * Returns a CharSequence that applies a foreground color to the
     * concatenation of the specified CharSequence objects.
     */
    public static String colorPurged(int color, CharSequence... content) {
        return purgeToHtmlOfSpannedCast(apply(content, new ForegroundColorSpan(color)));
    }

    /**
     * Returns a String without spanned cast extra html
     */
    public static String purgeToHtmlOfSpannedCast(String content)
    {
        return content.replaceAll("<p dir=\"ltr\">", "").replaceAll("<p dir=ltr>", "").replaceAll("</p>", "");
    }

    /**
     * Returns a String without spanned cast extra html
     */
    public static String purgeToHtmlOfSpannedCast(CharSequence content)
    {
        return content.toString().replaceAll("<p dir=\"ltr\">", "").replaceAll("<p dir=ltr>", "").replaceAll("</p>", "");
    }

    // Capitalizing
    //-----------------------------------------------------------------------
    /**
     * <p>Capitalizes all the whitespace separated words in a String.
     * Only the first letter of each word is changed. To convert the
     * rest of each word to lowercase at the same time,
     * use {@link #capitalizeFully(String)}.</p>
     *
     * <p>Whitespace is defined by {@link Character#isWhitespace(char)}.
     * A <code>null</code> input String returns <code>null</code>.
     * Capitalization uses the unicode title case, normally equivalent to
     * upper case.</p>
     *
     * <pre>
     * WordUtils.capitalize(null)        = null
     * WordUtils.capitalize("")          = ""
     * WordUtils.capitalize("i am FINE") = "I Am FINE"
     * </pre>
     *
     * @param str  the String to capitalize, may be null
     * @return capitalized String, <code>null</code> if null String input
     * @see #uncapitalize(String)
     * @see #capitalizeFully(String)
     */
    public static String capitalize(String str) {
        return capitalize(str, null);
    }

    /**
     * <p>Capitalizes all the delimiter separated words in a String.
     * Only the first letter of each word is changed. To convert the
     * rest of each word to lowercase at the same time,
     * use {@link #capitalizeFully(String, char[])}.</p>
     *
     * <p>The delimiters represent a set of characters understood to separate words.
     * The first string character and the first non-delimiter character after a
     * delimiter will be capitalized. </p>
     *
     * <p>A <code>null</code> input String returns <code>null</code>.
     * Capitalization uses the unicode title case, normally equivalent to
     * upper case.</p>
     *
     * <pre>
     * WordUtils.capitalize(null, *)            = null
     * WordUtils.capitalize("", *)              = ""
     * WordUtils.capitalize(*, new char[0])     = *
     * WordUtils.capitalize("i am fine", null)  = "I Am Fine"
     * WordUtils.capitalize("i aM.fine", {'.'}) = "I aM.Fine"
     * </pre>
     *
     * @param str  the String to capitalize, may be null
     * @param delimiters  set of characters to determine capitalization, null means whitespace
     * @return capitalized String, <code>null</code> if null String input
     * @see #uncapitalize(String)
     * @see #capitalizeFully(String)
     * @since 2.1
     */
    public static String capitalize(String str, char[] delimiters) {
        int delimLen = (delimiters == null ? -1 : delimiters.length);
        if (str == null || str.length() == 0 || delimLen == 0) {
            return str;
        }
        int strLen = str.length();
        StringBuffer buffer = new StringBuffer(strLen);
        boolean capitalizeNext = true;
        for (int i = 0; i < strLen; i++) {
            char ch = str.charAt(i);

            if (isDelimiter(ch, delimiters)) {
                buffer.append(ch);
                capitalizeNext = true;
            } else if (capitalizeNext) {
                buffer.append(Character.toTitleCase(ch));
                capitalizeNext = false;
            } else {
                buffer.append(ch);
            }
        }
        return buffer.toString();
    }

    //-----------------------------------------------------------------------
    /**
     * <p>Converts all the whitespace separated words in a String into capitalized words,
     * that is each word is made up of a titlecase character and then a series of
     * lowercase characters.  </p>
     *
     * <p>Whitespace is defined by {@link Character#isWhitespace(char)}.
     * A <code>null</code> input String returns <code>null</code>.
     * Capitalization uses the unicode title case, normally equivalent to
     * upper case.</p>
     *
     * <pre>
     * WordUtils.capitalizeFully(null)        = null
     * WordUtils.capitalizeFully("")          = ""
     * WordUtils.capitalizeFully("i am FINE") = "I Am Fine"
     * </pre>
     *
     * @param str  the String to capitalize, may be null
     * @return capitalized String, <code>null</code> if null String input
     */
    public static String capitalizeFully(String str) {
        return capitalizeFully(str, null);
    }

    /**
     * <p>Converts all the delimiter separated words in a String into capitalized words,
     * that is each word is made up of a titlecase character and then a series of
     * lowercase characters. </p>
     *
     * <p>The delimiters represent a set of characters understood to separate words.
     * The first string character and the first non-delimiter character after a
     * delimiter will be capitalized. </p>
     *
     * <p>A <code>null</code> input String returns <code>null</code>.
     * Capitalization uses the unicode title case, normally equivalent to
     * upper case.</p>
     *
     * <pre>
     * WordUtils.capitalizeFully(null, *)            = null
     * WordUtils.capitalizeFully("", *)              = ""
     * WordUtils.capitalizeFully(*, null)            = *
     * WordUtils.capitalizeFully(*, new char[0])     = *
     * WordUtils.capitalizeFully("i aM.fine", {'.'}) = "I am.Fine"
     * </pre>
     *
     * @param str  the String to capitalize, may be null
     * @param delimiters  set of characters to determine capitalization, null means whitespace
     * @return capitalized String, <code>null</code> if null String input
     * @since 2.1
     */
    public static String capitalizeFully(String str, char[] delimiters) {
        int delimLen = (delimiters == null ? -1 : delimiters.length);
        if (str == null || str.length() == 0 || delimLen == 0) {
            return str;
        }
        str = str.toLowerCase(Locale.getDefault());
        return capitalize(str, delimiters);
    }

    //-----------------------------------------------------------------------
    /**
     * Is the character a delimiter.
     *
     * @param ch  the character to check
     * @param delimiters  the delimiters
     * @return true if it is a delimiter
     */
    private static boolean isDelimiter(char ch, char[] delimiters) {
        if (delimiters == null) {
            return Character.isWhitespace(ch);
        }
        for (int i = 0, isize = delimiters.length; i < isize; i++) {
            if (ch == delimiters[i]) {
                return true;
            }
        }
        return false;
    }
}