package com.ejgv.gvi.utils;

import android.content.Context;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.content.res.Configuration;
import android.os.Build;
import android.support.v7.app.AlertDialog;

import com.ejgv.gvi.AccessData;
import com.ejgv.gvi.Constants;
import com.ejgv.gvi.R;
import com.ejgv.gvi.restapipns.dto.DTOText;
import com.ejgv.gvi.restapitrace.service.AddTraceParams;
import com.ejgv.gvi.restapitrace.service.MezuTraceService;

import java.util.Date;

/**
 * Created by Batura Mobile Solutions (Koldo Calvo).
 */
public class UtilsApp
{
    /**
     * Get current locale
     * @return
     */
    public static String getLocale(Context context)
    {
        Configuration conf = context.getResources().getConfiguration();
        if( conf.locale.toString().equals("eu_ES"))
            return "eu";
        else
            return "es";
    }

    /**
     * Get internationalized string from DTOText instance
     * @param dtoText
     * @return
     */
    public static String getTextLocale(Context context, DTOText dtoText)
    {
        if (dtoText == null)
            return null;
        Configuration conf = context.getResources().getConfiguration();
        if( conf.locale.toString().equals("eu_ES"))
            return dtoText.getEu();
        else
            return dtoText.getEs();
    }

    /**
     * Show basic alert dialog with title and message customization and standard ok button
     * @param title
     * @param message
     */
    public static void showAlertDialogBasic(Context context, String title, String message)
    {
        AlertDialog alertDialog = new AlertDialog.Builder(context)
                .setTitle(title)
                .setMessage(message)
                .setPositiveButton(R.string.action_accept, null)
                .create();
        alertDialog.show();
    }

    /**
     * Show basic alert dialog with title and message customization and standard ok button
     * @param title
     * @param description
     */
    public static void showAlertDialogBasic(Context context, String title, DTOText description)
    {
        AlertDialog alertDialog = new AlertDialog.Builder(context)
                .setTitle(title)
                .setMessage(getTextLocale(context, description))
                .setPositiveButton(R.string.action_accept, null)
                .create();
        alertDialog.show();
    }

    public static void sendTraceToServer(AddTraceParams params)
    {
        MezuTraceService mezuTraceService = new MezuTraceService();
        mezuTraceService.createAddTraceCall(params).start();
    }

    public static AddTraceParams createStandardAddTraceParams(String className, String methodName, Exception e)
    {
        AddTraceParams addTraceParams = new AddTraceParams();
        addTraceParams.setToken(Constants.APPLICATION_TRACE_PARAM_TOKEN);
        addTraceParams.setType(Constants.APPLICATION_TRACE_PARAM_TYPE);
        addTraceParams.setService(Constants.APPLICATION_TRACE_PARAM_SERVICE);
        addTraceParams.setApp(Constants.APPLICATION_TRACE_PARAM_APP);
        addTraceParams.setClicontext(AccessData.UUID_APP);
        addTraceParams.setEvent(Constants.APPLICATION_TRACE_PARAM_ERROR);
        addTraceParams.setTsclient((new Date()).getTime());
        addTraceParams.setMessage("pruebas");
        addTraceParams.setStacktrace(Utils.getStackTraceString(e));
        addTraceParams.setMethodName(methodName);
        addTraceParams.setClassName(className);
        return addTraceParams;
    }

    public static AddTraceParams createHttpClientAddTraceParams(String className, String methodName, String message)
    {
        AddTraceParams addTraceParams = new AddTraceParams();
        addTraceParams.setToken(Constants.APPLICATION_TRACE_PARAM_TOKEN);
        addTraceParams.setType(Constants.APPLICATION_TRACE_PARAM_TYPE);
        addTraceParams.setService(Constants.APPLICATION_TRACE_PARAM_SERVICE);
        addTraceParams.setApp(Constants.APPLICATION_TRACE_PARAM_APP);
        addTraceParams.setClicontext(AccessData.UUID_APP);
        addTraceParams.setEvent(Constants.APPLICATION_TRACE_PARAM_ERROR);
        addTraceParams.setTsclient((new Date()).getTime());
        addTraceParams.setMessage(message);
        addTraceParams.setMethodName(methodName);
        addTraceParams.setClassName(className);
        return addTraceParams;
    }

    public static String createResponseErrorMessage(String methodName, int statuscode, String message) throws Exception
    {
        return new StringBuilder()
                .append("Response error in ").append(methodName)
                .append("\nStatusCode: ").append(statuscode)
                .append(",\nMessage: ").append(message)
                .toString();
    }

    public static String getChnProvider()
    {
        return Constants.APPLICATION_LIMSP_INF;
    }

    public static String getAppVersion(Context context)
    {
        String strAppVersion = "unknown";
        try {
            PackageInfo info = context.getPackageManager().getPackageInfo(
                    context.getPackageName(), 0);
            strAppVersion = info.versionName;
        } catch (PackageManager.NameNotFoundException e) {
            e.printStackTrace();
        }
        return strAppVersion;
    }

    public static String getDevVendor()
    {
        return android.os.Build.MANUFACTURER;
    }

    public static String getDevModel()
    {
        return android.os.Build.MODEL;
    }

    public static String getOsVendor()
    {
        return Constants.APPLICATION_OS_VENDOR;
    }

    public static String getOsVersion()
    {
        return Build.VERSION.RELEASE;
    }

    /**
     * Limit the string to a certain number of characters, adding "..." if it was truncated
     *
     * @param value
     *        The string to limit.
     * @param length
     *        the length to limit to (as an int).
     * @return The limited string.
     */
    public static String limit(String value, int length)
    {
        StringBuilder buf = new StringBuilder(value);
        if (buf.length() > length)
        {
            buf.setLength(length);
            buf.append("...");
        }

        return buf.toString();
    }
}
